@@ -8,8 +8,17 @@ import unittest
 import os
 from stat import *
 
+import sys
+
+srcdir = os.getenv('srcdir')
+if srcdir:
+    sys.path.append("./src/config")
+    srcdir = srcdir + "/src/config"
+else:
+    srcdir = "."
 import SSSDConfig
 
+
 class SSSDConfigTestValid(unittest.TestCase):
     def setUp(self):
         pass
@@ -75,6 +84,9 @@ class SSSDConfigTestValid(unittest.TestCase):
         self.assertTrue('full_name_format' in new_options)
         self.assertEquals(new_options['full_name_format'][0], str)
 
+        self.assertTrue('default_domain_suffix' in new_options)
+        self.assertEquals(new_options['default_domain_suffix'][0], str)
+
         del sssdconfig
 
     def testDomains(self):
@@ -263,16 +275,19 @@ class SSSDConfigTestSSSDService(unittest.TestCase):
             'services',
             'domains',
             'timeout',
+            'force_timeout',
             'sbus_timeout',
             're_expression',
             'full_name_format',
             'krb5_rcache_dir',
+            'default_domain_suffix',
             'debug_level',
             'debug_timestamps',
             'debug_microseconds',
             'debug_to_files',
             'command',
             'reconnection_retries',
+            'fd_limit',
             'client_idle_timeout']
 
         self.assertTrue(type(options) == dict,
@@ -485,6 +500,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'entry_cache_netgroup_timeout',
             'entry_cache_service_timeout',
             'entry_cache_autofs_timeout',
+            'entry_cache_sudo_timeout',
             'lookup_family_order',
             'account_cache_expiration',
             'dns_resolver_timeout',
@@ -492,6 +508,9 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'override_gid',
             'case_sensitive',
             'override_homedir',
+            'fallback_homedir',
+            'override_shell',
+            'pwd_expiration_warning',
             'id_provider',
             'auth_provider',
             'access_provider',
@@ -499,7 +518,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'sudo_provider',
             'autofs_provider',
             'session_provider',
-            'hostid_provider']
+            'hostid_provider',
+            'subdomains_provider']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -565,8 +585,10 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         backup_list = control_list[:]
         control_list.extend(
             ['krb5_server',
+             'krb5_backup_server',
              'krb5_realm',
              'krb5_kpasswd',
+             'krb5_backup_kpasswd',
              'krb5_ccachedir',
              'krb5_ccname_template',
              'krb5_keytab',
@@ -689,7 +711,9 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         domain = SSSDConfig.SSSDDomain('sssd', self.schema)
 
         control_provider_dict = {
-            'ipa': ['id', 'auth', 'access', 'chpass', 'autofs' ],
+            'ipa': ['id', 'auth', 'access', 'chpass', 'autofs', 'session',
+                    'hostid', 'subdomains'],
+            'ad': ['id', 'auth', 'access', 'chpass'],
             'local': ['id', 'auth', 'chpass'],
             'ldap': ['id', 'auth', 'access', 'chpass', 'sudo', 'autofs'],
             'krb5': ['auth', 'access', 'chpass'],
@@ -718,9 +742,11 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         options = domain.list_provider_options('krb5', 'auth')
         control_list = [
             'krb5_server',
+            'krb5_backup_server',
             'krb5_kdcip',
             'krb5_realm',
             'krb5_kpasswd',
+            'krb5_backup_kpasswd',
             'krb5_ccachedir',
             'krb5_ccname_template',
             'krb5_keytab',
@@ -812,6 +838,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'entry_cache_netgroup_timeout',
             'entry_cache_service_timeout',
             'entry_cache_autofs_timeout',
+            'entry_cache_sudo_timeout',
             'account_cache_expiration',
             'lookup_family_order',
             'dns_resolver_timeout',
@@ -819,6 +846,9 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'override_gid',
             'case_sensitive',
             'override_homedir',
+            'fallback_homedir',
+            'override_shell',
+            'pwd_expiration_warning',
             'id_provider',
             'auth_provider',
             'access_provider',
@@ -826,7 +856,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'sudo_provider',
             'autofs_provider',
             'session_provider',
-            'hostid_provider']
+            'hostid_provider',
+            'subdomains_provider']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -892,9 +923,11 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         backup_list = control_list[:]
         control_list.extend(
             ['krb5_server',
+             'krb5_backup_server',
              'krb5_kdcip',
              'krb5_realm',
              'krb5_kpasswd',
+             'krb5_backup_kpasswd',
              'krb5_ccachedir',
              'krb5_ccname_template',
              'krb5_keytab',
@@ -928,6 +961,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         # Remove the local ID provider and add an LDAP one
         # LDAP ID providers can also use the krb5_realm
         domain.remove_provider('id')
+        self.assertFalse(domain.options.has_key('id_provider'))
 
         domain.add_provider('ldap', 'id')
 
@@ -952,6 +986,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         # Remove the auth domain and verify that the options
         # revert to the backup_list
         domain.remove_provider('auth')
+        self.assertFalse(domain.options.has_key('auth_provider'))
         options = domain.list_options()
 
         self.assertTrue(type(options) == dict,
@@ -974,14 +1009,17 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
 
         # Test removing nonexistent provider - Real
         domain.remove_provider('id')
+        self.assertFalse(domain.options.has_key('id_provider'))
 
         # Test removing nonexistent provider - Bad backend type
         # Should pass without complaint
         domain.remove_provider('id')
+        self.assertFalse(domain.options.has_key('id_provider'))
 
         # Test removing nonexistent provider - Bad provider type
         # Should pass without complaint
         domain.remove_provider('nosuchprovider')
+        self.assertFalse(domain.options.has_key('nosuchprovider_provider'))
 
     def testGetOption(self):
         domain = SSSDConfig.SSSDDomain('sssd', self.schema)
@@ -1158,7 +1196,8 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
             'pam',
             'sudo',
             'autofs',
-            'ssh']
+            'ssh',
+            'pac']
         for section in control_list:
             self.assertTrue(sssdconfig.has_section(section),
                             "Section [%s] missing" %
@@ -1250,7 +1289,8 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
             'nss',
             'sudo',
             'autofs',
-            'ssh']
+            'ssh',
+            'pac']
         service_list = sssdconfig.list_services()
         for service in control_list:
             self.assertTrue(service in service_list,
@@ -1770,14 +1810,6 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
 if __name__ == "__main__":
     error = 0
 
-    import os
-    import sys
-    srcdir = os.getenv('srcdir')
-    if srcdir:
-        srcdir = srcdir + "/src/config"
-    else:
-        srcdir = "."
-
     suite = unittest.TestLoader().loadTestsFromTestCase(SSSDConfigTestSSSDService)
     res = unittest.TextTestRunner().run(suite)
     if not res.wasSuccessful():
