@@ -36,17 +36,25 @@
 #define SYSDB_GROUPS_CONTAINER "cn=groups"
 #define SYSDB_CUSTOM_CONTAINER "cn=custom"
 #define SYSDB_NETGROUP_CONTAINER "cn=Netgroups"
+#define SYSDB_RANGE_CONTAINER "cn=ranges"
 #define SYSDB_TMPL_USER_BASE SYSDB_USERS_CONTAINER",cn=%s,"SYSDB_BASE
 #define SYSDB_TMPL_GROUP_BASE SYSDB_GROUPS_CONTAINER",cn=%s,"SYSDB_BASE
 #define SYSDB_TMPL_CUSTOM_BASE SYSDB_CUSTOM_CONTAINER",cn=%s,"SYSDB_BASE
 #define SYSDB_TMPL_NETGROUP_BASE SYSDB_NETGROUP_CONTAINER",cn=%s,"SYSDB_BASE
+#define SYSDB_TMPL_RANGE_BASE SYSDB_RANGE_CONTAINER",cn=%s,"SYSDB_BASE
 
+#define SYSDB_SUBDOMAIN_CLASS "subdomain"
+#define SYSDB_RANGE_CLASS "idrange"
 #define SYSDB_USER_CLASS "user"
 #define SYSDB_GROUP_CLASS "group"
 #define SYSDB_NETGROUP_CLASS "netgroup"
 #define SYSDB_HOST_CLASS "host"
+#define SYSDB_HOSTGROUP_CLASS "hostgroup"
 #define SYSDB_SELINUX_USERMAP_CLASS "selinuxusermap"
 #define SYSDB_SELINUX_CLASS "selinux"
+#define SYSDB_ID_RANGE_CLASS "idRange"
+#define SYSDB_DOMAIN_ID_RANGE_CLASS "domainIDRange"
+#define SYSDB_TRUSTED_AD_DOMAIN_RANGE_CLASS "TrustedADDomainRange"
 
 #define SYSDB_NAME "name"
 #define SYSDB_NAME_ALIAS "nameAlias"
@@ -66,6 +74,7 @@
 
 #define SYSDB_MEMBER "member"
 #define SYSDB_MEMBERUID "memberUid"
+#define SYSDB_GHOST "ghost"
 #define SYSDB_POSIX "isPosix"
 #define SYSDB_USER_CATEGORY "userCategory"
 #define SYSDB_HOST_CATEGORY "hostCategory"
@@ -95,13 +104,20 @@
 #define SYSDB_NETGROUP_MEMBER "memberNisNetgroup"
 #define SYSDB_DESCRIPTION   "description"
 
+#define SYSDB_FQDN "fqdn"
+#define SYSDB_SERVERHOSTNAME "serverHostname"
+
 #define SYSDB_SELINUX_SEEALSO "seeAlso"
 #define SYSDB_SELINUX_USER "selinuxUser"
 #define SYSDB_SELINUX_ENABLED "enabled"
+#define SYSDB_SELINUX_HOST_PRIORITY "hostPriority"
 
 #define SYSDB_CACHEDPWD "cachedPassword"
 
 #define SYSDB_UUID "uniqueID"
+#define SYSDB_SID "objectSID"
+#define SYSDB_PRIMARY_GROUP "ADPrimaryGroupID"
+#define SYSDB_SID_STR "objectSIDString"
 #define SYSDB_UPN "userPrincipalName"
 #define SYSDB_CCACHE_FILE "ccacheFile"
 
@@ -116,6 +132,16 @@
 
 #define SYSDB_SSH_PUBKEY "sshPublicKey"
 
+#define SYSDB_SUBDOMAIN_REALM "realmName"
+#define SYSDB_SUBDOMAIN_FLAT "flatName"
+#define SYSDB_SUBDOMAIN_ID "domainID"
+
+#define SYSDB_BASE_ID "baseID"
+#define SYSDB_ID_RANGE_SIZE "idRangeSize"
+#define SYSDB_BASE_RID "baseRID"
+#define SYSDB_SECONDARY_BASE_RID "secondaryBaseRID"
+#define SYSDB_DOMAIN_ID "domainID"
+
 #define SYSDB_NEXTID_FILTER "("SYSDB_NEXTID"=*)"
 
 #define SYSDB_UC "objectclass="SYSDB_USER_CLASS
@@ -156,6 +182,7 @@
                         NULL}
 #define SYSDB_GRSRC_ATTRS {SYSDB_NAME, SYSDB_GIDNUM, \
                            SYSDB_MEMBERUID, \
+                           SYSDB_GHOST, \
                            SYSDB_DEFAULT_ATTRS, \
                            NULL}
 #define SYSDB_GRPW_ATTRS {SYSDB_NAME, SYSDB_UIDNUM, \
@@ -180,6 +207,7 @@
 #define SYSDB_TMPL_NETGROUP SYSDB_NAME"=%s,"SYSDB_TMPL_NETGROUP_BASE
 #define SYSDB_TMPL_CUSTOM_SUBTREE "cn=%s,"SYSDB_TMPL_CUSTOM_BASE
 #define SYSDB_TMPL_CUSTOM SYSDB_NAME"=%s,cn=%s,"SYSDB_TMPL_CUSTOM_BASE
+#define SYSDB_TMPL_RANGE SYSDB_NAME"=%s,"SYSDB_TMPL_RANGE_BASE
 
 #define SYSDB_MOD_ADD LDB_FLAG_MOD_ADD
 #define SYSDB_MOD_DEL LDB_FLAG_MOD_DELETE
@@ -203,6 +231,23 @@ struct sysdb_attrs {
 /* sysdb_attrs helper functions */
 struct sysdb_attrs *sysdb_new_attrs(TALLOC_CTX *mem_ctx);
 
+struct sysdb_subdom {
+    const char *realm;
+    const char *name;
+    const char *flat_name;
+    const char *id;
+};
+
+struct range_info {
+    char *name;
+    uint32_t base_id;
+    uint32_t id_range_size;
+    uint32_t base_rid;
+    uint32_t secondary_base_rid;
+    char *trusted_dom_sid;
+};
+
+
 /* values are copied in the structure, allocated on "attrs" */
 int sysdb_attrs_add_val(struct sysdb_attrs *attrs,
                         const char *name, const struct ldb_val *val);
@@ -216,6 +261,9 @@ int sysdb_attrs_add_uint32(struct sysdb_attrs *attrs,
                            const char *name, uint32_t value);
 int sysdb_attrs_add_time_t(struct sysdb_attrs *attrs,
                            const char *name, time_t value);
+int sysdb_attrs_copy_values(struct sysdb_attrs *src,
+                            struct sysdb_attrs *dst,
+                            const char *name);
 int sysdb_attrs_get_el(struct sysdb_attrs *attrs, const char *name,
                        struct ldb_message_element **el);
 int sysdb_attrs_get_el_ext(struct sysdb_attrs *attrs, const char *name,
@@ -240,11 +288,6 @@ int sysdb_attrs_users_from_str_list(struct sysdb_attrs *attrs,
                                     const char *attr_name,
                                     const char *domain,
                                     const char *const *list);
-int sysdb_attrs_users_from_ldb_vals(struct sysdb_attrs *attrs,
-                                    const char *attr_name,
-                                    const char *domain,
-                                    struct ldb_val *values,
-                                    int num_values);
 errno_t sysdb_attrs_primary_name(struct sysdb_ctx *sysdb,
                                  struct sysdb_attrs *attrs,
                                  const char *ldap_attr,
@@ -273,25 +316,22 @@ errno_t sysdb_msg2attrs(TALLOC_CTX *mem_ctx, size_t count,
 int sysdb_error_to_errno(int ldberr);
 
 /* DNs related helper functions */
-errno_t sysdb_get_rdn(struct sysdb_ctx *sysdb, void *mem_ctx,
+errno_t sysdb_get_rdn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
                       const char *_dn, char **_name, char **_val);
-struct ldb_dn *sysdb_user_dn(struct sysdb_ctx *sysdb, void *mem_ctx,
-                             const char *domain, const char *name);
-struct ldb_dn *sysdb_group_dn(struct sysdb_ctx *sysdb, void *mem_ctx,
-                              const char *domain, const char *name);
-struct ldb_dn *sysdb_netgroup_dn(struct sysdb_ctx *sysdb, void *mem_ctx,
-                                 const char *domain, const char *name);
-struct ldb_dn *sysdb_netgroup_base_dn(struct sysdb_ctx *sysdb, void *mem_ctx,
-                                      const char *domain);
-errno_t sysdb_group_dn_name(struct sysdb_ctx *sysdb, void *mem_ctx,
+struct ldb_dn *sysdb_user_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
+                             const char *name);
+struct ldb_dn *sysdb_group_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
+                              const char *name);
+struct ldb_dn *sysdb_netgroup_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
+                                 const char *name);
+struct ldb_dn *sysdb_netgroup_base_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx);
+errno_t sysdb_group_dn_name(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
                             const char *dn_str, char **name);
-struct ldb_dn *sysdb_domain_dn(struct sysdb_ctx *sysdb, void *mem_ctx,
-                               const char *domain);
-struct ldb_dn *sysdb_custom_dn(struct sysdb_ctx *sysdb, void *mem_ctx,
-                                const char *domain, const char *object_name,
-                                const char *subtree_name);
-struct ldb_dn *sysdb_custom_subtree_dn(struct sysdb_ctx *sysdb, void *mem_ctx,
-                                       const char *domain,
+struct ldb_dn *sysdb_domain_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx);
+struct ldb_dn *sysdb_custom_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
+                               const char *object_name,
+                               const char *subtree_name);
+struct ldb_dn *sysdb_custom_subtree_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
                                        const char *subtree_name);
 
 char *sysdb_user_strdn(TALLOC_CTX *mem_ctx,
@@ -311,6 +351,82 @@ int sysdb_transaction_start(struct sysdb_ctx *sysdb);
 int sysdb_transaction_commit(struct sysdb_ctx *sysdb);
 int sysdb_transaction_cancel(struct sysdb_ctx *sysdb);
 
+/* functions related to subdomains */
+errno_t sysdb_get_subdomains(TALLOC_CTX *mem_ctx,
+                             struct sysdb_ctx *sysdb,
+                             size_t *subdomain_count,
+                             struct sysdb_subdom ***subdomain_list);
+
+errno_t sysdb_domain_create(struct sysdb_ctx *sysdb, const char *domain_name);
+
+errno_t sysdb_update_subdomains(struct sysdb_ctx *sysdb,
+                                int num_subdoms,
+                                struct sysdb_subdom *subdoms);
+
+errno_t sysdb_get_subdomain_context(TALLOC_CTX *mem_ctx,
+                                    struct sysdb_ctx *sysdb,
+                                    struct sss_domain_info *subdomain,
+                                    struct sysdb_ctx **subdomain_ctx);
+
+errno_t sysdb_master_domain_get_info(TALLOC_CTX *mem_ctx,
+                                     struct sysdb_ctx *sysdb,
+                                     struct sysdb_subdom **info);
+
+errno_t sysdb_master_domain_add_info(struct sysdb_ctx *sysdb,
+                                     struct sysdb_subdom *domain_info);
+
+
+errno_t sysdb_search_domuser_by_name(TALLOC_CTX *mem_ctx,
+                                     struct sss_domain_info *domain,
+                                     const char *name,
+                                     const char **attrs,
+                                     struct ldb_message **msg);
+errno_t sysdb_search_domuser_by_uid(TALLOC_CTX *mem_ctx,
+                                    struct sss_domain_info *domain,
+                                    uid_t uid,
+                                    const char **attrs,
+                                    struct ldb_message **msg);
+errno_t sysdb_store_domuser(struct sss_domain_info *domain,
+                            const char *name,
+                            const char *pwd,
+                            uid_t uid, gid_t gid,
+                            const char *gecos,
+                            const char *homedir,
+                            const char *shell,
+                            struct sysdb_attrs *attrs,
+                            char **remove_attrs,
+                            uint64_t cache_timeout,
+                            time_t now);
+errno_t sysdb_delete_domuser(struct sss_domain_info *domain,
+                             const char *name, uid_t uid);
+
+
+errno_t sysdb_search_domgroup_by_name(TALLOC_CTX *mem_ctx,
+                                      struct sss_domain_info *domain,
+                                      const char *name,
+                                      const char **attrs,
+                                      struct ldb_message **msg);
+errno_t sysdb_search_domgroup_by_gid(TALLOC_CTX *mem_ctx,
+                                     struct sss_domain_info *domain,
+                                     gid_t gid,
+                                     const char **attrs,
+                                     struct ldb_message **msg);
+errno_t sysdb_store_domgroup(struct sss_domain_info *domain,
+                             const char *name,
+                             gid_t gid,
+                             struct sysdb_attrs *attrs,
+                             uint64_t cache_timeout,
+                             time_t now);
+errno_t sysdb_delete_domgroup(struct sss_domain_info *domain,
+                              const char *name, gid_t gid);
+
+errno_t sysdb_get_ranges(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
+                             size_t *range_count,
+                             struct range_info ***range_list);
+errno_t sysdb_range_create(struct sysdb_ctx *sysdb, struct range_info *range);
+errno_t sysdb_update_ranges(struct sysdb_ctx *sysdb,
+                            struct range_info **ranges);
+
 /* Sysdb initialization.
  * call this function *only* once to initialize the database and get
  * the sysdb ctx */
@@ -326,15 +442,20 @@ int sysdb_domain_init(TALLOC_CTX *mem_ctx,
                       const char *db_path,
                       struct sysdb_ctx **_ctx);
 
+errno_t sysdb_init_domain_and_sysdb(TALLOC_CTX *mem_ctx,
+                                    struct confdb_ctx *cdb,
+                                    const char *domain_name,
+                                    const char *db_path,
+                                    struct sss_domain_info **_domain,
+                                    struct sysdb_ctx **_ctx);
+
 int sysdb_list_init(TALLOC_CTX *mem_ctx,
                     const char *path,
                     struct sysdb_ctx *sysdb,
                     struct sysdb_ctx_list **_list);
 
-int sysdb_get_ctx_from_list(struct sysdb_ctx_list *ctx_list,
-                            struct sss_domain_info *domain,
-                            struct sysdb_ctx **_ctx);
-
+errno_t sysdb_add_to_domain(struct sss_domain_info *domain,
+                            struct sysdb_ctx *ctx);
 /* functions to retrieve information from sysdb
  * These functions automatically starts an operation
  * therefore they cannot be called within a transaction */
@@ -504,11 +625,6 @@ int sysdb_add_user(struct sysdb_ctx *sysdb,
                    int cache_timeout,
                    time_t now);
 
-int sysdb_add_fake_user(struct sysdb_ctx *sysdb,
-                        const char *name,
-                        const char *original_dn,
-                        time_t now);
-
 /* Add group (only basic attrs and w/o checks) */
 int sysdb_add_basic_group(struct sysdb_ctx *sysdb,
                           const char *name, gid_t gid);
@@ -568,7 +684,6 @@ enum sysdb_member_type {
     SYSDB_MEMBER_GROUP,
     SYSDB_MEMBER_NETGROUP,
     SYSDB_MEMBER_SERVICE,
-    SYSDB_MEMBER_AUTOFSENTRY
 };
 
 int sysdb_add_group_member(struct sysdb_ctx *sysdb,
@@ -715,7 +830,7 @@ errno_t sysdb_netgr_to_entries(TALLOC_CTX *mem_ctx,
                                struct ldb_result *res,
                                struct sysdb_netgroup_ctx ***entries);
 
-errno_t sysdb_dn_sanitize(void *mem_ctx, const char *input,
+errno_t sysdb_dn_sanitize(TALLOC_CTX *mem_ctx, const char *input,
                           char **sanitized);
 
 errno_t sysdb_get_bool(struct sysdb_ctx *sysdb,
@@ -747,4 +862,35 @@ errno_t sysdb_get_direct_parents(TALLOC_CTX *mem_ctx,
                                  const char *name,
                                  char ***_direct_parents);
 
+/* === Functions related to ID-mapping === */
+
+#define SYSDB_IDMAP_CONTAINER "cn=id_mappings"
+
+#define SYSDB_IDMAP_SUBTREE "idmap"
+#define SYSDB_IDMAP_MAPPING_OC "id_mapping"
+#define SYSDB_IDMAP_FILTER "(objectClass="SYSDB_IDMAP_MAPPING_OC")"
+#define SYSDB_IDMAP_SID_ATTR "objectSID"
+#define SYSDB_IDMAP_SLICE_ATTR "slice"
+
+#define SYSDB_IDMAP_ATTRS { \
+    SYSDB_NAME, \
+    SYSDB_IDMAP_SID_ATTR, \
+    SYSDB_IDMAP_SLICE_ATTR, \
+    NULL }
+
+#define SYSDB_TMPL_IDMAP_BASE SYSDB_IDMAP_CONTAINER",cn=%s,"SYSDB_BASE
+#define SYSDB_TMPL_IDMAP SYSDB_IDMAP_SID_ATTR"=%s,"SYSDB_TMPL_IDMAP_BASE
+
+struct ldb_dn *sysdb_idmap_dn(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
+                              const char *object_sid);
+
+errno_t sysdb_idmap_store_mapping(struct sysdb_ctx *sysdb,
+                                  const char *dom_name,
+                                  const char *dom_sid,
+                                  id_t slice_num);
+
+errno_t sysdb_idmap_get_mappings(TALLOC_CTX *mem_ctx,
+                                 struct sysdb_ctx *sysdb,
+                                 struct ldb_result **_result);
+
 #endif /* __SYS_DB_H__ */
