@@ -24,38 +24,57 @@
 #include "db/sysdb_private.h"
 #include "db/sysdb_autofs.h"
 
-struct ldb_dn *
+#define SYSDB_TMPL_AUTOFS_ENTRY SYSDB_NAME"=%s,"SYSDB_TMPL_CUSTOM
+
+static struct ldb_dn *
 sysdb_autofsmap_dn(TALLOC_CTX *mem_ctx,
                    struct sysdb_ctx *sysdb,
                    const char *map_name)
 {
-    return sysdb_custom_dn(sysdb, mem_ctx, sysdb->domain->name,
-                           map_name, AUTOFS_MAP_SUBDIR);
+    return sysdb_custom_dn(sysdb, mem_ctx, map_name, AUTOFS_MAP_SUBDIR);
 }
 
-struct ldb_dn *
+static struct ldb_dn *
 sysdb_autofsentry_dn(TALLOC_CTX *mem_ctx,
                      struct sysdb_ctx *sysdb,
-                     const char *entry_name)
+                     const char *map_name,
+                     const char *entry_name,
+                     const char *entry_value)
 {
-    return sysdb_custom_dn(sysdb, mem_ctx, sysdb->domain->name,
-                           entry_name, AUTOFS_ENTRY_SUBDIR);
-}
+    errno_t ret;
+    TALLOC_CTX *tmp_ctx;
+    char *clean_name;
+    char *clean_value;
+    const char *rdn;
+    struct ldb_dn *dn = NULL;
 
-static char *
-sysdb_autofsmap_strdn(TALLOC_CTX *mem_ctx,
-                      struct sysdb_ctx *sysdb,
-                      const char *map_name)
-{
-    struct ldb_dn *dn;
-    char *strdn;
+    tmp_ctx = talloc_new(NULL);
+    if (!tmp_ctx) {
+        return NULL;
+    }
 
-    dn = sysdb_autofsmap_dn(mem_ctx, sysdb, map_name);
-    if (!dn) return NULL;
+    ret = sysdb_dn_sanitize(tmp_ctx, entry_name, &clean_name);
+    if (ret != EOK) {
+        goto done;
+    }
 
-    strdn = talloc_strdup(mem_ctx, ldb_dn_get_linearized(dn));
-    talloc_free(dn);
-    return strdn;
+    ret = sysdb_dn_sanitize(tmp_ctx, entry_value, &clean_value);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    rdn = talloc_asprintf(tmp_ctx, "%s%s", clean_name, clean_value);
+    if (!rdn) {
+        goto done;
+    }
+
+    dn = ldb_dn_new_fmt(mem_ctx, sysdb->ldb, SYSDB_TMPL_AUTOFS_ENTRY,
+                        rdn, map_name, AUTOFS_MAP_SUBDIR,
+                        sysdb->domain->name);
+
+done:
+    talloc_free(tmp_ctx);
+    return dn;
 }
 
 errno_t
@@ -208,12 +227,16 @@ done:
 
 errno_t
 sysdb_save_autofsentry(struct sysdb_ctx *sysdb_ctx,
+                       const char *map,
                        const char *key,
                        const char *value,
                        struct sysdb_attrs *attrs)
 {
     errno_t ret;
     TALLOC_CTX *tmp_ctx;
+    struct ldb_message *msg;
+    struct ldb_dn *dn;
+    const char *name;
 
     DEBUG(SSSDBG_TRACE_FUNC,
           ("Adding autofs entry [%s] - [%s]\n", key, value));
@@ -253,26 +276,61 @@ sysdb_save_autofsentry(struct sysdb_ctx *sysdb_ctx,
         goto done;
     }
 
-    ret = sysdb_attrs_add_string(attrs, SYSDB_NAME, key);
+    name = talloc_asprintf(tmp_ctx, "%s%s", key, value);
+    if (!name) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = sysdb_attrs_add_string(attrs, SYSDB_NAME, name);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, ("Could not set name attribute [%d]: %s\n",
               ret, strerror(ret)));
         goto done;
     }
 
-    ret = sysdb_store_custom(sysdb_ctx, key, AUTOFS_ENTRY_SUBDIR, attrs);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, ("sysdb_store_custom failed [%d]: %s\n",
-              ret, strerror(ret)));
+    dn = sysdb_autofsentry_dn(tmp_ctx, sysdb_ctx, map, key, value);
+    if (!dn) {
+        ret = ENOMEM;
         goto done;
     }
 
-    ret = EOK;
+    msg = ldb_msg_new(tmp_ctx);
+    if (!msg) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    msg->dn = dn;
+    msg->elements = attrs->a;
+    msg->num_elements = attrs->num;
+
+    ret = ldb_add(sysdb_ctx->ldb, msg);
+    ret = sysdb_error_to_errno(ret);
 done:
     talloc_free(tmp_ctx);
     return ret;
 }
 
+errno_t
+sysdb_del_autofsentry(struct sysdb_ctx *sysdb_ctx,
+                      const char *map,
+                      const char *key,
+                      const char *value)
+{
+    struct ldb_dn *dn;
+    errno_t ret;
+
+    dn = sysdb_autofsentry_dn(sysdb_ctx, sysdb_ctx, map, key, value);
+    if (!dn) {
+        return ENOMEM;
+    }
+
+    ret = sysdb_delete_entry(sysdb_ctx, dn, true);
+    talloc_free(dn);
+    return ret;
+}
+
 errno_t
 sysdb_autofs_entries_by_map(TALLOC_CTX *mem_ctx,
                             struct sysdb_ctx *sysdb,
@@ -288,7 +346,7 @@ sysdb_autofs_entries_by_map(TALLOC_CTX *mem_ctx,
                             NULL };
     size_t count;
     struct ldb_message **msgs;
-    char *mapdn;
+    struct ldb_dn *mapdn;
 
     DEBUG(SSSDBG_TRACE_FUNC, ("Getting entries for map %s\n", mapname));
 
@@ -297,21 +355,21 @@ sysdb_autofs_entries_by_map(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    mapdn = sysdb_autofsmap_strdn(tmp_ctx, sysdb, mapname);
+    mapdn = sysdb_autofsmap_dn(tmp_ctx, sysdb, mapname);
     if (!mapdn) {
         ret = ENOMEM;
         goto done;
     }
 
-    filter = talloc_asprintf(tmp_ctx, "(&(objectclass=%s)(%s=%s))",
-                             SYSDB_AUTOFS_ENTRY_OC, SYSDB_MEMBEROF, mapdn);
+    filter = talloc_asprintf(tmp_ctx, "(objectclass=%s)",
+                             SYSDB_AUTOFS_ENTRY_OC);
     if (!filter) {
         ret = ENOMEM;
         goto done;
     }
 
-    ret = sysdb_search_custom(tmp_ctx, sysdb, filter, AUTOFS_ENTRY_SUBDIR,
-                              attrs, &count, &msgs);
+    ret = sysdb_search_entry(tmp_ctx, sysdb, mapdn, LDB_SCOPE_ONELEVEL,
+                             filter, attrs, &count, &msgs);
     if (ret != EOK && ret != ENOENT) {
         DEBUG(SSSDBG_OP_FAILURE, ("sysdb search failed: %d\n", ret));
         goto done;
@@ -333,66 +391,29 @@ done:
 }
 
 errno_t
-sysdb_map_entry_name(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
-                     const char *entry_dn, char **_name)
-{
-    return sysdb_get_rdn(sysdb, mem_ctx, entry_dn, NULL, _name);
-}
-
-errno_t
-sysdb_autofs_map_update_members(struct sysdb_ctx *sysdb,
-                                const char *mapname,
-                                const char *const *add_entries,
-                                const char *const *del_entries)
+sysdb_set_autofsmap_attr(struct sysdb_ctx *sysdb,
+                         const char *name,
+                         struct sysdb_attrs *attrs,
+                         int mod_op)
 {
     errno_t ret;
-    int i;
+    struct ldb_dn *dn;
+    TALLOC_CTX *tmp_ctx;
 
-    TALLOC_CTX *tmp_ctx = talloc_new(NULL);
-    if(!tmp_ctx) {
+    tmp_ctx = talloc_new(NULL);
+    if (!tmp_ctx) {
         return ENOMEM;
     }
 
-    ret = sysdb_transaction_start(sysdb);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              ("Failed to start update transaction\n"));
+    dn = sysdb_autofsmap_dn(tmp_ctx, sysdb, name);
+    if (!dn) {
+        ret = ENOMEM;
         goto done;
     }
 
-    if (add_entries) {
-        /* Add the all te add_entries to the map */
-        for (i = 0; add_entries[i]; i++) {
-            ret = sysdb_add_group_member(sysdb, mapname, add_entries[i],
-                                         SYSDB_MEMBER_AUTOFSENTRY);
-            if (ret != EOK) {
-                DEBUG(SSSDBG_MINOR_FAILURE,
-                      ("Could not add entry [%s] to map [%s]. "
-                       "Skipping.\n", add_entries[i], mapname));
-                /* Continue on, we should try to finish the rest */
-            }
-        }
-    }
+    ret = sysdb_set_entry_attr(sysdb, dn, attrs, mod_op);
 
-    if (del_entries) {
-        /* Add the all te del_entries to the map */
-        for (i = 0; del_entries[i]; i++) {
-            ret = sysdb_remove_group_member(sysdb, mapname, del_entries[i],
-                                            SYSDB_MEMBER_AUTOFSENTRY);
-            if (ret != EOK) {
-                DEBUG(SSSDBG_MINOR_FAILURE,
-                      ("Could not del entry [%s] to map [%s]. "
-                       "Skipping.\n", del_entries[i], mapname));
-                /* Continue on, we should try to finish the rest */
-            }
-        }
-    }
-
-    ret = sysdb_transaction_commit(sysdb);
 done:
-    if (ret != EOK) {
-        sysdb_transaction_cancel(sysdb);
-    }
     talloc_free(tmp_ctx);
     return ret;
 }
