@@ -217,7 +217,7 @@ int sysdb_search_user_by_name(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    basedn = sysdb_user_dn(sysdb, tmp_ctx, sysdb->domain->name, name);
+    basedn = sysdb_user_dn(sysdb, tmp_ctx, name);
     if (!basedn) {
         ret = ENOMEM;
         goto done;
@@ -319,7 +319,7 @@ int sysdb_search_group_by_name(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    basedn = sysdb_group_dn(sysdb, tmp_ctx, sysdb->domain->name, name);
+    basedn = sysdb_group_dn(sysdb, tmp_ctx, name);
     if (!basedn) {
         ret = ENOMEM;
         goto done;
@@ -421,7 +421,7 @@ int sysdb_search_netgroup_by_name(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    basedn = sysdb_netgroup_dn(sysdb, tmp_ctx, sysdb->domain->name, name);
+    basedn = sysdb_netgroup_dn(sysdb, tmp_ctx, name);
     if (!basedn) {
         ret = ENOMEM;
         goto done;
@@ -455,6 +455,7 @@ int sysdb_set_entry_attr(struct sysdb_ctx *sysdb,
 {
     struct ldb_message *msg;
     int i, ret;
+    int lret;
     TALLOC_CTX *tmp_ctx;
 
     tmp_ctx = talloc_new(NULL);
@@ -488,8 +489,13 @@ int sysdb_set_entry_attr(struct sysdb_ctx *sysdb,
 
     msg->num_elements = attrs->num;
 
-    ret = ldb_modify(sysdb->ldb, msg);
-    ret = sysdb_error_to_errno(ret);
+    lret = ldb_modify(sysdb->ldb, msg);
+    if (lret != LDB_SUCCESS) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("ldb_modify failed: [%s]\n", ldb_strerror(lret)));
+    }
+
+    ret = sysdb_error_to_errno(lret);
 
 done:
     if (ret == ENOENT) {
@@ -519,7 +525,7 @@ int sysdb_set_user_attr(struct sysdb_ctx *sysdb,
         return ENOMEM;
     }
 
-    dn = sysdb_user_dn(sysdb, tmp_ctx, sysdb->domain->name, name);
+    dn = sysdb_user_dn(sysdb, tmp_ctx, name);
     if (!dn) {
         ret = ENOMEM;
         goto done;
@@ -554,7 +560,7 @@ int sysdb_set_group_attr(struct sysdb_ctx *sysdb,
         goto done;
     }
 
-    dn = sysdb_group_dn(sysdb, tmp_ctx, sysdb->domain->name, name);
+    dn = sysdb_group_dn(sysdb, tmp_ctx, name);
     if (!dn) {
         ret = ENOMEM;
         goto done;
@@ -587,7 +593,7 @@ int sysdb_set_netgroup_attr(struct sysdb_ctx *sysdb,
         return ENOMEM;
     }
 
-    dn = sysdb_netgroup_dn(sysdb, tmp_ctx, sysdb->domain->name, name);
+    dn = sysdb_netgroup_dn(sysdb, tmp_ctx, name);
     if (!dn) {
         ret = ENOMEM;
         goto done;
@@ -625,7 +631,7 @@ int sysdb_get_new_id(struct sysdb_ctx *sysdb,
         return ENOMEM;
     }
 
-    base_dn = sysdb_domain_dn(sysdb, tmp_ctx, domain->name);
+    base_dn = sysdb_domain_dn(sysdb, tmp_ctx);
     if (!base_dn) {
         talloc_zfree(tmp_ctx);
         return ENOMEM;
@@ -791,7 +797,7 @@ int sysdb_add_basic_user(struct sysdb_ctx *sysdb,
     }
 
     /* user dn */
-    msg->dn = sysdb_user_dn(sysdb, msg, sysdb->domain->name, name);
+    msg->dn = sysdb_user_dn(sysdb, msg, name);
     if (!msg->dn) {
         ERROR_OUT(ret, ENOMEM, done);
     }
@@ -860,9 +866,16 @@ int sysdb_add_user(struct sysdb_ctx *sysdb,
 {
     TALLOC_CTX *tmp_ctx;
     struct ldb_message *msg;
+    struct ldb_message **groups;
+    struct ldb_message_element *alias_el;
+    size_t group_count = 0;
     struct sysdb_attrs *id_attrs;
+    const char *group_attrs[] = {SYSDB_NAME, SYSDB_GHOST, NULL};
+    struct ldb_dn *tmpdn;
+    const char *userdn;
+    char *filter;
     uint32_t id;
-    int ret;
+    int ret, i, j;
 
     struct sss_domain_info *domain = sysdb->domain;
 
@@ -970,6 +983,106 @@ int sysdb_add_user(struct sysdb_ctx *sysdb,
     if (ret) goto done;
 
     ret = sysdb_set_user_attr(sysdb, name, attrs, SYSDB_MOD_REP);
+    if (ret) goto done;
+
+    /* remove all ghost users */
+    filter = talloc_asprintf(tmp_ctx, "(|(%s=%s)", SYSDB_GHOST, name);
+    if (!filter) {
+        ret = ENOMEM;
+        goto done;
+    }
+    ret = sysdb_attrs_get_el(attrs, SYSDB_NAME_ALIAS, &alias_el);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    for (i = 0; i < alias_el->num_values; i++) {
+        filter = talloc_asprintf_append(filter, "(%s=%s)",
+                                        SYSDB_GHOST, alias_el->values[i].data);
+        if (filter == NULL) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    filter = talloc_asprintf_append(filter, ")");
+    if (filter == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    tmpdn = sysdb_user_dn(sysdb, tmp_ctx, name);
+    if (!tmpdn) {
+        ERROR_OUT(ret, ENOMEM, done);
+    }
+
+    userdn = ldb_dn_get_linearized(tmpdn);
+    if (!userdn) {
+        ERROR_OUT(ret, EINVAL, done);
+    }
+
+    tmpdn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb,
+                            SYSDB_TMPL_GROUP_BASE, sysdb->domain->name);
+    if (!tmpdn) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    /* We need to find all groups that contain this object as a ghost user
+     * and replace the ghost user there by actual member record
+     * Note that this object can be referred to either by its name or any
+     * of its aliases
+     */
+    ret = sysdb_search_entry(tmp_ctx, sysdb, tmpdn, LDB_SCOPE_SUBTREE, filter,
+                             group_attrs, &group_count, &groups);
+    if (ret != EOK && ret != ENOENT) {
+        goto done;
+    }
+
+    for (i = 0; i < group_count; i++) {
+        msg = ldb_msg_new(tmp_ctx);
+        if (!msg) {
+            ERROR_OUT(ret, ENOMEM, done);
+        }
+
+        msg->dn = groups[i]->dn;
+        ret = ldb_msg_add_empty(msg, SYSDB_MEMBER, SYSDB_MOD_ADD, NULL);
+        if (ret != LDB_SUCCESS) {
+            ERROR_OUT(ret, ENOMEM, done);
+        }
+
+        ret = ldb_msg_add_string(msg, SYSDB_MEMBER, userdn);
+        if (ret != LDB_SUCCESS) {
+            ERROR_OUT(ret, EINVAL, done);
+        }
+
+        ret = ldb_msg_add_empty(msg, SYSDB_GHOST, SYSDB_MOD_DEL, NULL);
+        if (ret != LDB_SUCCESS) {
+            ERROR_OUT(ret, ENOMEM, done);
+        }
+        ret = ldb_msg_add_string(msg, SYSDB_GHOST, name);
+        if (ret != LDB_SUCCESS) {
+            ERROR_OUT(ret, EINVAL, done);
+        }
+        /* Delete aliases from the ghost attribute as well */
+        for (j = 0; j < alias_el->num_values; j++) {
+            ret = ldb_msg_add_string(msg, SYSDB_GHOST,
+                                     (char *) alias_el->values[j].data);
+            if (ret != LDB_SUCCESS) {
+                ERROR_OUT(ret, EINVAL, done);
+            }
+        }
+
+        ret = ldb_modify(sysdb->ldb, msg);
+        ret = sysdb_error_to_errno(ret);
+        if (ret != EOK) {
+            goto done;
+        }
+
+        talloc_zfree(msg);
+    }
+
+    ret = EOK;
 
 done:
     if (ret == EOK) {
@@ -983,76 +1096,6 @@ done:
     return ret;
 }
 
-int sysdb_add_fake_user(struct sysdb_ctx *sysdb,
-                        const char *name,
-                        const char *original_dn,
-                        time_t now)
-{
-    TALLOC_CTX *tmp_ctx;
-    struct ldb_message *msg;
-    int ret;
-
-    tmp_ctx = talloc_new(NULL);
-    if (!tmp_ctx) {
-        return ENOMEM;
-    }
-
-    msg = ldb_msg_new(tmp_ctx);
-    if (!msg) {
-        ERROR_OUT(ret, ENOMEM, done);
-    }
-
-    /* user dn */
-    msg->dn = sysdb_user_dn(sysdb, msg, sysdb->domain->name, name);
-    if (!msg->dn) {
-        ERROR_OUT(ret, ENOMEM, done);
-    }
-
-    ret = add_string(msg, LDB_FLAG_MOD_ADD, SYSDB_OBJECTCLASS, SYSDB_USER_CLASS);
-    if (ret) goto done;
-
-    ret = add_string(msg, LDB_FLAG_MOD_ADD, SYSDB_NAME, name);
-    if (ret) goto done;
-
-    if (!now) {
-        now = time(NULL);
-    }
-
-    ret = add_ulong(msg, LDB_FLAG_MOD_ADD, SYSDB_CREATE_TIME,
-                    (unsigned long) now);
-    if (ret) goto done;
-
-    /* set last login so that the fake entry does not get cleaned up
-     * immediately */
-    ret = add_ulong(msg, LDB_FLAG_MOD_ADD, SYSDB_LAST_LOGIN,
-                    (unsigned long) now);
-    if (ret) return ret;
-
-    ret = add_ulong(msg, LDB_FLAG_MOD_ADD, SYSDB_LAST_UPDATE,
-                    (unsigned long) now);
-    if (ret) goto done;
-
-    ret = add_ulong(msg, LDB_FLAG_MOD_ADD, SYSDB_CACHE_EXPIRE,
-                    (unsigned long) now-1);
-    if (ret) goto done;
-
-    if (original_dn) {
-        ret = add_string(msg, LDB_FLAG_MOD_ADD,
-                         SYSDB_ORIG_DN, original_dn);
-        if (ret) goto done;
-    }
-
-    ret = ldb_add(sysdb->ldb, msg);
-    ret = sysdb_error_to_errno(ret);
-
-done:
-    if (ret != EOK) {
-        DEBUG(6, ("Error: %d (%s)\n", ret, strerror(ret)));
-    }
-    talloc_zfree(tmp_ctx);
-    return ret;
-}
-
 /* =Add-Basic-Group-NO-CHECKS============================================= */
 
 int sysdb_add_basic_group(struct sysdb_ctx *sysdb,
@@ -1074,7 +1117,7 @@ int sysdb_add_basic_group(struct sysdb_ctx *sysdb,
     }
 
     /* group dn */
-    msg->dn = sysdb_group_dn(sysdb, msg, sysdb->domain->name, name);
+    msg->dn = sysdb_group_dn(sysdb, msg, name);
     if (!msg->dn) {
         ERROR_OUT(ret, ENOMEM, done);
     }
@@ -1302,7 +1345,7 @@ int sysdb_mod_group_member(struct sysdb_ctx *sysdb,
         ERROR_OUT(ret, EINVAL, fail);
     }
 
-    ret = ldb_msg_add_fmt(msg, SYSDB_MEMBER, "%s", dn);
+    ret = ldb_msg_add_string(msg, SYSDB_MEMBER, dn);
     if (ret != LDB_SUCCESS) {
         ERROR_OUT(ret, EINVAL, fail);
     }
@@ -1332,7 +1375,7 @@ int sysdb_add_basic_netgroup(struct sysdb_ctx *sysdb,
     }
 
     /* netgroup dn */
-    msg->dn = sysdb_netgroup_dn(sysdb, msg, sysdb->domain->name, name);
+    msg->dn = sysdb_netgroup_dn(sysdb, msg, name);
     if (!msg->dn) {
         ERROR_OUT(ret, ENOMEM, done);
     }
@@ -1473,24 +1516,27 @@ int sysdb_store_user(struct sysdb_ctx *sysdb,
         attrs = sysdb_new_attrs(tmp_ctx);
         if (!attrs) {
             ret = ENOMEM;
-            goto done;
+            goto fail;
         }
     }
 
     if (pwd && (sysdb->domain->legacy_passwords || !*pwd)) {
         ret = sysdb_attrs_add_string(attrs, SYSDB_PWD, pwd);
-        if (ret) goto done;
+        if (ret) goto fail;
     }
 
     ret = sysdb_transaction_start(sysdb);
-    if (ret != EOK) goto done;
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
+        goto fail;
+    }
 
     in_transaction = true;
 
     ret = sysdb_search_user_by_name(tmp_ctx, sysdb,
                                     name, NULL, &msg);
     if (ret && ret != ENOENT) {
-        goto done;
+        goto fail;
     }
 
     /* get transaction timestamp */
@@ -1511,60 +1557,68 @@ int sysdb_store_user(struct sysdb_ctx *sysdb,
                 /* Not found by UID, return the original EEXIST,
                  * this may be a conflict in MPG domain or something
                  * else */
-                return EEXIST;
+                ret = EEXIST;
+                goto fail;
             } else if (ret != EOK) {
-                goto done;
+                goto fail;
             }
             DEBUG(SSSDBG_MINOR_FAILURE,
                   ("A user with the same UID [%llu] was removed from the "
-                   "cache\n", uid));
+                   "cache\n", (unsigned long long) uid));
             ret = sysdb_add_user(sysdb, name, uid, gid, gecos,
                                  homedir, shell, attrs, cache_timeout, now);
         }
-        goto done;
+
+        /* Handle the result of sysdb_add_user */
+        if (ret == EOK) {
+            goto done;
+        } else {
+            DEBUG(SSSDBG_OP_FAILURE, ("Could not add user\n"));
+            goto fail;
+        }
     }
 
     /* the user exists, let's just replace attributes when set */
     if (uid) {
         ret = sysdb_attrs_add_uint32(attrs, SYSDB_UIDNUM, uid);
-        if (ret) goto done;
+        if (ret) goto fail;
     }
 
     if (gid) {
         ret = sysdb_attrs_add_uint32(attrs, SYSDB_GIDNUM, gid);
-        if (ret) goto done;
+        if (ret) goto fail;
     }
 
     if (uid && !gid && sysdb->mpg) {
         ret = sysdb_attrs_add_uint32(attrs, SYSDB_GIDNUM, uid);
-        if (ret) goto done;
+        if (ret) goto fail;
     }
 
     if (gecos) {
         ret = sysdb_attrs_add_string(attrs, SYSDB_GECOS, gecos);
-        if (ret) goto done;
+        if (ret) goto fail;
     }
 
     if (homedir) {
         ret = sysdb_attrs_add_string(attrs, SYSDB_HOMEDIR, homedir);
-        if (ret) goto done;
+        if (ret) goto fail;
     }
 
     if (shell) {
         ret = sysdb_attrs_add_string(attrs, SYSDB_SHELL, shell);
-        if (ret) goto done;
+        if (ret) goto fail;
     }
 
     ret = sysdb_attrs_add_time_t(attrs, SYSDB_LAST_UPDATE, now);
-    if (ret) goto done;
+    if (ret) goto fail;
 
     ret = sysdb_attrs_add_time_t(attrs, SYSDB_CACHE_EXPIRE,
                                  ((cache_timeout) ?
                                   (now + cache_timeout) : 0));
-    if (ret) goto done;
+    if (ret) goto fail;
 
     ret = sysdb_set_user_attr(sysdb, name, attrs, SYSDB_MOD_REP);
-    if (ret != EOK) goto done;
+    if (ret != EOK) goto fail;
 
     if (remove_attrs) {
         ret = sysdb_remove_attrs(sysdb, name,
@@ -1576,21 +1630,22 @@ int sysdb_store_user(struct sysdb_ctx *sysdb,
     }
 
 done:
-    if (in_transaction) {
-        if (ret == EOK) {
-            sret = sysdb_transaction_commit(sysdb);
-            if (sret != EOK) {
-                DEBUG(2, ("Could not commit transaction\n"));
-            }
-        }
+    ret = sysdb_transaction_commit(sysdb);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
+        goto fail;
+    }
 
-        if (ret != EOK || sret != EOK){
-            sret = sysdb_transaction_cancel(sysdb);
-            if (sret != EOK) {
-                DEBUG(2, ("Could not cancel transaction\n"));
-            }
+    in_transaction = false;
+
+fail:
+    if (in_transaction) {
+        sret = sysdb_transaction_cancel(sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Could not cancel transaction\n"));
         }
     }
+
     if (ret) {
         DEBUG(6, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
@@ -1664,7 +1719,7 @@ int sysdb_store_group(struct sysdb_ctx *sysdb,
             }
             DEBUG(SSSDBG_MINOR_FAILURE,
                   ("A group with the same GID [%llu] was removed from the "
-                   "cache\n", gid));
+                   "cache\n", (unsigned long long) gid));
             ret = sysdb_add_group(sysdb, name, gid, attrs, cache_timeout, now);
         }
         goto done;
@@ -1712,49 +1767,25 @@ sysdb_group_membership_mod(struct sysdb_ctx *sysdb,
     }
 
     if (type == SYSDB_MEMBER_USER) {
-        member_dn = sysdb_user_dn(sysdb, tmp_ctx, sysdb->domain->name, member);
-        if (!member_dn) {
-            ret = ENOMEM;
-            goto done;
-        }
-
-        group_dn = sysdb_group_dn(sysdb, tmp_ctx, sysdb->domain->name, group);
-        if (!group_dn) {
-            ret = ENOMEM;
-            goto done;
-        }
+        member_dn = sysdb_user_dn(sysdb, tmp_ctx, member);
     } else if (type == SYSDB_MEMBER_GROUP) {
-        member_dn = sysdb_group_dn(sysdb, tmp_ctx, sysdb->domain->name, member);
-        if (!member_dn) {
-            ret = ENOMEM;
-            goto done;
-        }
-
-        group_dn = sysdb_group_dn(sysdb, tmp_ctx, sysdb->domain->name, group);
-        if (!group_dn) {
-            ret = ENOMEM;
-            goto done;
-        }
-    } else if (type == SYSDB_MEMBER_AUTOFSENTRY) {
-        /* FIXME - I don't like autofs specific stuff in sysdb_ops.c
-         * Maybe we should introduce sysdb_common.c ?
-         */
-        member_dn = sysdb_autofsentry_dn(tmp_ctx, sysdb, member);
-        if (!member_dn) {
-            ret = ENOMEM;
-            goto done;
-        }
-
-        group_dn = sysdb_autofsmap_dn(tmp_ctx, sysdb, group);
-        if (!group_dn) {
-            ret = ENOMEM;
-            goto done;
-        }
+        member_dn = sysdb_group_dn(sysdb, tmp_ctx, member);
     } else {
         ret = EINVAL;
         goto done;
     }
 
+    if (!member_dn) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    group_dn = sysdb_group_dn(sysdb, tmp_ctx, group);
+    if (!group_dn) {
+        ret = ENOMEM;
+        goto done;
+    }
+
     ret = sysdb_mod_group_member(sysdb, member_dn, group_dn, modify_op);
 
 done:
@@ -1860,8 +1891,7 @@ int sysdb_search_custom(TALLOC_CTX *mem_ctx,
         return EINVAL;
     }
 
-    basedn = sysdb_custom_subtree_dn(sysdb, mem_ctx,
-                                     sysdb->domain->name, subtree_name);
+    basedn = sysdb_custom_subtree_dn(sysdb, mem_ctx, subtree_name);
     if (basedn == NULL) {
         DEBUG(1, ("sysdb_custom_subtree_dn failed.\n"));
         return ENOMEM;
@@ -1900,8 +1930,7 @@ int sysdb_search_custom_by_name(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    basedn = sysdb_custom_dn(sysdb, tmp_ctx,
-                             sysdb->domain->name, object_name, subtree_name);
+    basedn = sysdb_custom_dn(sysdb, tmp_ctx, object_name, subtree_name);
     if (basedn == NULL) {
         DEBUG(1, ("sysdb_custom_dn failed.\n"));
         ret = ENOMEM;
@@ -1983,8 +2012,7 @@ int sysdb_store_custom(struct sysdb_ctx *sysdb,
         goto done;
     }
 
-    msg->dn = sysdb_custom_dn(sysdb, tmp_ctx,
-                              sysdb->domain->name, object_name, subtree_name);
+    msg->dn = sysdb_custom_dn(sysdb, tmp_ctx, object_name, subtree_name);
     if (!msg->dn) {
         DEBUG(1, ("sysdb_custom_dn failed.\n"));
         ret = ENOMEM;
@@ -2018,7 +2046,7 @@ int sysdb_store_custom(struct sysdb_ctx *sysdb,
         ret = ldb_modify(sysdb->ldb, msg);
     }
     if (ret != LDB_SUCCESS) {
-        DEBUG(1, ("Failed to store custmo entry: %s(%d)[%s]\n",
+        DEBUG(1, ("Failed to store custom entry: %s(%d)[%s]\n",
                   ldb_strerror(ret), ret, ldb_errstring(sysdb->ldb)));
         ret = sysdb_error_to_errno(ret);
     }
@@ -2054,7 +2082,7 @@ int sysdb_delete_custom(struct sysdb_ctx *sysdb,
         return ENOMEM;
     }
 
-    dn = sysdb_custom_dn(sysdb, tmp_ctx, sysdb->domain->name, object_name, subtree_name);
+    dn = sysdb_custom_dn(sysdb, tmp_ctx, object_name, subtree_name);
     if (dn == NULL) {
         DEBUG(1, ("sysdb_custom_dn failed.\n"));
         ret = ENOMEM;
@@ -2210,7 +2238,8 @@ int sysdb_search_users(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    DEBUG(6, ("Search users with filter: %s\n", filter));
+    DEBUG(SSSDBG_TRACE_INTERNAL,
+          ("Search users with filter: %s\n", filter));
 
     ret = sysdb_search_entry(mem_ctx, sysdb, basedn,
                              LDB_SCOPE_SUBTREE, filter, attrs,
@@ -2224,10 +2253,10 @@ int sysdb_search_users(TALLOC_CTX *mem_ctx,
 
 fail:
     if (ret == ENOENT) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
+        DEBUG(SSSDBG_TRACE_INTERNAL, ("No such entry\n"));
     }
     else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_MINOR_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
     talloc_zfree(tmp_ctx);
     return ret;
@@ -2239,8 +2268,13 @@ int sysdb_delete_user(struct sysdb_ctx *sysdb,
                       const char *name, uid_t uid)
 {
     TALLOC_CTX *tmp_ctx;
+    const char *attrs[] = {SYSDB_GHOST, NULL};
+    size_t msg_count;
+    char *filter;
+    struct ldb_message **msgs;
     struct ldb_message *msg;
     int ret;
+    int i;
 
     tmp_ctx = talloc_new(NULL);
     if (!tmp_ctx) {
@@ -2254,33 +2288,70 @@ int sysdb_delete_user(struct sysdb_ctx *sysdb,
         ret = sysdb_search_user_by_uid(tmp_ctx, sysdb,
                                        uid, NULL, &msg);
     }
-    if (ret) {
+    if (ret == EOK) {
+        if (name && uid) {
+            /* verify name/gid match */
+            const char *c_name;
+            uint64_t c_uid;
+
+            c_name = ldb_msg_find_attr_as_string(msg, SYSDB_NAME, NULL);
+            c_uid = ldb_msg_find_attr_as_uint64(msg, SYSDB_UIDNUM, 0);
+            if (c_name == NULL || c_uid == 0) {
+                DEBUG(2, ("Attribute is missing but this should never happen!\n"));
+                ret = EFAULT;
+                goto fail;
+            }
+            if (strcmp(name, c_name) || uid != c_uid) {
+                /* this is not the entry we are looking for */
+                ret = EINVAL;
+                goto fail;
+            }
+        }
+
+        ret = sysdb_delete_entry(sysdb, msg->dn, false);
+        if (ret) {
+            goto fail;
+        }
+    } else if (ret == ENOENT && name != NULL) {
+        /* Perhaps a ghost user? */
+        filter = talloc_asprintf(tmp_ctx, "(%s=%s)", SYSDB_GHOST, name);
+        if (filter == NULL) {
+            ret = ENOMEM;
+            goto fail;
+        }
+
+        ret = sysdb_search_groups(tmp_ctx, sysdb, filter, attrs, &msg_count, &msgs);
+        if (ret != EOK) {
+            goto fail;
+        }
+
+        for (i = 0; i < msg_count; i++) {
+            msg = ldb_msg_new(tmp_ctx);
+            if (!msg) {
+                ERROR_OUT(ret, ENOMEM, fail);
+            }
+
+            msg->dn = msgs[i]->dn;
+            ret = ldb_msg_add_empty(msg, SYSDB_GHOST, SYSDB_MOD_DEL, NULL);
+            if (ret != LDB_SUCCESS) {
+                ERROR_OUT(ret, ENOMEM, fail);
+            }
+            ret = ldb_msg_add_string(msg, SYSDB_GHOST, name);
+            if (ret != LDB_SUCCESS) {
+                ERROR_OUT(ret, EINVAL, fail);
+            }
+            ret = ldb_modify(sysdb->ldb, msg);
+            ret = sysdb_error_to_errno(ret);
+            if (ret != EOK) {
+                goto fail;
+            }
+
+            talloc_zfree(msg);
+        }
+    } else {
         goto fail;
     }
 
-    if (name && uid) {
-        /* verify name/gid match */
-        const char *c_name;
-        uint64_t c_uid;
-
-        c_name = ldb_msg_find_attr_as_string(msg, SYSDB_NAME, NULL);
-        c_uid = ldb_msg_find_attr_as_uint64(msg, SYSDB_UIDNUM, 0);
-        if (c_name == NULL || c_uid == 0) {
-            DEBUG(2, ("Attribute is missing but this should never happen!\n"));
-            ret = EFAULT;
-            goto fail;
-        }
-        if (strcmp(name, c_name) || uid != c_uid) {
-            /* this is not the entry we are looking for */
-            ret = EINVAL;
-            goto fail;
-        }
-    }
-
-    ret = sysdb_delete_entry(sysdb, msg->dn, false);
-    if (ret) {
-        goto fail;
-    }
 
     talloc_zfree(tmp_ctx);
     return EOK;
@@ -2326,7 +2397,8 @@ int sysdb_search_groups(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    DEBUG(6, ("Search groups with filter: %s\n", filter));
+    DEBUG(SSSDBG_TRACE_INTERNAL,
+          ("Search groups with filter: %s\n", filter));
 
     ret = sysdb_search_entry(mem_ctx, sysdb, basedn,
                              LDB_SCOPE_SUBTREE, filter, attrs,
@@ -2340,10 +2412,10 @@ int sysdb_search_groups(TALLOC_CTX *mem_ctx,
 
 fail:
     if (ret == ENOENT) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
+        DEBUG(SSSDBG_TRACE_INTERNAL, ("No such entry\n"));
     }
     else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_MINOR_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
     talloc_zfree(tmp_ctx);
     return ret;
@@ -2529,7 +2601,7 @@ errno_t check_failed_login_attempts(struct confdb_ctx *cdb,
                                                 SYSDB_FAILED_LOGIN_ATTEMPTS, 0);
     last_failed_login = (time_t) ldb_msg_find_attr_as_int64(ldb_msg,
                                                     SYSDB_LAST_FAILED_LOGIN, 0);
-    ret = confdb_get_int(cdb, tmp_ctx, CONFDB_PAM_CONF_ENTRY,
+    ret = confdb_get_int(cdb, CONFDB_PAM_CONF_ENTRY,
                          CONFDB_PAM_FAILED_LOGIN_ATTEMPTS,
                          CONFDB_DEFAULT_PAM_FAILED_LOGIN_ATTEMPTS,
                          &allowed_failed_login_attempts);
@@ -2539,7 +2611,7 @@ errno_t check_failed_login_attempts(struct confdb_ctx *cdb,
         ret = EIO;
         goto done;
     }
-    ret = confdb_get_int(cdb, tmp_ctx, CONFDB_PAM_CONF_ENTRY,
+    ret = confdb_get_int(cdb, CONFDB_PAM_CONF_ENTRY,
                          CONFDB_PAM_FAILED_LOGIN_DELAY,
                          CONFDB_DEFAULT_PAM_FAILED_LOGIN_DELAY,
                          &failed_login_delay);
@@ -2654,7 +2726,7 @@ int sysdb_cache_auth(struct sysdb_ctx *sysdb,
                                             SYSDB_LAST_ONLINE_AUTH,
                                             0);
 
-    ret = confdb_get_int(cdb, tmp_ctx, CONFDB_PAM_CONF_ENTRY,
+    ret = confdb_get_int(cdb, CONFDB_PAM_CONF_ENTRY,
                          CONFDB_PAM_CRED_TIMEOUT, 0, &cred_expiration);
     if (ret != EOK) {
         DEBUG(1, ("Failed to read expiration time of offline credentials.\n"));
@@ -2802,7 +2874,9 @@ errno_t sysdb_update_members(struct sysdb_ctx *sysdb,
                              const char *const *del_groups)
 {
     errno_t ret;
+    errno_t sret;
     int i;
+    bool in_transaction = false;
 
     TALLOC_CTX *tmp_ctx = talloc_new(NULL);
     if(!tmp_ctx) {
@@ -2815,6 +2889,8 @@ errno_t sysdb_update_members(struct sysdb_ctx *sysdb,
         goto done;
     }
 
+    in_transaction = true;
+
     if (add_groups) {
         /* Add the user to all add_groups */
         for (i = 0; add_groups[i]; i++) {
@@ -2842,10 +2918,19 @@ errno_t sysdb_update_members(struct sysdb_ctx *sysdb,
     }
 
     ret = sysdb_transaction_commit(sysdb);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
+        goto done;
+    }
+
+    in_transaction = false;
 
 done:
-    if (ret != EOK) {
-        sysdb_transaction_cancel(sysdb);
+    if (in_transaction) {
+        sret = sysdb_transaction_cancel(sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Could not cancel transaction\n"));
+        }
     }
     talloc_free(tmp_ctx);
     return ret;
@@ -2888,7 +2973,7 @@ errno_t sysdb_mod_netgroup_tuple(struct sysdb_ctx *sysdb,
         ERROR_OUT(ret, ENOMEM, done);
     }
 
-    msg->dn = sysdb_netgroup_dn(sysdb, msg, sysdb->domain->name, netgroup);
+    msg->dn = sysdb_netgroup_dn(sysdb, msg, netgroup);
     if (!msg->dn) {
         ERROR_OUT(ret, ENOMEM, done);
     }
@@ -2946,7 +3031,7 @@ errno_t sysdb_mod_netgroup_member(struct sysdb_ctx *sysdb,
         ERROR_OUT(ret, ENOMEM, done);
     }
 
-    msg->dn = sysdb_netgroup_dn(sysdb, msg, sysdb->domain->name, netgroup);
+    msg->dn = sysdb_netgroup_dn(sysdb, msg, netgroup);
     if (!msg->dn) {
         ERROR_OUT(ret, ENOMEM, done);
     }
@@ -2991,24 +3076,20 @@ errno_t sysdb_remove_attrs(struct sysdb_ctx *sysdb,
 
     switch(type) {
     case SYSDB_MEMBER_USER:
-        msg->dn = sysdb_user_dn(sysdb, msg, sysdb->domain->name, name);
+        msg->dn = sysdb_user_dn(sysdb, msg, name);
         break;
 
     case SYSDB_MEMBER_GROUP:
-        msg->dn = sysdb_group_dn(sysdb, msg, sysdb->domain->name, name);
+        msg->dn = sysdb_group_dn(sysdb, msg, name);
         break;
 
     case SYSDB_MEMBER_NETGROUP:
-        msg->dn = sysdb_netgroup_dn(sysdb, msg, sysdb->domain->name, name);
+        msg->dn = sysdb_netgroup_dn(sysdb, msg, name);
         break;
 
     case SYSDB_MEMBER_SERVICE:
         msg->dn = sysdb_svc_dn(sysdb, msg, sysdb->domain->name, name);
         break;
-
-    case SYSDB_MEMBER_AUTOFSENTRY:
-        msg->dn = sysdb_autofsmap_dn(msg, sysdb, name);
-        break;
     }
     if (!msg->dn) {
         ret = ENOMEM;
@@ -3016,7 +3097,10 @@ errno_t sysdb_remove_attrs(struct sysdb_ctx *sysdb,
     }
 
     ret = sysdb_transaction_start(sysdb);
-    if (ret != EOK) goto done;
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
+        goto done;
+    }
 
     in_transaction = true;
 
@@ -3051,20 +3135,19 @@ errno_t sysdb_remove_attrs(struct sysdb_ctx *sysdb,
 
     ret = EOK;
 
+    ret = sysdb_transaction_commit(sysdb);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
+        goto done;
+    }
+
+    in_transaction = false;
+
 done:
     if (in_transaction) {
-        if (ret == EOK) {
-            sret = sysdb_transaction_commit(sysdb);
-            if (sret != EOK) {
-                DEBUG(2, ("Could not commit transaction\n"));
-            }
-        }
-
-        if (ret != EOK || sret != EOK){
-            sret = sysdb_transaction_cancel(sysdb);
-            if (sret != EOK) {
-                DEBUG(2, ("Could not cancel transaction\n"));
-            }
+        sret = sysdb_transaction_cancel(sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Could not cancel transaction\n"));
         }
     }
     talloc_free(msg);
