@@ -84,6 +84,7 @@ static int be_pam_handler(DBusMessage *message, struct sbus_connection *conn);
 static int be_sudo_handler(DBusMessage *message, struct sbus_connection *conn);
 static int be_autofs_handler(DBusMessage *message, struct sbus_connection *conn);
 static int be_host_handler(DBusMessage *message, struct sbus_connection *conn);
+static int be_get_subdomains(DBusMessage *message, struct sbus_connection *conn);
 
 struct sbus_method be_methods[] = {
     { DP_METHOD_REGISTER, client_registration },
@@ -92,6 +93,7 @@ struct sbus_method be_methods[] = {
     { DP_METHOD_SUDOHANDLER, be_sudo_handler },
     { DP_METHOD_AUTOFSHANDLER, be_autofs_handler },
     { DP_METHOD_HOSTHANDLER, be_host_handler },
+    { DP_METHOD_GETDOMAINS, be_get_subdomains },
     { NULL, NULL }
 };
 
@@ -110,9 +112,10 @@ static struct bet_data bet_data[] = {
     {BET_ACCESS, CONFDB_DOMAIN_ACCESS_PROVIDER, "sssm_%s_access_init"},
     {BET_CHPASS, CONFDB_DOMAIN_CHPASS_PROVIDER, "sssm_%s_chpass_init"},
     {BET_SUDO, CONFDB_DOMAIN_SUDO_PROVIDER, "sssm_%s_sudo_init"},
-    {BET_AUTOFS, CONFDB_DOMAIN_SUDO_PROVIDER, "sssm_%s_autofs_init"},
-    {BET_SESSION, CONFDB_DOMAIN_SESSION_PROVIDER, "sssm_%s_session_init"},
+    {BET_AUTOFS, CONFDB_DOMAIN_AUTOFS_PROVIDER, "sssm_%s_autofs_init"},
+    {BET_SELINUX, CONFDB_DOMAIN_SELINUX_PROVIDER, "sssm_%s_selinux_init"},
     {BET_HOSTID, CONFDB_DOMAIN_HOSTID_PROVIDER, "sssm_%s_hostid_init"},
+    {BET_SUBDOMAINS, CONFDB_DOMAIN_SUBDOMAINS_PROVIDER, "sssm_%s_subdomains_init"},
     {BET_MAX, NULL, NULL}
 };
 
@@ -305,6 +308,198 @@ static char *dp_pam_err_to_string(TALLOC_CTX *memctx, int dp_err_type, int errnu
     return NULL;
 }
 
+static void get_subdomains_callback(struct be_req *req,
+                                    int dp_err_type,
+                                    int errnum,
+                                    const char *errstr)
+{
+    DBusMessage *reply;
+    DBusConnection *dbus_conn;
+    dbus_bool_t dbret;
+    dbus_uint16_t err_maj = 0;
+    dbus_uint32_t err_min = 0;
+    const char *err_msg = NULL;
+
+    DEBUG(SSSDBG_TRACE_FUNC, ("Backend returned: (%d, %d, %s) [%s]\n",
+              dp_err_type, errnum, errstr?errstr:"<NULL>",
+              dp_pam_err_to_string(req, dp_err_type, errnum)));
+
+    reply = (DBusMessage *)req->pvt;
+
+    if (reply) {
+        /* Return a reply if one was requested
+         * There may not be one if this request began
+         * while we were offline
+         */
+        err_maj = dp_err_type;
+        err_min = errnum;
+        if (errstr) {
+            err_msg = errstr;
+        } else {
+            err_msg = dp_pam_err_to_string(req, dp_err_type, errnum);
+        }
+        if (!err_msg) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  ("Failed to set err_msg, Out of memory?\n"));
+            err_msg = "OOM";
+        }
+
+        dbret = dbus_message_append_args(reply,
+                                         DBUS_TYPE_UINT16, &err_maj,
+                                         DBUS_TYPE_UINT32, &err_min,
+                                         DBUS_TYPE_STRING, &err_msg,
+                                         DBUS_TYPE_INVALID);
+
+        if (!dbret) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to generate dbus reply\n"));
+            dbus_message_unref(reply);
+            goto done;
+        }
+
+        dbus_conn = sbus_get_connection(req->becli->conn);
+        if (dbus_conn == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("D-BUS not connected\n"));
+            goto done;
+        }
+        dbus_connection_send(dbus_conn, reply, NULL);
+        dbus_message_unref(reply);
+    }
+
+done:
+    talloc_free(req);
+}
+
+static int be_get_subdomains(DBusMessage *message, struct sbus_connection *conn)
+{
+    struct be_subdom_req *req;
+    struct be_req *be_req = NULL;
+    struct be_client *becli;
+    DBusMessage *reply;
+    DBusError dbus_error;
+    dbus_bool_t dbret;
+    void *user_data;
+    bool force;
+    char *domain_hint;
+    dbus_uint16_t err_maj;
+    dbus_uint32_t err_min;
+    const char *err_msg;
+    int ret;
+
+    user_data = sbus_conn_get_private_data(conn);
+    if (!user_data) return EINVAL;
+    becli = talloc_get_type(user_data, struct be_client);
+    if (!becli) return EINVAL;
+
+    dbus_error_init(&dbus_error);
+
+    ret = dbus_message_get_args(message, &dbus_error,
+                                DBUS_TYPE_BOOLEAN, &force,
+                                DBUS_TYPE_STRING, &domain_hint,
+                                DBUS_TYPE_INVALID);
+    if (!ret) {
+        DEBUG(SSSDBG_CRIT_FAILURE,("Failed, to parse message!\n"));
+        if (dbus_error_is_set(&dbus_error)) dbus_error_free(&dbus_error);
+        return EIO;
+    }
+
+    reply = dbus_message_new_method_return(message);
+    if (!reply) return ENOMEM;
+
+    /* return an error if corresponding backend target is not configured */
+    if (becli->bectx->bet_info[BET_SUBDOMAINS].bet_ops == NULL) {
+        DEBUG(SSSDBG_TRACE_INTERNAL, ("Undefined backend target.\n"));
+        err_maj = DP_ERR_FATAL;
+        err_min = ENODEV;
+        err_msg = "Subdomains back end target is not configured";
+        goto immediate;
+    }
+
+    DEBUG(SSSDBG_TRACE_FUNC,
+          ("Got get subdomains [%sforced][%s]\n", force ? "" : "not ",
+          domain_hint == NULL ? "no hint": domain_hint ));
+
+    /* If we are offline return immediately
+     */
+    if (becli->bectx->offstat.offline) {
+        DEBUG(SSSDBG_TRACE_FUNC, ("Cannot proceed, provider is offline.\n"));
+        err_maj = DP_ERR_OFFLINE;
+        err_min = EAGAIN;
+        err_msg = "Provider is offline";
+        goto immediate;
+    }
+
+    /* process request */
+    be_req = talloc_zero(becli, struct be_req);
+    if (!be_req) {
+        err_maj = DP_ERR_FATAL;
+        err_min = ENOMEM;
+        err_msg = "Out of memory";
+        goto immediate;
+    }
+    be_req->becli = becli;
+    be_req->be_ctx = becli->bectx;
+    be_req->fn = get_subdomains_callback;
+    be_req->pvt = reply;
+
+    req = talloc(be_req, struct be_subdom_req);
+    if (!req) {
+        err_maj = DP_ERR_FATAL;
+        err_min = ENOMEM;
+        err_msg = "Out of memory";
+        goto immediate;
+    }
+    req->force = force;
+    req->domain_hint = talloc_strdup(req, domain_hint);
+    if (!req->domain_hint) {
+        err_maj = DP_ERR_FATAL;
+        err_min = ENOMEM;
+        err_msg = "Out of memory";
+        goto immediate;
+    }
+
+    be_req->req_data = req;
+
+    ret = be_file_request(becli->bectx,
+                          be_req,
+                          becli->bectx->bet_info[BET_SUBDOMAINS].bet_ops->handler);
+    if (ret != EOK) {
+        err_maj = DP_ERR_FATAL;
+        err_min = ret;
+        err_msg = "Cannot file back end request";
+        goto immediate;
+    }
+
+    return EOK;
+
+immediate:
+    if (be_req) {
+        talloc_free(be_req);
+    }
+
+    if (reply) {
+        dbret = dbus_message_append_args(reply,
+                                         DBUS_TYPE_UINT16, &err_maj,
+                                         DBUS_TYPE_UINT32, &err_min,
+                                         DBUS_TYPE_STRING, &err_msg,
+                                         DBUS_TYPE_INVALID);
+        if (!dbret) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to generate dbus reply\n"));
+            dbus_message_unref(reply);
+            return EIO;
+        }
+
+        if (!(err_maj == DP_ERR_FATAL && err_min == ENODEV)) {
+            DEBUG(SSSDBG_TRACE_LIBS, ("Request processed. Returned %d,%d,%s\n",
+                  err_maj, err_min, err_msg));
+        }
+
+        /* send reply back */
+        sbus_conn_send_reply(conn, reply);
+        dbus_message_unref(reply);
+    }
+
+    return EOK;
+}
 
 static void acctinfo_callback(struct be_req *req,
                               int dp_err_type,
@@ -366,10 +561,10 @@ static void acctinfo_callback(struct be_req *req,
 }
 
 static errno_t
-split_service_name_filter(TALLOC_CTX *mem_ctx,
-                          const char *filter,
-                          char **name,
-                          char **protocol)
+split_name_extended(TALLOC_CTX *mem_ctx,
+                    const char *filter,
+                    char **name,
+                    char **extended)
 {
     char *p;
 
@@ -380,12 +575,12 @@ split_service_name_filter(TALLOC_CTX *mem_ctx,
 
     p = strchr(*name, ':');
     if (p) {
-        /* Protocol included */
+        /* Extended info included */
         *p = '\0';
 
-        *protocol = p + 1;
+        *extended = p + 1;
     } else {
-        *protocol = NULL;
+        *extended = NULL;
     }
 
     return EOK;
@@ -402,6 +597,7 @@ static int be_get_account_info(DBusMessage *message, struct sbus_connection *con
     void *user_data;
     uint32_t type;
     char *filter;
+    char *domain;
     uint32_t attr_type;
     int ret;
     dbus_uint16_t err_maj;
@@ -421,6 +617,7 @@ static int be_get_account_info(DBusMessage *message, struct sbus_connection *con
                                 DBUS_TYPE_UINT32, &type,
                                 DBUS_TYPE_UINT32, &attr_type,
                                 DBUS_TYPE_STRING, &filter,
+                                DBUS_TYPE_STRING, &domain,
                                 DBUS_TYPE_INVALID);
     if (!ret) {
         DEBUG(1,("Failed, to parse message!\n"));
@@ -484,6 +681,13 @@ static int be_get_account_info(DBusMessage *message, struct sbus_connection *con
     }
     req->entry_type = type;
     req->attr_type = (int)attr_type;
+    req->domain = talloc_strdup(req, domain);
+    if (!req->domain) {
+        err_maj = DP_ERR_FATAL;
+        err_min = ENOMEM;
+        err_msg = "Out of memory";
+        goto done;
+    }
 
     be_req->req_data = req;
 
@@ -501,14 +705,14 @@ static int be_get_account_info(DBusMessage *message, struct sbus_connection *con
         ret = EOK;
         if (strncmp(filter, "name=", 5) == 0) {
             req->filter_type = BE_FILTER_NAME;
-            ret = split_service_name_filter(req, &filter[5],
-                                            &req->filter_value,
-                                            &req->extra_value);
+            ret = split_name_extended(req, &filter[5],
+                                      &req->filter_value,
+                                      &req->extra_value);
         } else if (strncmp(filter, "idnumber=", 9) == 0) {
             req->filter_type = BE_FILTER_IDNUM;
-            ret = split_service_name_filter(req, &filter[9],
-                                            &req->filter_value,
-                                            &req->extra_value);
+            ret = split_name_extended(req, &filter[9],
+                                      &req->filter_value,
+                                      &req->extra_value);
         } else if (strcmp(filter, ENUM_INDICATOR) == 0) {
             req->filter_type = BE_FILTER_ENUM;
             req->filter_value = NULL;
@@ -576,10 +780,12 @@ static void be_pam_handler_callback(struct be_req *req,
                                     int errnum,
                                     const char *errstr)
 {
+    struct be_client *becli = req->becli;
     struct pam_data *pd;
     DBusMessage *reply;
     DBusConnection *dbus_conn;
     dbus_bool_t dbret;
+    errno_t ret;
 
     DEBUG(4, ("Backend returned: (%d, %d, %s) [%s]\n",
               dp_err_type, errnum, errstr?errstr:"<NULL>",
@@ -587,19 +793,42 @@ static void be_pam_handler_callback(struct be_req *req,
 
     pd = talloc_get_type(req->req_data, struct pam_data);
 
+    if (pd->cmd == SSS_PAM_ACCT_MGMT &&
+        pd->pam_status == PAM_SUCCESS &&
+        req->phase == REQ_PHASE_ACCESS &&
+        dp_err_type == DP_ERR_OK) {
+        if (!becli->bectx->bet_info[BET_SELINUX].bet_ops) {
+            DEBUG(SSSDBG_TRACE_FUNC,
+                  ("SELinux provider doesn't exist, "
+                   "not sending the request to it.\n"));
+        } else {
+            req->phase = REQ_PHASE_SELINUX;
+
+            /* Now is the time to call SELinux provider */
+            ret = be_file_request(becli->bectx->bet_info[BET_SELINUX].pvt_bet_data,
+                                  req,
+                                  becli->bectx->bet_info[BET_SELINUX].bet_ops->handler);
+            if (ret != EOK) {
+                DEBUG(SSSDBG_OP_FAILURE, ("be_file_request failed.\n"));
+                goto done;
+            }
+            return;
+        }
+    }
+
     DEBUG(4, ("Sending result [%d][%s]\n", pd->pam_status, pd->domain));
     reply = (DBusMessage *)req->pvt;
     dbret = dp_pack_pam_response(reply, pd);
     if (!dbret) {
         DEBUG(1, ("Failed to generate dbus reply\n"));
         dbus_message_unref(reply);
-        return;
+        goto done;
     }
 
     dbus_conn = sbus_get_connection(req->becli->conn);
     if (!dbus_conn) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("D-BUS not connected\n"));
-        return;
+        goto done;
     }
 
     dbus_connection_send(dbus_conn, reply, NULL);
@@ -607,6 +836,7 @@ static void be_pam_handler_callback(struct be_req *req,
 
     DEBUG(4, ("Sent result [%d][%s]\n", pd->pam_status, pd->domain));
 
+done:
     talloc_free(req);
 }
 
@@ -643,6 +873,8 @@ static int be_pam_handler(DBusMessage *message, struct sbus_connection *conn)
     be_req->be_ctx = becli->bectx;
     be_req->fn = be_pam_handler_callback;
     be_req->pvt = reply;
+    be_req->domain = be_req->be_ctx->domain;
+    be_req->sysdb = be_req->be_ctx->sysdb;
 
     dbus_error_init(&dbus_error);
 
@@ -654,10 +886,12 @@ static int be_pam_handler(DBusMessage *message, struct sbus_connection *conn)
     }
 
     pd->pam_status = PAM_SYSTEM_ERR;
-    pd->domain = talloc_strdup(pd, becli->bectx->domain->name);
     if (pd->domain == NULL) {
-        talloc_free(be_req);
-        return ENOMEM;
+        pd->domain = talloc_strdup(pd, becli->bectx->domain->name);
+        if (pd->domain == NULL) {
+            talloc_free(be_req);
+            return ENOMEM;
+        }
     }
 
 
@@ -670,14 +904,13 @@ static int be_pam_handler(DBusMessage *message, struct sbus_connection *conn)
             break;
         case SSS_PAM_ACCT_MGMT:
             target = BET_ACCESS;
+            be_req->phase = REQ_PHASE_ACCESS;
             break;
         case SSS_PAM_CHAUTHTOK:
         case SSS_PAM_CHAUTHTOK_PRELIM:
             target = BET_CHPASS;
             break;
         case SSS_PAM_OPEN_SESSION:
-            target = BET_SESSION;
-            break;
         case SSS_PAM_SETCRED:
         case SSS_PAM_CLOSE_SESSION:
             pd->pam_status = PAM_SUCCESS;
@@ -787,14 +1020,18 @@ static int be_sudo_handler(DBusMessage *message, struct sbus_connection *conn)
 {
     DBusError dbus_error;
     DBusMessage *reply = NULL;
+    DBusMessageIter iter;
+    dbus_bool_t iter_next = FALSE;
     struct be_client *be_cli = NULL;
     struct be_req *be_req = NULL;
     struct be_sudo_req *sudo_req = NULL;
     void *user_data = NULL;
     int ret = 0;
     uint32_t type;
-    char *filter;
+    uint32_t rules_num = 0;
+    const char *rule = NULL;
     const char *err_msg = NULL;
+    int i;
 
     DEBUG(SSSDBG_TRACE_FUNC, ("Entering be_sudo_handler()\n"));
 
@@ -828,21 +1065,18 @@ static int be_sudo_handler(DBusMessage *message, struct sbus_connection *conn)
     be_req->pvt = reply;
     be_req->fn = be_sudo_handler_callback;
 
-    /* get arguments */
     dbus_error_init(&dbus_error);
+    dbus_message_iter_init(message, &iter);
 
-    ret = dbus_message_get_args(message, &dbus_error,
-                                DBUS_TYPE_UINT32, &type,
-                                DBUS_TYPE_STRING, &filter,
-                                DBUS_TYPE_INVALID);
-    if (!ret) {
-        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed, to parse message!\n"));
-        if (dbus_error_is_set(&dbus_error)) dbus_error_free(&dbus_error);
+    /* get type of the request */
+    if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_UINT32) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed, to parse the message!\n"));
         ret = EIO;
-        err_msg = "dbus_message_get_args failed";
+        err_msg = "Invalid D-Bus message format";
         goto fail;
     }
-
+    dbus_message_iter_get_basic(&iter, &type);
+    dbus_message_iter_next(&iter); /* step behind the request type */
 
     /* If we are offline and fast reply was requested
      * return offline immediately
@@ -867,32 +1101,61 @@ static int be_sudo_handler(DBusMessage *message, struct sbus_connection *conn)
     }
 
     sudo_req->type = (~BE_REQ_FAST) & type;
-    sudo_req->uid = 0;
-    sudo_req->groups = NULL;
 
+    /* get additional arguments according to the request type */
     switch (sudo_req->type) {
-    case BE_REQ_SUDO_ALL:
-    case BE_REQ_SUDO_DEFAULTS:
-        sudo_req->username = NULL;
+    case BE_REQ_SUDO_FULL:
+        /* no arguments required */
         break;
-    case BE_REQ_SUDO_USER:
-        if (filter) {
-            if (strncmp(filter, "name=", 5) == 0) {
-                sudo_req->username = talloc_strdup(sudo_req, &filter[5]);
-                if (sudo_req->username == NULL) {
-                    ret = ENOMEM;
-                    goto fail;
-                }
-            } else {
-                ret = EINVAL;
-                err_msg = "Invalid Filter";
-                goto fail;
-            }
-        } else {
-            ret = EINVAL;
-            err_msg = "Missing Filter Parameter";
+    case BE_REQ_SUDO_RULES:
+        /* additional arguments:
+         * rules_num
+         * rules[rules_num]
+         */
+        /* read rules_num */
+        if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_UINT32) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Failed, to parse the message!\n"));
+            ret = EIO;
+            err_msg = "Invalid D-Bus message format";
             goto fail;
         }
+
+        dbus_message_iter_get_basic(&iter, &rules_num);
+
+        sudo_req->rules = talloc_array(sudo_req, char*, rules_num + 1);
+        if (sudo_req->rules == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("talloc_array failed.\n"));
+            ret = ENOMEM;
+            goto fail;
+        }
+
+        /* read the rules */
+        for (i = 0; i < rules_num; i++) {
+            iter_next = dbus_message_iter_next(&iter);
+            if (iter_next == FALSE) {
+                DEBUG(SSSDBG_CRIT_FAILURE, ("Failed, to parse the message!\n"));
+                ret = EIO;
+                err_msg = "Invalid D-Bus message format";
+                goto fail;
+            }
+            if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
+                DEBUG(SSSDBG_CRIT_FAILURE, ("Failed, to parse the message!\n"));
+                ret = EIO;
+                err_msg = "Invalid D-Bus message format";
+                goto fail;
+            }
+
+            dbus_message_iter_get_basic(&iter, &rule);
+            sudo_req->rules[i] = talloc_strdup(sudo_req->rules, rule);
+            if (sudo_req->rules[i] == NULL) {
+                DEBUG(SSSDBG_CRIT_FAILURE, ("talloc_strdup failed.\n"));
+                ret = ENOMEM;
+                goto fail;
+            }
+        }
+
+        sudo_req->rules[rules_num] = NULL;
+
         break;
     default:
         DEBUG(SSSDBG_CRIT_FAILURE, ("Invalid request type %d\n", sudo_req->type));
@@ -1173,7 +1436,7 @@ static void be_autofs_handler_callback(struct be_req *req,
 
 static int be_host_handler(DBusMessage *message, struct sbus_connection *conn)
 {
-    struct be_acct_req *req;
+    struct be_host_req *req;
     struct be_req *be_req;
     struct be_client *becli;
     DBusMessage *reply;
@@ -1182,7 +1445,6 @@ static int be_host_handler(DBusMessage *message, struct sbus_connection *conn)
     void *user_data;
     uint32_t flags;
     char *filter;
-    uint32_t attr_type;
     int ret;
     dbus_uint16_t err_maj;
     dbus_uint32_t err_min;
@@ -1199,7 +1461,6 @@ static int be_host_handler(DBusMessage *message, struct sbus_connection *conn)
 
     ret = dbus_message_get_args(message, &dbus_error,
                                 DBUS_TYPE_UINT32, &flags,
-                                DBUS_TYPE_UINT32, &attr_type,
                                 DBUS_TYPE_STRING, &filter,
                                 DBUS_TYPE_INVALID);
     if (!ret) {
@@ -1209,7 +1470,7 @@ static int be_host_handler(DBusMessage *message, struct sbus_connection *conn)
     }
 
     DEBUG(SSSDBG_TRACE_LIBS,
-          ("Got request for [%u][%d][%s]\n", flags, attr_type, filter));
+          ("Got request for [%u][%s]\n", flags, filter));
 
     reply = dbus_message_new_method_return(message);
     if (!reply) return ENOMEM;
@@ -1257,35 +1518,27 @@ static int be_host_handler(DBusMessage *message, struct sbus_connection *conn)
     be_req->fn = acctinfo_callback;
     be_req->pvt = reply;
 
-    req = talloc(be_req, struct be_acct_req);
+    req = talloc(be_req, struct be_host_req);
     if (!req) {
         err_maj = DP_ERR_FATAL;
         err_min = ENOMEM;
         err_msg = "Out of memory";
         goto done;
     }
-    req->entry_type = BE_REQ_HOST | (flags & BE_REQ_FAST);
-    req->attr_type = (int)attr_type;
+    req->type = BE_REQ_HOST | (flags & BE_REQ_FAST);
 
     be_req->req_data = req;
 
-    if ((attr_type != BE_ATTR_CORE) &&
-        (attr_type != BE_ATTR_MEM) &&
-        (attr_type != BE_ATTR_ALL)) {
-        /* Unrecognized attr type */
-        err_maj = DP_ERR_FATAL;
-        err_min = EINVAL;
-        err_msg = "Invalid Attrs Parameter";
-        goto done;
-    }
-
     if (filter) {
-        if (strncmp(filter, "name=", 5) == 0) {
+        ret = strncmp(filter, "name=", 5);
+        if (ret == 0) {
             req->filter_type = BE_FILTER_NAME;
-            ret = split_service_name_filter(req, &filter[5],
-                                            &req->filter_value,
-                                            &req->extra_value);
-        } else {
+            ret = split_name_extended(req, &filter[5],
+                                      &req->name,
+                                      &req->alias);
+        }
+
+        if (ret) {
             err_maj = DP_ERR_FATAL;
             err_min = EINVAL;
             err_msg = "Invalid Filter";
@@ -1824,6 +2077,68 @@ static void signal_be_offline(struct tevent_context *ev,
     be_mark_offline(ctx);
 }
 
+int be_process_init_sudo(struct be_ctx *be_ctx)
+{
+    TALLOC_CTX *tmp_ctx = NULL;
+    char **services = NULL;
+    char *provider = NULL;
+    bool responder_enabled = false;
+    int i;
+    int ret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("talloc_new() failed\n"));
+        return ENOMEM;
+    }
+
+    ret = confdb_get_string_as_list(be_ctx->cdb, tmp_ctx,
+                                    CONFDB_MONITOR_CONF_ENTRY,
+                                    CONFDB_MONITOR_ACTIVE_SERVICES, &services);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, ("Unable to read from confdb [%d]: %s\n",
+                                     ret, strerror(ret)));
+        goto done;
+    }
+
+    for (i = 0; services[i] != NULL; i++) {
+        if (strcmp(services[i], "sudo") == 0) {
+            responder_enabled = true;
+            break;
+        }
+    }
+
+    ret = confdb_get_string(be_ctx->cdb, tmp_ctx, be_ctx->conf_path,
+                            CONFDB_DOMAIN_SUDO_PROVIDER, NULL, &provider);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, ("Unable to read from confdb [%d]: %s\n",
+                                     ret, strerror(ret)));
+        goto done;
+    }
+
+    if (!responder_enabled && provider == NULL) {
+        /* provider is not set explicitly */
+        DEBUG(SSSDBG_TRACE_FUNC,
+              ("SUDO is not listed in services, disabling SUDO module.\n"));
+        ret = ENOENT;
+        goto done;
+    }
+
+    if (!responder_enabled && provider != NULL
+            && strcmp(provider, NO_PROVIDER) != 0) {
+        /* provider is set but responder is disabled */
+        DEBUG(SSSDBG_MINOR_FAILURE, ("SUDO provider is set, but it is not "
+              "listed in active services. SUDO support will not work!\n"));
+    }
+
+    ret = load_backend_module(be_ctx, BET_SUDO, &be_ctx->bet_info[BET_SUDO],
+                              be_ctx->bet_info[BET_ID].mod_name);
+
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
 int be_process_init(TALLOC_CTX *mem_ctx,
                     const char *be_domain,
                     struct tevent_context *ev,
@@ -1844,27 +2159,22 @@ int be_process_init(TALLOC_CTX *mem_ctx,
     ctx->conf_path = talloc_asprintf(ctx, CONFDB_DOMAIN_PATH_TMPL, be_domain);
     if (!ctx->identity || !ctx->conf_path) {
         DEBUG(0, ("Out of memory!?\n"));
-        return ENOMEM;
+        ret = ENOMEM;
+        goto fail;
     }
 
     ret = be_init_failover(ctx);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               ("fatal error initializing failover context\n"));
-        return ret;
+        goto fail;
     }
 
-    ret = confdb_get_domain(cdb, be_domain, &ctx->domain);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_FATAL_FAILURE,
-              ("fatal error retrieving domain configuration\n"));
-        return ret;
-    }
-
-    ret = sysdb_domain_init(ctx, ctx->domain, DB_PATH, &ctx->sysdb);
+    ret = sysdb_init_domain_and_sysdb(ctx, cdb, be_domain, DB_PATH,
+                                      &ctx->domain, &ctx->sysdb);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, ("fatal error opening cache database\n"));
-        return ret;
+        goto fail;
     }
 
     ret = sss_monitor_init(ctx, ctx->ev, &monitor_be_interface,
@@ -1873,13 +2183,13 @@ int be_process_init(TALLOC_CTX *mem_ctx,
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               ("fatal error setting up monitor bus\n"));
-        return ret;
+        goto fail;
     }
 
     ret = be_srv_init(ctx);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, ("fatal error setting up server bus\n"));
-        return ret;
+        goto fail;
     }
 
     ret = load_backend_module(ctx, BET_ID,
@@ -1887,7 +2197,7 @@ int be_process_init(TALLOC_CTX *mem_ctx,
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               ("fatal error initializing data providers\n"));
-        return ret;
+        goto fail;
     }
     DEBUG(SSSDBG_TRACE_INTERNAL,
           ("ID backend target successfully loaded from provider [%s].\n",
@@ -1900,7 +2210,7 @@ int be_process_init(TALLOC_CTX *mem_ctx,
         if (ret != ENOENT) {
             DEBUG(SSSDBG_FATAL_FAILURE,
                   ("fatal error initializing data providers\n"));
-            return ret;
+            goto fail;
         }
         DEBUG(SSSDBG_MINOR_FAILURE,
               ("No authentication module provided for [%s] !!\n",
@@ -1916,7 +2226,7 @@ int be_process_init(TALLOC_CTX *mem_ctx,
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               ("Failed to setup ACCESS backend.\n"));
-        return ret;
+        goto fail;
     }
     DEBUG(SSSDBG_TRACE_INTERNAL,
           ("ACCESS backend target successfully loaded "
@@ -1929,7 +2239,7 @@ int be_process_init(TALLOC_CTX *mem_ctx,
         if (ret != ENOENT) {
             DEBUG(SSSDBG_FATAL_FAILURE,
                   ("fatal error initializing data providers\n"));
-            return ret;
+            goto fail;
         }
         DEBUG(SSSDBG_MINOR_FAILURE,
               ("No change password module provided for [%s] !!\n",
@@ -1940,14 +2250,12 @@ int be_process_init(TALLOC_CTX *mem_ctx,
                "from provider [%s].\n", ctx->bet_info[BET_CHPASS].mod_name));
     }
 
-    ret = load_backend_module(ctx, BET_SUDO,
-                              &ctx->bet_info[BET_SUDO],
-                              ctx->bet_info[BET_ID].mod_name);
+    ret = be_process_init_sudo(ctx);
     if (ret != EOK) {
         if (ret != ENOENT) {
             DEBUG(SSSDBG_FATAL_FAILURE,
                   ("fatal error initializing data providers\n"));
-            return ret;
+            goto fail;
         }
         DEBUG(SSSDBG_MINOR_FAILURE,
               ("No SUDO module provided for [%s] !!\n", be_domain));
@@ -1964,7 +2272,7 @@ int be_process_init(TALLOC_CTX *mem_ctx,
         if (ret != ENOENT) {
             DEBUG(SSSDBG_FATAL_FAILURE,
                   ("fatal error initializing data providers\n"));
-            return ret;
+            goto fail;
         }
         DEBUG(SSSDBG_MINOR_FAILURE,
               ("No autofs module provided for [%s] !!\n", be_domain));
@@ -1974,19 +2282,19 @@ int be_process_init(TALLOC_CTX *mem_ctx,
                "from provider [%s].\n", ctx->bet_info[BET_AUTOFS].mod_name));
     }
 
-    ret = load_backend_module(ctx, BET_SESSION,
-                              &ctx->bet_info[BET_SESSION],
+    ret = load_backend_module(ctx, BET_SELINUX,
+                              &ctx->bet_info[BET_SELINUX],
                               ctx->bet_info[BET_ID].mod_name);
     if (ret != EOK) {
         if (ret != ENOENT) {
             DEBUG(SSSDBG_FATAL_FAILURE, ("fatal error initializing data providers\n"));
-            return ret;
+            goto fail;
         }
-        DEBUG(SSSDBG_CRIT_FAILURE, ("No Session module provided for [%s] !!\n",
+        DEBUG(SSSDBG_CRIT_FAILURE, ("No selinux module provided for [%s] !!\n",
                   be_domain));
     } else {
-        DEBUG(SSSDBG_TRACE_ALL, ("Session backend target successfully loaded "
-                  "from provider [%s].\n", ctx->bet_info[BET_SESSION].mod_name));
+        DEBUG(SSSDBG_TRACE_ALL, ("selinux backend target successfully loaded "
+                  "from provider [%s].\n", ctx->bet_info[BET_SELINUX].mod_name));
     }
 
     ret = load_backend_module(ctx, BET_HOSTID,
@@ -1996,7 +2304,7 @@ int be_process_init(TALLOC_CTX *mem_ctx,
         if (ret != ENOENT) {
             DEBUG(SSSDBG_FATAL_FAILURE,
                   ("fatal error initializing data providers\n"));
-            return ret;
+            goto fail;
         }
         DEBUG(SSSDBG_CRIT_FAILURE,
               ("No host info module provided for [%s] !!\n", be_domain));
@@ -2006,12 +2314,23 @@ int be_process_init(TALLOC_CTX *mem_ctx,
                ctx->bet_info[BET_HOSTID].mod_name));
     }
 
+    ret = load_backend_module(ctx, BET_SUBDOMAINS,
+                              &ctx->bet_info[BET_SUBDOMAINS], NULL);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Subdomains are not supported for [%s] !!\n", be_domain));
+    } else {
+        DEBUG(SSSDBG_TRACE_ALL, ("Get-Subdomains backend target successfully loaded "
+                  "from provider [%s].\n",
+                  ctx->bet_info[BET_SUBDOMAINS].mod_name));
+    }
+
     /* Handle SIGUSR1 to force offline behavior */
     BlockSignals(false, SIGUSR1);
     tes = tevent_add_signal(ctx->ev, ctx, SIGUSR1, 0,
                             signal_be_offline, ctx);
     if (tes == NULL) {
-        return EIO;
+        ret = EIO;
+        goto fail;
     }
 
     ret = sss_sigchld_init(ctx, ctx->ev, &ctx->sigchld_ctx);
@@ -2019,10 +2338,14 @@ int be_process_init(TALLOC_CTX *mem_ctx,
         DEBUG(SSSDBG_FATAL_FAILURE,
               ("Could not initialize sigchld context: [%s]\n",
                strerror(ret)));
-        return ret;
+        goto fail;
     }
 
     return EOK;
+
+fail:
+    talloc_free(ctx);
+    return ret;
 }
 
 int main(int argc, const char *argv[])
@@ -2099,7 +2422,7 @@ int main(int argc, const char *argv[])
         return 3;
     }
 
-    DEBUG(1, ("Backend provider (%s) started!\n", be_domain));
+    DEBUG(SSSDBG_TRACE_FUNC, ("Backend provider (%s) started!\n", be_domain));
 
     /* loop on main */
     server_loop(main_ctx);
