@@ -35,6 +35,7 @@ bool dp_pack_pam_request(DBusMessage *msg, struct pam_data *pd)
     db_ret = dbus_message_append_args(msg,
                                       DBUS_TYPE_INT32,  &(pd->cmd),
                                       DBUS_TYPE_STRING, &(pd->user),
+                                      DBUS_TYPE_STRING, &(pd->domain),
                                       DBUS_TYPE_STRING, &(pd->service),
                                       DBUS_TYPE_STRING, &(pd->tty),
                                       DBUS_TYPE_STRING, &(pd->ruser),
@@ -66,6 +67,7 @@ bool dp_unpack_pam_request(DBusMessage *msg, TALLOC_CTX *mem_ctx,
     db_ret = dbus_message_get_args(msg, dbus_error,
                                    DBUS_TYPE_INT32,  &(pd.cmd),
                                    DBUS_TYPE_STRING, &(pd.user),
+                                   DBUS_TYPE_STRING, &(pd.domain),
                                    DBUS_TYPE_STRING, &(pd.service),
                                    DBUS_TYPE_STRING, &(pd.tty),
                                    DBUS_TYPE_STRING, &(pd.ruser),
