@@ -51,8 +51,9 @@ enum bet_type {
     BET_CHPASS,
     BET_SUDO,
     BET_AUTOFS,
-    BET_SESSION,
+    BET_SELINUX,
     BET_HOSTID,
+    BET_SUBDOMAINS,
     BET_MAX
 };
 
@@ -106,6 +107,7 @@ struct be_ctx {
     struct be_cb *online_cb_list;
     bool run_online_cb;
     struct be_cb *offline_cb_list;
+    struct be_cb *reconnect_cb_list;
 
     struct be_offline_status offstat;
 
@@ -130,7 +132,8 @@ struct bet_ops {
     be_req_fn_t finalize;
 };
 
-#define MAX_BE_REQ_RESTARTS 2
+#define REQ_PHASE_ACCESS 0
+#define REQ_PHASE_SELINUX 1
 
 struct be_req {
     struct be_client *becli;
@@ -140,7 +143,14 @@ struct be_req {
     be_async_callback_t fn;
     void *pvt;
 
-    int restarts;
+    /* This is utilized in access provider
+     * request handling to indicate if access or
+     * selinux provider is calling the callback.
+     */
+    int phase;
+
+    struct sss_domain_info *domain;
+    struct sysdb_ctx *sysdb;
 };
 
 struct be_acct_req {
@@ -149,22 +159,40 @@ struct be_acct_req {
     int filter_type;
     char *filter_value;
     char *extra_value;
+    char *domain;
 };
 
 struct be_sudo_req {
     uint32_t type;
-    char *username;
-    uid_t uid;
-    char **groups;
+    char **rules;
 };
 
 struct be_autofs_req {
     char *mapname;
 };
 
+struct be_subdom_req {
+    bool force;
+    char *domain_hint;
+};
+
+struct be_host_req {
+    uint32_t type;
+    int filter_type;
+    char *name;
+    char *alias;
+};
+
 bool be_is_offline(struct be_ctx *ctx);
 void be_mark_offline(struct be_ctx *ctx);
 
+int be_add_reconnect_cb(TALLOC_CTX *mem_ctx,
+                        struct be_ctx *ctx,
+                        be_callback_t cb,
+                        void *pvt,
+                        struct be_cb **reconnect_cb);
+void be_run_reconnect_cb(struct be_ctx *be);
+
 int be_add_online_cb(TALLOC_CTX *mem_ctx,
                      struct be_ctx *ctx,
                      be_callback_t cb,
@@ -190,7 +218,8 @@ typedef void (be_svc_callback_fn_t)(void *, struct fo_server *);
 
 int be_init_failover(struct be_ctx *ctx);
 int be_fo_is_srv_identifier(const char *server);
-int be_fo_add_service(struct be_ctx *ctx, const char *service_name);
+int be_fo_add_service(struct be_ctx *ctx, const char *service_name,
+                      datacmp_fn user_data_cmp);
 int be_fo_service_add_callback(TALLOC_CTX *memctx,
                                struct be_ctx *ctx, const char *service_name,
                                be_svc_callback_fn_t *fn, void *private_data);
@@ -202,7 +231,8 @@ int be_fo_add_srv_server(struct be_ctx *ctx,
                          enum be_fo_protocol proto,
                          bool proto_fallback, void *user_data);
 int be_fo_add_server(struct be_ctx *ctx, const char *service_name,
-                     const char *server, int port, void *user_data);
+                     const char *server, int port, void *user_data,
+                     bool primary);
 
 struct tevent_req *be_resolve_server_send(TALLOC_CTX *memctx,
                                           struct tevent_context *ev,
