@@ -66,15 +66,22 @@ struct fo_service {
     struct fo_server *active_server;
     struct fo_server *last_tried_server;
     struct fo_server *server_list;
+
+    /* Function pointed by user_data_cmp returns 0 if user_data is equal
+     * or nonzero value if not. Set to NULL if no user data comparison
+     * is needed in fail over duplicate servers detection.
+     */
+    datacmp_fn user_data_cmp;
 };
 
 struct fo_server {
     struct fo_server *prev;
     struct fo_server *next;
 
+    bool primary;
     void *user_data;
     int port;
-    int port_status;
+    enum port_status port_status;
     struct srv_data *srv_data;
     struct fo_service *service;
     struct timeval last_status_change;
@@ -92,7 +99,7 @@ struct server_common {
     char *name;
     struct resolv_hostent *rhostent;
     struct resolve_service_request *request_list;
-    int server_status;
+    enum server_status server_status;
     struct timeval last_status_change;
 };
 
@@ -143,8 +150,8 @@ fo_context_init(TALLOC_CTX *mem_ctx, struct fo_options *opts)
     ctx->opts->family_order  = opts->family_order;
     ctx->opts->service_resolv_timeout = opts->service_resolv_timeout;
 
-    DEBUG(3, ("Created new fail over context, retry timeout is %d\n",
-              ctx->opts->retry_timeout));
+    DEBUG(SSSDBG_TRACE_FUNC, ("Created new fail over context, retry timeout is %d\n",
+                              ctx->opts->retry_timeout));
     return ctx;
 }
 
@@ -392,12 +399,13 @@ service_destructor(struct fo_service *service)
 
 int
 fo_new_service(struct fo_ctx *ctx, const char *name,
+               datacmp_fn user_data_cmp,
                struct fo_service **_service)
 {
     struct fo_service *service;
     int ret;
 
-    DEBUG(3, ("Creating new service '%s'\n", name));
+    DEBUG(SSSDBG_TRACE_FUNC, ("Creating new service '%s'\n", name));
     ret = fo_get_service(ctx, name, &service);
     if (ret == EOK) {
         DEBUG(5, ("Service '%s' already exists\n", name));
@@ -419,6 +427,8 @@ fo_new_service(struct fo_ctx *ctx, const char *name,
         return ENOMEM;
     }
 
+    service->user_data_cmp = user_data_cmp;
+
     service->ctx = ctx;
     DLIST_ADD(ctx->service_list, service);
 
@@ -514,13 +524,19 @@ fo_add_srv_server(struct fo_service *service, const char *srv,
 {
     struct fo_server *server;
 
-    DEBUG(3, ("Adding new SRV server in domain '%s', to service '%s' using %s\n",
-              discovery_domain ? discovery_domain : "unknown",
-              service->name, proto));
+    DEBUG(SSSDBG_TRACE_FUNC,
+          ("Adding new SRV server to service '%s' using '%s'.\n",
+           service->name, proto));
 
     DLIST_FOR_EACH(server, service->server_list) {
-        if (server->user_data != user_data)
-            continue;
+        /* Compare user data only if user_data_cmp and both arguments
+         * are not NULL.
+         */
+        if (server->service->user_data_cmp && user_data && server->user_data) {
+            if (server->service->user_data_cmp(server->user_data, user_data)) {
+                continue;
+            }
+        }
 
         if (fo_is_srv_lookup(server)) {
             if (((discovery_domain == NULL &&
@@ -541,6 +557,7 @@ fo_add_srv_server(struct fo_service *service, const char *srv,
     server->user_data = user_data;
     server->service = service;
     server->port_status = DEFAULT_PORT_STATUS;
+    server->primary = true; /* SRV servers are never back up */
 
     /* add the SRV-specific data */
     server->srv_data = talloc_zero(service, struct srv_data);
@@ -577,7 +594,7 @@ fo_add_srv_server(struct fo_service *service, const char *srv,
 
 static struct fo_server *
 create_fo_server(struct fo_service *service, const char *name,
-                 int port, void *user_data)
+                 int port, void *user_data, bool primary)
 {
     struct fo_server *server;
     int ret;
@@ -590,6 +607,7 @@ create_fo_server(struct fo_service *service, const char *name,
     server->user_data = user_data;
     server->service = service;
     server->port_status = DEFAULT_PORT_STATUS;
+    server->primary = primary;
 
     if (name != NULL) {
         ret = get_server_common(server, service->ctx, name, &server->common);
@@ -621,26 +639,51 @@ fo_get_server_count(struct fo_service *service)
     return count;
 }
 
+static bool fo_server_match(struct fo_server *server,
+                           const char *name,
+                           int port,
+                           void *user_data)
+{
+    if (server->port != port) {
+        return false;
+    }
+
+    /* Compare user data only if user_data_cmp and both arguments
+     * are not NULL.
+     */
+    if (server->service->user_data_cmp && server->user_data && user_data) {
+        if (server->service->user_data_cmp(server->user_data, user_data)) {
+            return false;
+        }
+    }
+
+    if (name == NULL && server->common == NULL) {
+        return true;
+    }
+
+    if (name != NULL && server->common != NULL) {
+        if (!strcasecmp(name, server->common->name))
+            return true;
+    }
+
+    return false;
+}
+
 int
 fo_add_server(struct fo_service *service, const char *name, int port,
-              void *user_data)
+              void *user_data, bool primary)
 {
     struct fo_server *server;
 
     DEBUG(3, ("Adding new server '%s', to service '%s'\n",
               name ? name : "(no name)", service->name));
     DLIST_FOR_EACH(server, service->server_list) {
-        if (server->port != port || server->user_data != user_data)
-            continue;
-        if (name == NULL && server->common == NULL) {
+        if (fo_server_match(server, name, port, user_data)) {
             return EEXIST;
-        } else if (name != NULL && server->common != NULL) {
-            if (!strcasecmp(name, server->common->name))
-                return EEXIST;
         }
     }
 
-    server = create_fo_server(service, name, port, user_data);
+    server = create_fo_server(service, name, port, user_data, primary);
     if (!server) {
         return ENOMEM;
     }
@@ -658,7 +701,7 @@ get_first_server_entity(struct fo_service *service, struct fo_server **_server)
     /* If we already have a working server, use that one. */
     server = service->active_server;
     if (server != NULL) {
-        if (service_works(server)) {
+        if (service_works(server) && fo_is_server_primary(server)) {
             goto done;
         }
         service->active_server = NULL;
@@ -668,17 +711,32 @@ get_first_server_entity(struct fo_service *service, struct fo_server **_server)
      * Otherwise iterate through the server list.
      */
 
-    /* First, try servers after the last one we tried. */
-    if (service->last_tried_server != NULL) {
+    /* First, try primary servers after the last one we tried.
+     * (only if the last one was primary as well)
+     */
+    if (service->last_tried_server != NULL &&
+        service->last_tried_server->primary) {
+        if (service->last_tried_server->port_status == PORT_NEUTRAL &&
+            server_works(service->last_tried_server)) {
+            server = service->last_tried_server;
+            goto done;
+        }
+
         DLIST_FOR_EACH(server, service->last_tried_server->next) {
+            /* Go only through primary servers */
+            if (!server->primary) continue;
+
             if (service_works(server)) {
                 goto done;
             }
         }
     }
 
-    /* If none were found, try at the start. */
+    /* If none were found, try at the start, primary first */
     DLIST_FOR_EACH(server, service->server_list) {
+        /* First iterate only over primary servers */
+        if (!server->primary) continue;
+
         if (service_works(server)) {
             goto done;
         }
@@ -687,6 +745,15 @@ get_first_server_entity(struct fo_service *service, struct fo_server **_server)
         }
     }
 
+    DLIST_FOR_EACH(server, service->server_list) {
+        /* Now iterate only over backup servers */
+        if (server->primary) continue;
+
+        if (service_works(server)) {
+            goto done;
+        }
+    }
+
     service->last_tried_server = NULL;
     return ENOENT;
 
@@ -727,6 +794,8 @@ set_lookup_hook(struct fo_server *server, struct tevent_req *req)
     return EOK;
 }
 
+
+
 /*******************************************************************
  * Get server to connect to.                                       *
  *******************************************************************/
@@ -740,7 +809,6 @@ struct resolve_service_state {
     struct fo_ctx *fo_ctx;
 };
 
-
 static errno_t fo_resolve_service_activate_timeout(struct tevent_req *req,
             struct tevent_context *ev, const unsigned long timeout_seconds);
 static void fo_resolve_service_cont(struct tevent_req *subreq);
@@ -938,6 +1006,15 @@ fo_resolve_service_done(struct tevent_req *subreq)
         DEBUG(1, ("Failed to resolve server '%s': %s\n",
                   common->name,
                   resolv_strerror(resolv_status)));
+        /* If the resolver failed to resolve a hostname but did not
+         * encounter an error, tell the caller to retry another server.
+         *
+         * If there are no more servers to try, the next request would
+         * just shortcut with ENOENT.
+         */
+        if (ret == ENOENT) {
+            ret = EAGAIN;
+        }
         set_server_common_status(common, SERVER_NOT_WORKING);
     } else {
         set_server_common_status(common, SERVER_NAME_RESOLVED);
@@ -1018,18 +1095,25 @@ resolve_srv_send(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
     state->ev = ev;
     state->resolv = resolv;
     state->fo_ctx = ctx;
-    state->meta = server;
+    state->meta = server->srv_data->meta;
 
     status = get_srv_data_status(server->srv_data);
-    DEBUG(6, ("The status of SRV lookup is %s\n",
-              str_srv_data_status(status)));
+    DEBUG(SSSDBG_FUNC_DATA, ("The status of SRV lookup is %s\n",
+          str_srv_data_status(status)));
     switch(status) {
     case SRV_EXPIRED: /* Need a refresh */
         state->meta = collapse_srv_lookup(server);
-        /* FALLTHROUGH */
+        /* FALLTHROUGH.
+         * "server" might be invalid now if the SRV
+         * query collapsed
+         * */
     case SRV_NEUTRAL: /* Request SRV lookup */
-        if (server->srv_data->dns_domain == NULL) {
+        if (state->meta->srv_data->dns_domain == NULL) {
             /* we need to look up our DNS domain first */
+            DEBUG(SSSDBG_TRACE_FUNC,
+                  ("SRV resolution of service '%s'. "
+                   "dns_discovery_domain not specified. Need to look it up.\n",
+                   server->service->name));
             subreq = resolve_get_domain_send(state, ev, ctx, resolv);
             if (subreq == NULL) {
                 ret = ENOMEM;
@@ -1039,6 +1123,10 @@ resolve_srv_send(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
             break;
         }
         /* we know the DNS domain, just do the lookup */
+        DEBUG(SSSDBG_TRACE_FUNC,
+              ("SRV resolution of service '%s'. "
+               "Will use DNS discovery domain '%s'\n",
+               server->service->name, state->meta->srv_data->dns_domain));
         resolve_srv_cont(req);
         break;
     case SRV_RESOLVE_ERROR: /* query could not be resolved but don't retry yet */
@@ -1171,7 +1259,8 @@ resolve_srv_done(struct tevent_req *subreq)
 
     for (reply = reply_list; reply; reply = reply->next) {
         server = create_fo_server(state->service, reply->host,
-                                  reply->port, state->meta->user_data);
+                                  reply->port, state->meta->user_data,
+                                  true);
         if (!server) {
             ret = ENOMEM;
             goto fail;
@@ -1451,6 +1540,12 @@ fo_get_server_hostent(struct fo_server *server)
     return server->common->rhostent;
 }
 
+bool
+fo_is_server_primary(struct fo_server *server)
+{
+    return server->primary;
+}
+
 time_t
 fo_get_server_hostname_last_change(struct fo_server *server)
 {
@@ -1467,10 +1562,11 @@ void fo_reset_services(struct fo_ctx *fo_ctx)
 
     DLIST_FOR_EACH(service, fo_ctx->service_list) {
         DLIST_FOR_EACH(server, service->server_list) {
-            fo_set_server_status(server, SERVER_NAME_NOT_RESOLVED);
-            fo_set_port_status(server, PORT_NEUTRAL);
             if (server->srv_data != NULL) {
                 set_srv_data_status(server->srv_data, SRV_NEUTRAL);
+            } else {
+                fo_set_server_status(server, SERVER_NAME_NOT_RESOLVED);
+                fo_set_port_status(server, PORT_NEUTRAL);
             }
         }
     }
