@@ -89,13 +89,20 @@ struct fo_options {
 struct fo_ctx *fo_context_init(TALLOC_CTX *mem_ctx,
                                struct fo_options *opts);
 
+typedef int (*datacmp_fn)(void*, void*);
+
 /*
  * Create a new service structure for 'ctx', saving it to the location pointed
  * to by '_service'. The needed memory will be allocated from 'ctx'.
  * Service name will be set to 'name'.
+ *
+ * Function pointed by user_data_cmp returns 0 if user_data is equal
+ * or nonzero value if not. Set to NULL if no user data comparison
+ * is needed in fail over duplicate servers detection.
  */
 int fo_new_service(struct fo_ctx *ctx,
                    const char *name,
+                   datacmp_fn user_data_cmp,
                    struct fo_service **_service);
 
 /*
@@ -116,9 +123,8 @@ int fo_get_server_count(struct fo_service *service);
  * connection. If 'name' is NULL, no server resolution will be done.
  */
 int fo_add_server(struct fo_service *service,
-                  const char *name,
-                  int port,
-                  void *user_data);
+                  const char *name, int port,
+                  void *user_data, bool primary);
 
 
 int fo_add_srv_server(struct fo_service *service,
@@ -180,6 +186,8 @@ const char *fo_get_server_str_name(struct fo_server *server);
 
 struct resolv_hostent *fo_get_server_hostent(struct fo_server *server);
 
+bool fo_is_server_primary(struct fo_server *server);
+
 time_t fo_get_server_hostname_last_change(struct fo_server *server);
 
 int fo_is_srv_lookup(struct fo_server *s);
