@@ -85,6 +85,16 @@ void ipa_access_handler(struct be_req *be_req)
                               be_req->be_ctx->bet_info[BET_ACCESS].pvt_bet_data,
                               struct ipa_access_ctx);
 
+    if (strcasecmp(pd->domain, be_req->be_ctx->domain->name) != 0) {
+        be_req->domain = new_subdomain(be_req, be_req->be_ctx->domain, pd->domain, NULL, NULL);
+        if (be_req->domain == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, ("new_subdomain failed.\n"));
+            be_req->fn(be_req, DP_ERR_FATAL, PAM_SYSTEM_ERR, NULL);
+            return;
+        }
+        be_req->sysdb = be_req->domain->sysdb;
+    }
+
     /* First, verify that this account isn't locked.
      * We need to do this in case the auth phase was
      * skipped (such as during GSSAPI single-sign-on
@@ -92,7 +102,7 @@ void ipa_access_handler(struct be_req *be_req)
      */
     req = sdap_access_send(be_req,
                            be_req->be_ctx->ev,
-                           be_req->be_ctx,
+                           be_req,
                            ipa_access_ctx->sdap_access_ctx,
                            pd);
     if (!req) {
@@ -116,6 +126,7 @@ static void ipa_hbac_check(struct tevent_req *req)
     pd = talloc_get_type(be_req->req_data, struct pam_data);
 
     ret = sdap_access_recv(req, &pam_status);
+    talloc_zfree(req);
     if (ret != EOK) goto fail;
 
     switch(pam_status) {
@@ -328,20 +339,6 @@ static int hbac_get_host_info_step(struct hbac_ctx *hbac_ctx)
     const char *hostname;
     struct tevent_req *req;
 
-    hbac_ctx->host_attrs = talloc_array(hbac_ctx, const char *, 8);
-    if (hbac_ctx->host_attrs == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to allocate host attribute list.\n"));
-        return ENOMEM;
-    }
-    hbac_ctx->host_attrs[0] = "objectClass";
-    hbac_ctx->host_attrs[1] = IPA_HOST_SERVERHOSTNAME;
-    hbac_ctx->host_attrs[2] = IPA_HOST_FQDN;
-    hbac_ctx->host_attrs[3] = IPA_UNIQUE_ID;
-    hbac_ctx->host_attrs[4] = IPA_MEMBER;
-    hbac_ctx->host_attrs[5] = IPA_MEMBEROF;
-    hbac_ctx->host_attrs[6] = IPA_CN;
-    hbac_ctx->host_attrs[7] = NULL;
-
     if (dp_opt_get_bool(hbac_ctx->ipa_options, IPA_HBAC_SUPPORT_SRCHOST)) {
         /* Support srchost
          * -> we don't want any particular host,
@@ -357,7 +354,9 @@ static int hbac_get_host_info_step(struct hbac_ctx *hbac_ctx)
                              hbac_ctx_sysdb(hbac_ctx),
                              sdap_id_op_handle(hbac_ctx->sdap_op),
                              hbac_ctx_sdap_id_ctx(hbac_ctx)->opts,
-                             hostname, hbac_ctx->host_attrs, NULL, 0, true,
+                             hostname,
+                             hbac_ctx->access_ctx->host_map,
+                             hbac_ctx->access_ctx->hostgroup_map,
                              hbac_ctx->access_ctx->host_search_bases);
     if (req == NULL) {
         DEBUG(1, ("Could not get host info\n"));
@@ -431,7 +430,7 @@ static void hbac_get_rule_info_step(struct tevent_req *req)
 
     for (i = 0; i < hbac_ctx->host_count; i++) {
         ret = sysdb_attrs_get_string(hbac_ctx->hosts[i],
-                                     IPA_HOST_FQDN,
+                                     SYSDB_FQDN,
                                      &hostname);
         if (ret != EOK) {
             DEBUG(1, ("Could not locate IPA host\n"));
@@ -502,7 +501,6 @@ static void hbac_sysdb_save(struct tevent_req *req)
          * are also denied.
          */
         base_dn = sysdb_custom_subtree_dn(sysdb, tmp_ctx,
-                                          domain->name,
                                           HBAC_RULES_SUBDIR);
         if (base_dn == NULL) {
             talloc_free(tmp_ctx);
@@ -536,9 +534,9 @@ static void hbac_sysdb_save(struct tevent_req *req)
 
     /* Save the hosts */
     ret = ipa_hbac_sysdb_save(sysdb, domain,
-                              HBAC_HOSTS_SUBDIR, IPA_HOST_FQDN,
+                              HBAC_HOSTS_SUBDIR, SYSDB_FQDN,
                               hbac_ctx->host_count, hbac_ctx->hosts,
-                              HBAC_HOSTGROUPS_SUBDIR, IPA_CN,
+                              HBAC_HOSTGROUPS_SUBDIR, SYSDB_NAME,
                               hbac_ctx->hostgroup_count,
                               hbac_ctx->hostgroups);
     if (ret != EOK) {
@@ -573,9 +571,10 @@ static void hbac_sysdb_save(struct tevent_req *req)
 
     ret = sysdb_transaction_commit(sysdb);
     if (ret != EOK) {
-        DEBUG(0, ("Failed to commit transaction\n"));
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
         goto fail;
     }
+    in_transaction = false;
 
     /* We don't need the rule data any longer,
      * the rest of the processing relies on
@@ -601,9 +600,6 @@ fail:
     ipa_access_reply(hbac_ctx, PAM_SYSTEM_ERR);
 }
 
-static errno_t hbac_get_cached_rules(TALLOC_CTX *mem_ctx,
-                                     struct hbac_ctx *hbac_ctx);
-
 void ipa_hbac_evaluate_rules(struct hbac_ctx *hbac_ctx)
 {
     errno_t ret;
@@ -613,7 +609,8 @@ void ipa_hbac_evaluate_rules(struct hbac_ctx *hbac_ctx)
     struct hbac_info *info;
 
     /* Get HBAC rules from the sysdb */
-    ret = hbac_get_cached_rules(hbac_ctx, hbac_ctx);
+    ret = hbac_get_cached_rules(hbac_ctx, hbac_ctx_sysdb(hbac_ctx),
+                                &hbac_ctx->rule_count, &hbac_ctx->rules);
     if (ret != EOK) {
         DEBUG(1, ("Could not retrieve rules from the cache\n"));
         ipa_access_reply(hbac_ctx, PAM_SYSTEM_ERR);
@@ -656,17 +653,20 @@ void ipa_hbac_evaluate_rules(struct hbac_ctx *hbac_ctx)
     ipa_access_reply(hbac_ctx, PAM_PERM_DENIED);
 }
 
-static errno_t hbac_get_cached_rules(TALLOC_CTX *mem_ctx,
-                                     struct hbac_ctx *hbac_ctx)
+errno_t hbac_get_cached_rules(TALLOC_CTX *mem_ctx,
+                              struct sysdb_ctx *sysdb,
+                              size_t *_rule_count,
+                              struct sysdb_attrs ***_rules)
 {
     errno_t ret;
-    struct sysdb_ctx *sysdb = hbac_ctx_sysdb(hbac_ctx);
-    size_t count;
     struct ldb_message **msgs;
+    struct sysdb_attrs **rules;
+    size_t rule_count;
     TALLOC_CTX *tmp_ctx;
     char *filter;
     const char *attrs[] = { OBJECTCLASS,
                             IPA_CN,
+                            SYSDB_ORIG_DN,
                             IPA_UNIQUE_ID,
                             IPA_ENABLED_FLAG,
                             IPA_ACCESS_RULE_TYPE,
@@ -681,7 +681,7 @@ static errno_t hbac_get_cached_rules(TALLOC_CTX *mem_ctx,
                             IPA_HOST_CATEGORY,
                             NULL };
 
-    tmp_ctx = talloc_new(hbac_ctx);
+    tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) return ENOMEM;
 
     filter = talloc_asprintf(tmp_ctx, "(objectClass=%s)", IPA_HBAC_RULE);
@@ -690,22 +690,24 @@ static errno_t hbac_get_cached_rules(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    ret = sysdb_search_custom(mem_ctx, sysdb, filter,
+    ret = sysdb_search_custom(tmp_ctx, sysdb, filter,
                               HBAC_RULES_SUBDIR, attrs,
-                              &count, &msgs);
+                              &rule_count, &msgs);
     if (ret != EOK && ret != ENOENT) {
         DEBUG(1, ("Error looking up HBAC rules"));
         goto done;
     } if (ret == ENOENT) {
-       count = 0;
+       rule_count = 0;
     }
 
-    ret = sysdb_msg2attrs(mem_ctx, count, msgs, &hbac_ctx->rules);
+    ret = sysdb_msg2attrs(tmp_ctx, rule_count, msgs, &rules);
     if (ret != EOK) {
         DEBUG(1, ("Could not convert ldb message to sysdb_attrs\n"));
         goto done;
     }
-    hbac_ctx->rule_count = count;
+
+    if (_rules) *_rules = talloc_steal(mem_ctx, rules);
+    if (_rule_count) *_rule_count = rule_count;
 
     ret = EOK;
 done:
