@@ -44,7 +44,7 @@ ipa_hbac_save_list(struct sysdb_ctx *sysdb, bool delete_subdir,
     }
 
     if (delete_subdir) {
-        base_dn = sysdb_custom_subtree_dn(sysdb, tmp_ctx, domain->name, subdir);
+        base_dn = sysdb_custom_subtree_dn(sysdb, tmp_ctx, subdir);
         if (base_dn == NULL) {
             ret = ENOMEM;
             goto done;
@@ -111,7 +111,10 @@ ipa_hbac_sysdb_save(struct sysdb_ctx *sysdb, struct sss_domain_info *domain,
 
     /* Save the entries and groups to the cache */
     ret = sysdb_transaction_start(sysdb);
-    if (ret != EOK) return ret;
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
+        goto done;
+    };
     in_transaction = true;
 
     /* First, save the specific entries */
@@ -143,7 +146,10 @@ ipa_hbac_sysdb_save(struct sysdb_ctx *sysdb, struct sss_domain_info *domain,
     }
 
     ret = sysdb_transaction_commit(sysdb);
-    if (ret != EOK) goto done;
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
+        goto done;
+    }
     in_transaction = false;
 
 done:
@@ -611,8 +617,7 @@ hbac_eval_service_element(TALLOC_CTX *mem_ctx,
 
     svc->name = servicename;
 
-    svc_dn = sysdb_custom_dn(sysdb, tmp_ctx, domain->name,
-                             svc->name, HBAC_SERVICES_SUBDIR);
+    svc_dn = sysdb_custom_dn(sysdb, tmp_ctx, svc->name, HBAC_SERVICES_SUBDIR);
     if (svc_dn == NULL) {
         ret = ENOMEM;
         goto done;
@@ -719,8 +724,7 @@ hbac_eval_host_element(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    host_dn = sysdb_custom_dn(sysdb, tmp_ctx, domain->name,
-                              host->name, HBAC_HOSTS_SUBDIR);
+    host_dn = sysdb_custom_dn(sysdb, tmp_ctx, host->name, HBAC_HOSTS_SUBDIR);
     if (host_dn == NULL) {
         ret = ENOMEM;
         goto done;
