@@ -56,16 +56,16 @@ static errno_t unpack_buffer(uint8_t *buf, size_t size,
     size_t p = 0;
     uint32_t len;
 
-    DEBUG(7, ("total buffer size: %d\n", size));
+    DEBUG(SSSDBG_TRACE_LIBS, ("total buffer size: %d\n", size));
 
     /* realm_str size and length */
     SAFEALIGN_COPY_UINT32_CHECK(&len, buf + p, size, &p);
 
-    DEBUG(7, ("realm_str size: %d\n", len));
+    DEBUG(SSSDBG_TRACE_LIBS, ("realm_str size: %d\n", len));
     if (len) {
         if ((p + len ) > size) return EINVAL;
         ibuf->realm_str = talloc_strndup(ibuf, (char *)(buf + p), len);
-        DEBUG(7, ("got realm_str: %s\n", ibuf->realm_str));
+        DEBUG(SSSDBG_TRACE_LIBS, ("got realm_str: %s\n", ibuf->realm_str));
         if (ibuf->realm_str == NULL) return ENOMEM;
         p += len;
     }
@@ -73,11 +73,11 @@ static errno_t unpack_buffer(uint8_t *buf, size_t size,
     /* princ_str size and length */
     SAFEALIGN_COPY_UINT32_CHECK(&len, buf + p, size, &p);
 
-    DEBUG(7, ("princ_str size: %d\n", len));
+    DEBUG(SSSDBG_TRACE_LIBS, ("princ_str size: %d\n", len));
     if (len) {
         if ((p + len ) > size) return EINVAL;
         ibuf->princ_str = talloc_strndup(ibuf, (char *)(buf + p), len);
-        DEBUG(7, ("got princ_str: %s\n", ibuf->princ_str));
+        DEBUG(SSSDBG_TRACE_LIBS, ("got princ_str: %s\n", ibuf->princ_str));
         if (ibuf->princ_str == NULL) return ENOMEM;
         p += len;
     }
@@ -85,18 +85,18 @@ static errno_t unpack_buffer(uint8_t *buf, size_t size,
     /* keytab_name size and length */
     SAFEALIGN_COPY_UINT32_CHECK(&len, buf + p, size, &p);
 
-    DEBUG(7, ("keytab_name size: %d\n", len));
+    DEBUG(SSSDBG_TRACE_LIBS, ("keytab_name size: %d\n", len));
     if (len) {
         if ((p + len ) > size) return EINVAL;
         ibuf->keytab_name = talloc_strndup(ibuf, (char *)(buf + p), len);
-        DEBUG(7, ("got keytab_name: %s\n", ibuf->keytab_name));
+        DEBUG(SSSDBG_TRACE_LIBS, ("got keytab_name: %s\n", ibuf->keytab_name));
         if (ibuf->keytab_name == NULL) return ENOMEM;
         p += len;
     }
 
     /* ticket lifetime */
     SAFEALIGN_COPY_INT32_CHECK(&ibuf->lifetime, buf + p, size, &p);
-    DEBUG(7, ("lifetime: %d\n", ibuf->lifetime));
+    DEBUG(SSSDBG_TRACE_LIBS, ("lifetime: %d\n", ibuf->lifetime));
 
     return EOK;
 }
@@ -111,11 +111,17 @@ static int pack_buffer(struct response *r, int result, krb5_error_code krberr,
     r->size = 2 * sizeof(uint32_t) + sizeof(krb5_error_code) +
               len + sizeof(time_t);
 
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("response size: %d\n",r->size));
+
     r->buf = talloc_array(r, uint8_t, r->size);
     if(!r->buf) {
         return ENOMEM;
     }
 
+    DEBUG(SSSDBG_TRACE_LIBS,
+          ("result [%d] krberr [%d] msgsize [%d] msg [%s]\n",
+           result, krberr, len, msg));
+
     /* result */
     SAFEALIGN_SET_UINT32(&r->buf[p], result, &p);
 
@@ -155,22 +161,21 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
     krb5_get_init_creds_opt options;
     krb5_error_code krberr;
     krb5_timestamp kdc_time_offset;
-    krb5_enctype *etype_list;
-    int n_etype_list;
     int canonicalize = 0;
     int kdc_time_offset_usec;
     int ret;
 
     krberr = krb5_init_context(&context);
     if (krberr) {
-        DEBUG(2, ("Failed to init kerberos context\n"));
+        DEBUG(SSSDBG_OP_FAILURE, ("Failed to init kerberos context\n"));
         return krberr;
     }
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("Kerberos context initialized\n"));
 
     if (!realm_str) {
         krberr = krb5_get_default_realm(context, &default_realm);
         if (krberr) {
-            DEBUG(2, ("Failed to get default realm name: %s\n",
+            DEBUG(SSSDBG_OP_FAILURE, ("Failed to get default realm name: %s\n",
                       sss_krb5_get_error_message(context, krberr)));
             goto done;
         }
@@ -181,7 +186,6 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
             krberr = KRB5KRB_ERR_GENERIC;
             goto done;
         }
-
     } else {
         realm_name = talloc_strdup(memctx, realm_str);
         if (!realm_name) {
@@ -190,6 +194,8 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
         }
     }
 
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("got realm_name: [%s]\n", realm_name));
+
     if (princ_str) {
         if (!strchr(princ_str, '@')) {
             full_princ = talloc_asprintf(memctx, "%s@%s",
@@ -207,15 +213,17 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
         }
         hostname[511] = '\0';
 
+        DEBUG(SSSDBG_TRACE_LIBS, ("got hostname: [%s]\n", hostname));
+
         ret = select_principal_from_keytab(memctx, hostname, realm_name,
-                                           keytab_name, &full_princ, NULL, NULL);
+                keytab_name, &full_princ, NULL, NULL);
         if (ret) goto done;
     }
     if (!full_princ) {
         krberr = KRB5KRB_ERR_GENERIC;
         goto done;
     }
-    DEBUG(4, ("Principal name is: [%s]\n", full_princ));
+    DEBUG(SSSDBG_CONF_SETTINGS, ("Principal name is: [%s]\n", full_princ));
 
     krberr = krb5_parse_name(context, full_princ, &kprinc);
     if (krberr) {
@@ -229,6 +237,7 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
     } else {
         krberr = krb5_kt_default(context, &keytab);
     }
+    DEBUG(SSSDBG_CONF_SETTINGS, ("Using keytab [%s]\n", KEYTAB_CLEAN_NAME));
     if (krberr) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               ("Failed to read keytab file [%s]: %s\n",
@@ -240,7 +249,8 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
     /* Verify the keytab */
     ret = sss_krb5_verify_keytab_ex(full_princ, keytab_name, context, keytab);
     if (ret) {
-        DEBUG(2, ("Unable to verify principal is present in the keytab\n"));
+        DEBUG(SSSDBG_OP_FAILURE,
+                ("Unable to verify principal is present in the keytab\n"));
         krberr = KRB5_KT_IOERR;
         goto done;
     }
@@ -250,10 +260,11 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
         krberr = KRB5KRB_ERR_GENERIC;
         goto done;
     }
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("keytab ccname: [%s]\n", ccname));
 
     krberr = krb5_cc_resolve(context, ccname, &ccache);
     if (krberr) {
-        DEBUG(2, ("Failed to set cache name: %s\n",
+        DEBUG(SSSDBG_OP_FAILURE, ("Failed to set cache name: %s\n",
                   sss_krb5_get_error_message(context, krberr)));
         goto done;
     }
@@ -268,26 +279,13 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
 
     tmp_str = getenv("KRB5_CANONICALIZE");
     if (tmp_str != NULL && strcasecmp(tmp_str, "true") == 0) {
+        DEBUG(SSSDBG_CONF_SETTINGS, ("Will canonicalize principals\n"));
         canonicalize = 1;
     }
     sss_krb5_get_init_creds_opt_set_canonicalize(&options, canonicalize);
 
-    krberr = sss_krb5_read_etypes_for_keytab(memctx, context, keytab, kprinc,
-                                             &etype_list, &n_etype_list);
-    if (krberr) {
-        DEBUG(SSSDBG_MINOR_FAILURE, ("Failed to load etypes from keytab: %s\n",
-                                     sss_krb5_get_error_message(context,
-                                                                krberr)));
-    } else if (n_etype_list > 0) {
-        krb5_get_init_creds_opt_set_etype_list(&options, etype_list,
-                                               n_etype_list);
-        DEBUG(SSSDBG_FUNC_DATA, ("Loaded %d enctypes from keytab for %s\n",
-                                 n_etype_list, full_princ));
-    }
-
     krberr = krb5_get_init_creds_keytab(context, &my_creds, kprinc,
                                         keytab, 0, NULL, &options);
-
     if (krberr) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               ("Failed to init credentials: %s\n",
@@ -299,26 +297,29 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
                 sss_krb5_get_error_message(context, krberr));
         goto done;
     }
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("credentials initialized\n"));
 
     /* Use updated principal if changed due to canonicalization. */
     krberr = krb5_cc_initialize(context, ccache, my_creds.client);
     if (krberr) {
-        DEBUG(2, ("Failed to init ccache: %s\n",
+        DEBUG(SSSDBG_OP_FAILURE, ("Failed to init ccache: %s\n",
                   sss_krb5_get_error_message(context, krberr)));
         goto done;
     }
 
     krberr = krb5_cc_store_cred(context, ccache, &my_creds);
     if (krberr) {
-        DEBUG(2, ("Failed to store creds: %s\n",
+        DEBUG(SSSDBG_OP_FAILURE, ("Failed to store creds: %s\n",
                   sss_krb5_get_error_message(context, krberr)));
         goto done;
     }
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("credentials stored\n"));
 
 #ifdef HAVE_KRB5_GET_TIME_OFFSETS
-    krberr = krb5_get_time_offsets(context, &kdc_time_offset, &kdc_time_offset_usec);
+    krberr = krb5_get_time_offsets(context, &kdc_time_offset,
+            &kdc_time_offset_usec);
     if (krberr) {
-        DEBUG(2, ("Failed to get KDC time offset: %s\n",
+        DEBUG(SSSDBG_OP_FAILURE, ("Failed to get KDC time offset: %s\n",
                   sss_krb5_get_error_message(context, krberr)));
         kdc_time_offset = 0;
     } else {
@@ -326,6 +327,7 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
             kdc_time_offset++;
         }
     }
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("Got KDC time offset\n"));
 #else
     /* If we don't have this function, just assume no offset */
     kdc_time_offset = 0;
@@ -358,12 +360,15 @@ static int prepare_response(TALLOC_CTX *mem_ctx,
     r->buf = NULL;
     r->size = 0;
 
+    DEBUG(SSSDBG_TRACE_FUNC, ("Building response for result [%d]\n", kerr));
+
     if (kerr == 0) {
         ret = pack_buffer(r, EOK, kerr, ccname, expire_time);
     } else {
         krb5_msg = sss_krb5_get_error_message(krb5_error_ctx, kerr);
         if (krb5_msg == NULL) {
-            DEBUG(1, ("sss_krb5_get_error_message failed.\n"));
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                    ("sss_krb5_get_error_message failed.\n"));
             return ENOMEM;
         }
 
@@ -462,25 +467,16 @@ int main(int argc, const char *argv[])
         goto fail;
     }
 
-    while ((ret = read(STDIN_FILENO, buf + len, IN_BUF_SIZE - len)) != 0) {
-        if (ret == -1) {
-            if (errno == EINTR || errno == EAGAIN) {
-                continue;
-            }
-            DEBUG(1, ("read failed [%d][%s].\n", errno, strerror(errno)));
-            goto fail;
-        } else if (ret > 0) {
-            len += ret;
-            if (len > IN_BUF_SIZE) {
-                DEBUG(1, ("read too much, this should never happen.\n"));
-                goto fail;
-            }
-            continue;
-        } else {
-            DEBUG(1, ("unexpected return code of read [%d].\n", ret));
-            goto fail;
-        }
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("context initialized\n"));
+
+    errno = 0;
+    len = sss_atomic_read_s(STDIN_FILENO, buf, IN_BUF_SIZE);
+    if (len == -1) {
+        ret = errno;
+        DEBUG(SSSDBG_CRIT_FAILURE, ("read failed [%d][%s].\n", ret, strerror(ret)));
+        goto fail;
     }
+
     close(STDIN_FILENO);
 
     ret = unpack_buffer(buf, len, ibuf);
@@ -489,40 +485,45 @@ int main(int argc, const char *argv[])
         goto fail;
     }
 
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("getting TGT sync\n"));
     kerr = ldap_child_get_tgt_sync(main_ctx,
                                    ibuf->realm_str, ibuf->princ_str,
                                    ibuf->keytab_name, ibuf->lifetime,
                                    &ccname, &expire_time);
     if (kerr != EOK) {
-        DEBUG(1, ("ldap_child_get_tgt_sync failed.\n"));
+        DEBUG(SSSDBG_CRIT_FAILURE, ("ldap_child_get_tgt_sync failed.\n"));
         /* Do not return, must report failure */
     }
 
     ret = prepare_response(main_ctx, ccname, expire_time, kerr, &resp);
     if (ret != EOK) {
-        DEBUG(1, ("prepare_response failed. [%d][%s].\n", ret, strerror(ret)));
-        return ENOMEM;
+        DEBUG(SSSDBG_CRIT_FAILURE, ("prepare_response failed. [%d][%s].\n",
+                    ret, strerror(ret)));
+        goto fail;
     }
 
-    written = 0;
-    while (written < resp->size) {
-        ret = write(STDOUT_FILENO, resp->buf + written, resp->size - written);
-        if (ret == -1) {
-            if (errno == EAGAIN || errno == EINTR) {
-                continue;
-            }
-            ret = errno;
-            DEBUG(1, ("write failed [%d][%s].\n", ret, strerror(ret)));
-            return ret;
-        }
-        written += ret;
+    errno = 0;
+    written = sss_atomic_write_s(STDOUT_FILENO, resp->buf, resp->size);
+    if (written == -1) {
+        ret = errno;
+        DEBUG(SSSDBG_CRIT_FAILURE, ("write failed [%d][%s].\n", ret,
+                    strerror(ret)));
+        goto fail;
     }
 
+    if (written != resp->size) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Expected to write %d bytes, wrote %d\n",
+              resp->size, written));
+        goto fail;
+    }
+
+    DEBUG(SSSDBG_TRACE_FUNC, ("ldap_child completed successfully\n"));
     close(STDOUT_FILENO);
     talloc_free(main_ctx);
     _exit(0);
 
 fail:
+    DEBUG(SSSDBG_CRIT_FAILURE, ("ldap_child failed!\n"));
     close(STDOUT_FILENO);
     talloc_free(main_ctx);
     _exit(-1);
