@@ -28,6 +28,7 @@
 #include "providers/ldap/sdap_access.h"
 #include "providers/ldap/sdap_sudo.h"
 #include "providers/ldap/sdap_autofs.h"
+#include "providers/ldap/sdap_idmap.h"
 
 static void sdap_shutdown(struct be_req *req);
 
@@ -87,6 +88,7 @@ int sssm_ldap_id_init(struct be_ctx *bectx,
 {
     struct sdap_id_ctx *ctx;
     const char *urls;
+    const char *backup_urls;
     const char *dns_service_name;
     const char *sasl_mech;
     int ret;
@@ -116,12 +118,11 @@ int sssm_ldap_id_init(struct be_ctx *bectx,
     DEBUG(7, ("Service name for discovery set to %s\n", dns_service_name));
 
     urls = dp_opt_get_string(ctx->opts->basic, SDAP_URI);
-    if (!urls) {
-        DEBUG(1, ("Missing ldap_uri, will use service discovery\n"));
-    }
+    backup_urls = dp_opt_get_string(ctx->opts->basic, SDAP_BACKUP_URI);
 
     ret = sdap_service_init(ctx, ctx->be, "LDAP",
-                            dns_service_name, urls, &ctx->service);
+                            dns_service_name, urls, backup_urls,
+                            &ctx->service);
     if (ret != EOK) {
         DEBUG(1, ("Failed to initialize failover service!\n"));
         goto done;
@@ -153,6 +154,12 @@ int sssm_ldap_id_init(struct be_ctx *bectx,
         goto done;
     }
 
+    if (dp_opt_get_bool(ctx->opts->basic, SDAP_ID_MAPPING)) {
+        /* Set up the ID mapping object */
+        ret = sdap_idmap_init(ctx, ctx, &ctx->opts->idmap_ctx);
+        if (ret != EOK) goto done;
+    }
+
     ret = sdap_id_setup_tasks(ctx);
     if (ret != EOK) {
         goto done;
@@ -236,7 +243,7 @@ int sssm_ldap_chpass_init(struct be_ctx *bectx,
         ctx->chpass_service = NULL;
     } else {
         ret = sdap_service_init(ctx, ctx->be, "LDAP_CHPASS", dns_service_name,
-                                urls, &ctx->chpass_service);
+                                urls, NULL, &ctx->chpass_service);
         if (ret != EOK) {
             DEBUG(1, ("Failed to initialize failover service!\n"));
             goto done;
