@@ -742,6 +742,14 @@ static char *get_naming_context(TALLOC_CTX *mem_ctx,
         }
     }
 
+    /* Some directory servers such as Novell eDirectory will return
+     * a zero-length namingContexts value in some situations. In this
+     * case, we should return it as NULL so things fail gracefully.
+     */
+    if (naming_context[0] == '\0') {
+        talloc_zfree(naming_context);
+    }
+
     return naming_context;
 }
 
