@@ -129,6 +129,7 @@ struct sdap_ppolicy_data {
 
 #define SDAP_ROOTDSE_ATTR_NAMING_CONTEXTS "namingContexts"
 #define SDAP_ROOTDSE_ATTR_DEFAULT_NAMING_CONTEXT "defaultNamingContext"
+#define SDAP_ROOTDSE_ATTR_AD_VERSION "domainControllerFunctionality"
 
 #define SDAP_IPA_USN "entryUSN"
 #define SDAP_IPA_LAST_USN "lastUSN"
@@ -150,6 +151,7 @@ enum sdap_result {
 
 enum sdap_basic_opt {
     SDAP_URI = 0,
+    SDAP_BACKUP_URI,
     SDAP_SEARCH_BASE,
     SDAP_DEFAULT_BIND_DN,
     SDAP_DEFAULT_AUTHTOK_TYPE,
@@ -166,8 +168,13 @@ enum sdap_basic_opt {
     SDAP_GROUP_SEARCH_FILTER,
     SDAP_SERVICE_SEARCH_BASE,
     SDAP_SUDO_SEARCH_BASE,
-    SDAP_SUDO_REFRESH_ENABLED,
-    SDAP_SUDO_REFRESH_TIMEOUT,
+    SDAP_SUDO_FULL_REFRESH_INTERVAL,
+    SDAP_SUDO_SMART_REFRESH_INTERVAL,
+    SDAP_SUDO_USE_HOST_FILTER,
+    SDAP_SUDO_HOSTNAMES,
+    SDAP_SUDO_IP,
+    SDAP_SUDO_INCLUDE_NETGROUPS,
+    SDAP_SUDO_INCLUDE_REGEXP,
     SDAP_AUTOFS_SEARCH_BASE,
     SDAP_SCHEMA,
     SDAP_OFFLINE_TIMEOUT,
@@ -180,6 +187,7 @@ enum sdap_basic_opt {
     SDAP_TLS_KEY,
     SDAP_TLS_CIPHER_SUITE,
     SDAP_ID_TLS,
+    SDAP_ID_MAPPING,
     SDAP_SASL_MECH,
     SDAP_SASL_AUTHID,
     SDAP_SASL_REALM,
@@ -187,6 +195,7 @@ enum sdap_basic_opt {
     SDAP_KRB5_KEYTAB,
     SDAP_KRB5_KINIT,
     SDAP_KRB5_KDC,
+    SDAP_KRB5_BACKUP_KDC,
     SDAP_KRB5_REALM,
     SDAP_KRB5_CANONICALIZE,
     SDAP_PWD_POLICY,
@@ -201,6 +210,7 @@ enum sdap_basic_opt {
     SDAP_ACCOUNT_EXPIRE_POLICY,
     SDAP_ACCESS_ORDER,
     SDAP_CHPASS_URI,
+    SDAP_CHPASS_BACKUP_URI,
     SDAP_CHPASS_DNS_SERVICE_NAME,
     SDAP_CHPASS_UPDATE_LAST_CHANGE,
     SDAP_ENUM_SEARCH_TIMEOUT,
@@ -210,6 +220,14 @@ enum sdap_basic_opt {
     SDAP_SASL_CANONICALIZE,
     SDAP_EXPIRE_TIMEOUT,
     SDAP_DISABLE_PAGING,
+    SDAP_IDMAP_LOWER,
+    SDAP_IDMAP_UPPER,
+    SDAP_IDMAP_RANGESIZE,
+    SDAP_IDMAP_AUTORID_COMPAT,
+    SDAP_IDMAP_DEFAULT_DOMAIN,
+    SDAP_IDMAP_DEFAULT_DOMAIN_SID,
+    SDAP_AD_MATCHING_RULE_GROUPS,
+    SDAP_AD_MATCHING_RULE_INITGROUPS,
 
     SDAP_OPTS_BASIC /* opts counter */
 };
@@ -236,6 +254,8 @@ enum sdap_user_attrs {
     SDAP_AT_USER_FULLNAME,
     SDAP_AT_USER_MEMBEROF,
     SDAP_AT_USER_UUID,
+    SDAP_AT_USER_OBJECTSID,
+    SDAP_AT_USER_PRIMARY_GROUP,
     SDAP_AT_USER_MODSTAMP,
     SDAP_AT_USER_USN,
     SDAP_AT_SP_LSTCHG,
@@ -272,6 +292,7 @@ enum sdap_group_attrs {
     SDAP_AT_GROUP_GID,
     SDAP_AT_GROUP_MEMBER,
     SDAP_AT_GROUP_UUID,
+    SDAP_AT_GROUP_OBJECTSID,
     SDAP_AT_GROUP_MODSTAMP,
     SDAP_AT_GROUP_USN,
 
@@ -301,6 +322,7 @@ enum sdap_sudorule_attrs {
     SDAP_AT_SUDO_NOTBEFORE,
     SDAP_AT_SUDO_NOTAFTER,
     SDAP_AT_SUDO_ORDER,
+    SDAP_AT_SUDO_USN,
 
     SDAP_OPTS_SUDO  /* attrs counter */
 };
@@ -335,6 +357,7 @@ struct sdap_attr_map {
     const char *sys_name;
     char *name;
 };
+#define SDAP_ATTR_MAP_TERMINATOR { NULL, NULL, NULL, NULL }
 
 struct sdap_search_base {
     const char *basedn;
@@ -342,20 +365,32 @@ struct sdap_search_base {
     const char *filter;
 };
 
+/* Values from
+ * http://msdn.microsoft.com/en-us/library/cc223272%28v=prot.13%29.aspx
+ */
+enum dc_functional_level {
+    DS_BEHAVIOR_WIN2000 = 0,
+    DS_BEHAVIOR_WIN2003 = 2,
+    DS_BEHAVIOR_WIN2008 = 3,
+    DS_BEHAVIOR_WIN2008R2 = 4,
+    DS_BEHAVIOR_WIN2012 = 5
+};
+
 struct sdap_options {
     struct dp_option *basic;
     struct sdap_attr_map *gen_map;
     struct sdap_attr_map *user_map;
     struct sdap_attr_map *group_map;
     struct sdap_attr_map *netgroup_map;
-    struct sdap_attr_map *host_map;
     struct sdap_attr_map *service_map;
 
+    /* ID-mapping support */
+    struct sdap_idmap_ctx *idmap_ctx;
+
     /* FIXME - should this go to a special struct to avoid mixing with name-service-switch maps? */
     struct sdap_attr_map *sudorule_map;
     struct sdap_attr_map *autofs_mobject_map;
     struct sdap_attr_map *autofs_entry_map;
-    struct sdap_attr_map *selinuxuser_map;
 
     /* supported schema types */
     enum schema_type {
@@ -372,6 +407,9 @@ struct sdap_options {
     struct sdap_search_base **sudo_search_bases;
     struct sdap_search_base **service_search_bases;
     struct sdap_search_base **autofs_search_bases;
+
+    bool support_matching_rule;
+    enum dc_functional_level dc_functional_level;
 };
 
 struct sdap_server_opts {
@@ -381,6 +419,7 @@ struct sdap_server_opts {
     char *max_user_value;
     char *max_group_value;
     char *max_service_value;
+    char *max_sudo_value;
 };
 
 struct sdap_id_ctx;
@@ -441,7 +480,10 @@ bool sdap_check_sup_list(struct sup_list *l, const char *val);
 
 int build_attrs_from_map(TALLOC_CTX *memctx,
                          struct sdap_attr_map *map,
-                         size_t size, const char ***_attrs);
+                         size_t size,
+                         const char **filter,
+                         const char ***_attrs,
+                         size_t *attr_count);
 
 int sdap_control_create(struct sdap_handle *sh, const char *oid, int iscritical,
                         struct berval *value, int dupval, LDAPControl **ctrlp);
