@@ -26,6 +26,7 @@
 #include "util/util.h"
 #include "util/sss_krb5.h"
 #include "util/sss_ldap.h"
+#include "util/strtonum.h"
 #include "providers/ldap/sdap_async_private.h"
 #include "providers/ldap/ldap_common.h"
 
@@ -158,6 +159,7 @@ static void sdap_sys_connect_done(struct tevent_req *subreq)
     bool sasl_nocanon;
     const char *sasl_mech;
     int sasl_minssf;
+    ber_len_t ber_sasl_minssf;
 
     ret = sss_ldap_init_recv(subreq, &state->sh->ldap, &sd);
     talloc_zfree(subreq);
@@ -287,12 +289,12 @@ static void sdap_sys_connect_done(struct tevent_req *subreq)
     if (sasl_mech != NULL) {
         sasl_minssf = dp_opt_get_int(state->opts->basic, SDAP_SASL_MINSSF);
         if (sasl_minssf >= 0) {
+            ber_sasl_minssf = (ber_len_t)sasl_minssf;
             lret = ldap_set_option(state->sh->ldap, LDAP_OPT_X_SASL_SSF_MIN,
-                                   &sasl_minssf);
+                                   &ber_sasl_minssf);
             if (lret != LDAP_OPT_SUCCESS) {
-                DEBUG(SSSDBG_CRIT_FAILURE,
-                      ("Failed to set LDAP MIN SSF option to %d\n",
-                       sasl_minssf));
+                DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to set LDAP MIN SSF option "
+                                            "to %lu\n", sasl_minssf));
                 goto fail;
             }
         }
@@ -541,7 +543,9 @@ static void simple_bind_done(struct sdap_op *op,
     struct simple_bind_state *state = tevent_req_data(req,
                                             struct simple_bind_state);
     char *errmsg = NULL;
-    int ret;
+    char *nval;
+    errno_t ret;
+    int lret;
     LDAPControl **response_controls;
     int c;
     ber_int_t pp_grace;
@@ -555,30 +559,33 @@ static void simple_bind_done(struct sdap_op *op,
 
     state->reply = talloc_steal(state, reply);
 
-    ret = ldap_parse_result(state->sh->ldap, state->reply->msg,
+    lret = ldap_parse_result(state->sh->ldap, state->reply->msg,
                             &state->result, NULL, &errmsg, NULL,
                             &response_controls, 0);
-    if (ret != LDAP_SUCCESS) {
-        DEBUG(2, ("ldap_parse_result failed (%d)\n", state->op->msgid));
+    if (lret != LDAP_SUCCESS) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("ldap_parse_result failed (%d)\n", state->op->msgid));
         ret = EIO;
         goto done;
     }
 
     if (response_controls == NULL) {
-        DEBUG(5, ("Server returned no controls.\n"));
+        DEBUG(SSSDBG_TRACE_LIBS, ("Server returned no controls.\n"));
         state->ppolicy = NULL;
     } else {
         for (c = 0; response_controls[c] != NULL; c++) {
-            DEBUG(9, ("Server returned control [%s].\n",
-                      response_controls[c]->ldctl_oid));
+            DEBUG(SSSDBG_TRACE_INTERNAL,
+                  ("Server returned control [%s].\n",
+                   response_controls[c]->ldctl_oid));
             if (strcmp(response_controls[c]->ldctl_oid,
                        LDAP_CONTROL_PASSWORDPOLICYRESPONSE) == 0) {
-                ret = ldap_parse_passwordpolicy_control(state->sh->ldap,
+                lret = ldap_parse_passwordpolicy_control(state->sh->ldap,
                                                         response_controls[c],
                                                         &pp_expire, &pp_grace,
                                                         &pp_error);
-                if (ret != LDAP_SUCCESS) {
-                    DEBUG(1, ("ldap_parse_passwordpolicy_control failed.\n"));
+                if (lret != LDAP_SUCCESS) {
+                    DEBUG(SSSDBG_MINOR_FAILURE,
+                          ("ldap_parse_passwordpolicy_control failed.\n"));
                     ret = EIO;
                     goto done;
                 }
@@ -586,9 +593,10 @@ static void simple_bind_done(struct sdap_op *op,
                 DEBUG(7, ("Password Policy Response: expire [%d] grace [%d] "
                           "error [%s].\n", pp_expire, pp_grace,
                           ldap_passwordpolicy_err2txt(pp_error)));
-                state->ppolicy = talloc(state, struct sdap_ppolicy_data);
+                if (!state->ppolicy)
+                    state->ppolicy = talloc_zero(state,
+                                                 struct sdap_ppolicy_data);
                 if (state->ppolicy == NULL) {
-                    DEBUG(1, ("talloc failed.\n"));
                     ret = ENOMEM;
                     goto done;
                 }
@@ -596,36 +604,81 @@ static void simple_bind_done(struct sdap_op *op,
                 state->ppolicy->expire = pp_expire;
                 if (state->result == LDAP_SUCCESS) {
                     if (pp_error == PP_changeAfterReset) {
-                        DEBUG(4, ("Password was reset. "
-                                  "User must set a new password.\n"));
+                        DEBUG(SSSDBG_TRACE_LIBS,
+                              ("Password was reset. "
+                               "User must set a new password.\n"));
                         state->result = LDAP_X_SSSD_PASSWORD_EXPIRED;
                     } else if (pp_grace > 0) {
-                        DEBUG(4, ("Password expired. "
-                                  "[%d] grace logins remaining.\n", pp_grace));
+                        DEBUG(SSSDBG_TRACE_LIBS,
+                              ("Password expired. "
+                               "[%d] grace logins remaining.\n",
+                               pp_grace));
                     } else if (pp_expire > 0) {
-                        DEBUG(4, ("Password will expire in [%d] seconds.\n",
-                                  pp_expire));
+                        DEBUG(SSSDBG_TRACE_LIBS,
+                              ("Password will expire in [%d] seconds.\n",
+                               pp_expire));
                     }
                 } else if (state->result == LDAP_INVALID_CREDENTIALS &&
                            pp_error == PP_passwordExpired) {
-                    DEBUG(4,
+                    DEBUG(SSSDBG_TRACE_LIBS,
                           ("Password expired user must set a new password.\n"));
                     state->result = LDAP_X_SSSD_PASSWORD_EXPIRED;
                 }
+            } else if (strcmp(response_controls[c]->ldctl_oid,
+                              LDAP_CONTROL_PWEXPIRED) == 0) {
+                DEBUG(SSSDBG_TRACE_LIBS,
+                      ("Password expired user must set a new password.\n"));
+                state->result = LDAP_X_SSSD_PASSWORD_EXPIRED;
+            } else if (strcmp(response_controls[c]->ldctl_oid,
+                              LDAP_CONTROL_PWEXPIRING) == 0) {
+                /* ignore controls with suspiciously long values */
+                if (response_controls[c]->ldctl_value.bv_len > 32) {
+                    continue;
+                }
+
+                if (!state->ppolicy) {
+                    state->ppolicy = talloc(state, struct sdap_ppolicy_data);
+                }
+
+                if (state->ppolicy == NULL) {
+                    ret = ENOMEM;
+                    goto done;
+                }
+                /* ensure that bv_val is a null-terminated string */
+                nval = talloc_strndup(NULL,
+                                      response_controls[c]->ldctl_value.bv_val,
+                                      response_controls[c]->ldctl_value.bv_len);
+                if (nval == NULL) {
+                    ret = ENOMEM;
+                    goto done;
+                }
+                state->ppolicy->expire = strtouint32(nval, NULL, 10);
+                ret = errno;
+                talloc_zfree(nval);
+                if (ret != EOK) {
+                    DEBUG(SSSDBG_MINOR_FAILURE,
+                          ("Could not convert control response to an integer. ",
+                           "[%s]\n", strerror(ret)));
+                    goto done;
+                }
+
+                DEBUG(SSSDBG_TRACE_LIBS,
+                      ("Password will expire in [%d] seconds.\n",
+                       state->ppolicy->expire));
             }
         }
     }
 
-    DEBUG(3, ("Bind result: %s(%d), %s\n",
+    DEBUG(SSSDBG_TRACE_FUNC, ("Bind result: %s(%d), %s\n",
               sss_ldap_err2string(state->result), state->result,
               errmsg ? errmsg : "no errmsg set"));
 
-    ret = LDAP_SUCCESS;
+    ret = EOK;
 done:
     ldap_controls_free(response_controls);
     ldap_memfree(errmsg);
 
-    if (ret == LDAP_SUCCESS) {
+    if (ret == EOK) {
         tevent_req_done(req);
     } else {
         tevent_req_error(req, ret);
@@ -674,6 +727,8 @@ static struct tevent_req *sasl_bind_send(TALLOC_CTX *memctx,
     struct tevent_req *req;
     struct sasl_bind_state *state;
     int ret = EOK;
+    int optret;
+    char *diag_msg = NULL;
 
     req = tevent_req_create(memctx, &state, struct sasl_bind_state);
     if (!req) return NULL;
@@ -696,8 +751,18 @@ static struct tevent_req *sasl_bind_send(TALLOC_CTX *memctx,
                                        (*sdap_sasl_interact), state);
     state->result = ret;
     if (ret != LDAP_SUCCESS) {
-        DEBUG(1, ("ldap_sasl_bind failed (%d)[%s]\n",
-                  ret, sss_ldap_err2string(ret)));
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("ldap_sasl_bind failed (%d)[%s]\n",
+               ret, sss_ldap_err2string(ret)));
+
+        optret = sss_ldap_get_diagnostic_msg(state, state->sh->ldap,
+                                             &diag_msg);
+        if (optret == EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  ("Extended failure message: [%s]\n", diag_msg));
+        }
+        talloc_zfree(diag_msg);
+
         goto fail;
     }
 
@@ -1250,7 +1315,7 @@ static void sdap_cli_resolve_done(struct tevent_req *subreq)
     struct sdap_cli_connect_state *state = tevent_req_data(req,
                                              struct sdap_cli_connect_state);
     int ret;
-    bool use_tls;
+    bool use_tls = true;
 
     switch (state->force_tls) {
     case CON_TLS_DFL:
@@ -1529,9 +1594,15 @@ static void sdap_cli_auth_step(struct tevent_req *req)
     struct tevent_req *subreq;
     time_t now;
     int expire_timeout;
+    const char *sasl_mech = dp_opt_get_string(state->opts->basic,
+                                              SDAP_SASL_MECH);
+    const char *user_dn = dp_opt_get_string(state->opts->basic,
+                                            SDAP_DEFAULT_BIND_DN);
 
-    if (!state->do_auth) {
-        /* No authentication requested or GSSAPI auth forced off */
+    if (!state->do_auth ||
+        (sasl_mech == NULL && user_dn == NULL)) {
+        DEBUG(SSSDBG_TRACE_LIBS,
+              ("No authentication requested or SASL auth forced off\n"));
         tevent_req_done(req);
         return;
     }
@@ -1549,12 +1620,10 @@ static void sdap_cli_auth_step(struct tevent_req *req)
     subreq = sdap_auth_send(state,
                             state->ev,
                             state->sh,
-                            dp_opt_get_string(state->opts->basic,
-                                                          SDAP_SASL_MECH),
+                            sasl_mech,
                             dp_opt_get_string(state->opts->basic,
                                                         SDAP_SASL_AUTHID),
-                            dp_opt_get_string(state->opts->basic,
-                                                    SDAP_DEFAULT_BIND_DN),
+                            user_dn,
                             dp_opt_get_string(state->opts->basic,
                                                SDAP_DEFAULT_AUTHTOK_TYPE),
                             dp_opt_get_blob(state->opts->basic,
@@ -1831,7 +1900,8 @@ static int sdap_rebind_proc(LDAP *ldap, LDAP_CONST char *url, ber_tag_t request,
                                                         SDAP_DEFAULT_AUTHTOK),
                                         &password);
             if (ret != EOK) {
-                DEBUG(1, ("sdap_auth_get_authtok failed.\n"));
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                     ("sdap_auth_get_authtok failed.\n"));
                 ret = LDAP_LOCAL_ERROR;
                 goto done;
             }
@@ -1840,8 +1910,9 @@ static int sdap_rebind_proc(LDAP *ldap, LDAP_CONST char *url, ber_tag_t request,
         ret = ldap_sasl_bind_s(ldap, user_dn, LDAP_SASL_SIMPLE, &password,
                                request_controls, NULL, NULL);
         if (ret != LDAP_SUCCESS) {
-            DEBUG(1, ("ldap_sasl_bind_s failed (%d)[%s]\n", ret,
-                      sss_ldap_err2string(ret)));
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  ("ldap_sasl_bind_s failed (%d)[%s]\n", ret,
+                  sss_ldap_err2string(ret)));
         }
     } else {
         sasl_bind_state = talloc_zero(tmp_ctx, struct sasl_bind_state);
@@ -1867,7 +1938,7 @@ static int sdap_rebind_proc(LDAP *ldap, LDAP_CONST char *url, ber_tag_t request,
              (ret == LDAP_SUCCESS ? "Successfully" : "Failed to"), url));
 
 done:
+    if (ctrls[0]) ldap_control_free(ctrls[0]);
     talloc_free(tmp_ctx);
-
     return ret;
 }
