@@ -25,6 +25,7 @@
 #include "db/sysdb.h"
 #include "providers/ldap/sdap_async_private.h"
 #include "providers/ldap/ldap_common.h"
+#include "providers/ldap/sdap_idmap.h"
 
 /* ==Save-fake-group-list=====================================*/
 static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
@@ -41,9 +42,12 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
     char **missing;
     gid_t gid;
     int ret;
+    errno_t sret;
     bool in_transaction = false;
     bool posix;
     time_t now;
+    char *sid_str;
+    bool use_id_mapping = dp_opt_get_bool(opts->basic, SDAP_ID_MAPPING);
 
     /* There are no groups in LDAP but we should add user to groups ?? */
     if (ldap_groups_count == 0) return EOK;
@@ -54,18 +58,10 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
     missing = talloc_array(tmp_ctx, char *, ldap_groups_count+1);
     if (!missing) {
         ret = ENOMEM;
-        goto fail;
+        goto done;
     }
     mi = 0;
 
-    ret = sysdb_transaction_start(sysdb);
-    if (ret != EOK) {
-        DEBUG(1, ("Cannot start sysdb transaction [%d]: %s\n",
-                   ret, strerror(ret)));
-        goto fail;
-    }
-    in_transaction = true;
-
     for (i=0; groupnames[i]; i++) {
         ret = sysdb_search_group_by_name(tmp_ctx, sysdb, groupnames[i], NULL, &msg);
         if (ret == EOK) {
@@ -79,17 +75,27 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
         } else if (ret != ENOENT) {
             DEBUG(1, ("search for group failed [%d]: %s\n",
                       ret, strerror(ret)));
-            goto fail;
+            goto done;
         }
     }
     missing[mi] = NULL;
 
     /* All groups are cached, nothing to do */
     if (mi == 0) {
-        talloc_zfree(tmp_ctx);
+        ret = EOK;
         goto done;
     }
 
+    ret = sysdb_transaction_start(sysdb);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("Cannot start sysdb transaction [%d]: %s\n",
+               ret, strerror(ret)));
+        goto done;
+    }
+    in_transaction = true;
+
+
     now = time(NULL);
     for (i=0; missing[i]; i++) {
         /* The group is not in sysdb, need to add a fake entry */
@@ -99,23 +105,57 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
                                            &name);
             if (ret != EOK) {
                 DEBUG(1, ("The group has no name attribute\n"));
-                goto fail;
+                goto done;
             }
 
             if (strcmp(name, missing[i]) == 0) {
                 posix = true;
-                ret = sysdb_attrs_get_uint32_t(ldap_groups[ai],
-                                               SYSDB_GIDNUM,
-                                               &gid);
-                if (ret == ENOENT || (ret == EOK && gid == 0)) {
-                    DEBUG(9, ("The group %s gid was %s\n",
-                              name, ret == ENOENT ? "missing" : "zero"));
-                    DEBUG(8, ("Marking group %s as non-posix and setting GID=0!\n", name));
-                    gid = 0;
-                    posix = false;
-                } else if (ret) {
-                    DEBUG(1, ("The GID attribute is malformed\n"));
-                    goto fail;
+
+                if (use_id_mapping) {
+                    DEBUG(SSSDBG_TRACE_LIBS,
+                          ("Mapping group [%s] objectSID to unix ID\n", name));
+
+                    ret = sdap_attrs_get_sid_str(
+                            tmp_ctx, opts->idmap_ctx, ldap_groups[ai],
+                            opts->group_map[SDAP_AT_GROUP_OBJECTSID].sys_name,
+                            &sid_str);
+                    if (ret != EOK) goto done;
+
+                    DEBUG(SSSDBG_TRACE_INTERNAL,
+                          ("Group [%s] has objectSID [%s]\n",
+                           name, sid_str));
+
+                    /* Convert the SID into a UNIX group ID */
+                    ret = sdap_idmap_sid_to_unix(opts->idmap_ctx, sid_str,
+                                                 &gid);
+                    if (ret == EOK) {
+                        DEBUG(SSSDBG_TRACE_INTERNAL,
+                              ("Group [%s] has mapped gid [%lu]\n",
+                               name, (unsigned long)gid));
+                    } else {
+                        posix = false;
+                        gid = 0;
+
+                        DEBUG(SSSDBG_TRACE_INTERNAL,
+                              ("Group [%s] cannot be mapped. "
+                               "Treating as a non-POSIX group\n",
+                               name));
+                    }
+
+                } else {
+                    ret = sysdb_attrs_get_uint32_t(ldap_groups[ai],
+                                                   SYSDB_GIDNUM,
+                                                   &gid);
+                    if (ret == ENOENT || (ret == EOK && gid == 0)) {
+                        DEBUG(9, ("The group %s gid was %s\n",
+                                  name, ret == ENOENT ? "missing" : "zero"));
+                        DEBUG(8, ("Marking group %s as non-posix and setting GID=0!\n", name));
+                        gid = 0;
+                        posix = false;
+                    } else if (ret) {
+                        DEBUG(1, ("The GID attribute is malformed\n"));
+                        goto done;
+                    }
                 }
 
                 ret = sysdb_attrs_get_string(ldap_groups[ai],
@@ -130,7 +170,7 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
                 ret = sysdb_add_incomplete_group(sysdb, name, gid, original_dn,
                                                  posix, now);
                 if (ret != EOK) {
-                    goto fail;
+                    goto done;
                 }
                 break;
             }
@@ -139,33 +179,36 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
         if (ai == ldap_groups_count) {
             DEBUG(2, ("Group %s not present in LDAP\n", missing[i]));
             ret = EINVAL;
-            goto fail;
+            goto done;
         }
     }
 
-done:
     ret = sysdb_transaction_commit(sysdb);
     if (ret != EOK) {
-        DEBUG(1, ("sysdb_transaction_commit failed.\n"));
-        goto fail;
+        DEBUG(SSSDBG_CRIT_FAILURE, ("sysdb_transaction_commit failed.\n"));
+        goto done;
     }
     in_transaction = false;
     ret = EOK;
-fail:
+
+done:
     if (in_transaction) {
-        sysdb_transaction_cancel(sysdb);
+        sret = sysdb_transaction_cancel(sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to cancel transaction\n"));
+        }
     }
     talloc_free(tmp_ctx);
     return ret;
 }
 
-static int sdap_initgr_common_store(struct sysdb_ctx *sysdb,
-                                    struct sdap_options *opts,
-                                    const char *name,
-                                    enum sysdb_member_type type,
-                                    char **sysdb_grouplist,
-                                    struct sysdb_attrs **ldap_groups,
-                                    int ldap_groups_count)
+int sdap_initgr_common_store(struct sysdb_ctx *sysdb,
+                             struct sdap_options *opts,
+                             const char *name,
+                             enum sysdb_member_type type,
+                             char **sysdb_grouplist,
+                             struct sysdb_attrs **ldap_groups,
+                             int ldap_groups_count)
 {
     TALLOC_CTX *tmp_ctx;
     char **ldap_grouplist = NULL;
@@ -286,6 +329,7 @@ struct tevent_req *sdap_initgr_rfc2307_send(TALLOC_CTX *memctx,
 {
     struct tevent_req *req;
     struct sdap_initgr_rfc2307_state *state;
+    const char **attr_filter;
     char *clean_name;
     errno_t ret;
 
@@ -316,8 +360,17 @@ struct tevent_req *sdap_initgr_rfc2307_send(TALLOC_CTX *memctx,
         return NULL;
     }
 
-    ret = build_attrs_from_map(state, opts->group_map,
-                               SDAP_OPTS_GROUP, &state->attrs);
+    attr_filter = talloc_array(state, const char *, 2);
+    if (!attr_filter) {
+        talloc_free(req);
+        return NULL;
+    }
+
+    attr_filter[0] = opts->group_map[SDAP_AT_GROUP_MEMBER].name;
+    attr_filter[1] = NULL;
+
+    ret = build_attrs_from_map(state, opts->group_map, SDAP_OPTS_GROUP,
+                               attr_filter, &state->attrs, NULL);
     if (ret != EOK) {
         talloc_free(req);
         return NULL;
@@ -397,10 +450,7 @@ static void sdap_initgr_rfc2307_process(struct tevent_req *subreq)
     struct sdap_initgr_rfc2307_state *state;
     struct sysdb_attrs **ldap_groups;
     char **sysdb_grouplist = NULL;
-    struct ldb_message *msg;
-    struct ldb_message_element *groups;
     size_t count;
-    const char *attrs[2];
     int ret;
     int i;
 
@@ -452,41 +502,13 @@ static void sdap_initgr_rfc2307_process(struct tevent_req *subreq)
     }
 
     /* Search for all groups for which this user is a member */
-    attrs[0] = SYSDB_MEMBEROF;
-    attrs[1] = NULL;
-
-    ret = sysdb_search_user_by_name(state, state->sysdb, state->name,
-                                    attrs, &msg);
+    ret = get_sysdb_grouplist(state, state->sysdb, state->name,
+                              &sysdb_grouplist);
     if (ret != EOK) {
         tevent_req_error(req, ret);
         return;
     }
 
-    groups = ldb_msg_find_element(msg, SYSDB_MEMBEROF);
-    if (!groups || groups->num_values == 0) {
-        /* No groups for this user in sysdb currently */
-        sysdb_grouplist = NULL;
-    } else {
-        sysdb_grouplist = talloc_array(state, char *, groups->num_values+1);
-        if (!sysdb_grouplist) {
-            tevent_req_error(req, ENOMEM);
-            return;
-        }
-
-        /* Get a list of the groups by groupname only */
-        for (i=0; i < groups->num_values; i++) {
-            ret = sysdb_group_dn_name(state->sysdb,
-                                      sysdb_grouplist,
-                                      (const char *)groups->values[i].data,
-                                      &sysdb_grouplist[i]);
-            if (ret != EOK) {
-                tevent_req_error(req, ret);
-                return;
-            }
-        }
-        sysdb_grouplist[groups->num_values] = NULL;
-    }
-
     /* There are no nested groups here so we can just update the
      * memberships */
     ret = sdap_initgr_common_store(state->sysdb, state->opts,
@@ -806,8 +828,8 @@ static errno_t sdap_initgr_nested_deref_search(struct tevent_req *req)
     maps[0].num_attrs = SDAP_OPTS_GROUP;
     maps[1].map = NULL;
 
-    ret = build_attrs_from_map(state, state->opts->group_map,
-                               SDAP_OPTS_GROUP, &sdap_attrs);
+    ret = build_attrs_from_map(state, state->opts->group_map, SDAP_OPTS_GROUP,
+                               NULL, &sdap_attrs, NULL);
     if (ret != EOK) goto fail;
 
     timeout = dp_opt_get_int(state->opts->basic, SDAP_SEARCH_TIMEOUT);
@@ -1426,6 +1448,7 @@ static struct tevent_req *sdap_initgr_rfc2307bis_send(
     errno_t ret;
     struct tevent_req *req;
     struct sdap_initgr_rfc2307bis_state *state;
+    const char **attr_filter;
     char *clean_orig_dn;
 
     req = tevent_req_create(memctx, &state, struct sdap_initgr_rfc2307bis_state);
@@ -1458,8 +1481,17 @@ static struct tevent_req *sdap_initgr_rfc2307bis_send(
         return NULL;
     }
 
-    ret = build_attrs_from_map(state, opts->group_map,
-                               SDAP_OPTS_GROUP, &state->attrs);
+    attr_filter = talloc_array(state, const char *, 2);
+    if (!attr_filter) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    attr_filter[0] = opts->group_map[SDAP_AT_GROUP_MEMBER].name;
+    attr_filter[1] = NULL;
+
+    ret = build_attrs_from_map(state, opts->group_map, SDAP_OPTS_GROUP,
+                               attr_filter, &state->attrs, NULL);
     if (ret != EOK) goto done;
 
     ret = sss_filter_sanitize(state, orig_dn, &clean_orig_dn);
@@ -1755,7 +1787,14 @@ save_rfc2307bis_group_memberships(struct sdap_initgr_rfc2307bis_state *state)
     TALLOC_CTX *tmp_ctx;
     struct rfc2307bis_group_memberships_state *membership_state;
     struct membership_diff *iter;
+    struct membership_diff *iter_start;
+    struct membership_diff *iter_tmp;
     bool in_transaction = false;
+    int num_added;
+    int i;
+    int grp_count;
+    int grp_count_old = 0;
+    char **add = NULL;
 
     tmp_ctx = talloc_new(NULL);
     if (!tmp_ctx) return ENOMEM;
@@ -1787,10 +1826,47 @@ save_rfc2307bis_group_memberships(struct sdap_initgr_rfc2307bis_state *state)
     }
     in_transaction = true;
 
+    iter_tmp = membership_state->memberships;
+    iter_start = membership_state->memberships;
+
     DLIST_FOR_EACH(iter, membership_state->memberships) {
+        /* Create a copy of iter->add array but do not include groups outside
+         * nesting limit. This array must be NULL terminated.
+         */
+        for (grp_count = 0; iter->add[grp_count]; grp_count++);
+        if (grp_count > grp_count_old) {
+            add = talloc_realloc(tmp_ctx, add, char*, grp_count + 1);
+            if (add == NULL) {
+                ret = ENOMEM;
+                goto done;
+            }
+        }
+
+        num_added = 0;
+        for (i = 0; i < grp_count; i++) {
+            DLIST_FOR_EACH(iter_tmp, iter_start) {
+                if (!strcmp(iter_tmp->name,iter->add[i])) {
+                    add[num_added] = iter->add[i];
+                    num_added++;
+                    break;
+                }
+            }
+        }
+
+        /* Swap old and new group counter. */
+        grp_count ^= grp_count_old;
+        grp_count_old ^= grp_count;
+        grp_count ^= grp_count_old;
+
+        if (num_added == 0) {
+            /* Nothing to add. Skip. */
+            continue;
+        } else {
+            add[num_added] = NULL;
+        }
         ret = sysdb_update_members(state->sysdb, iter->name,
                                    SYSDB_MEMBER_GROUP,
-                                  (const char *const *) iter->add,
+                                  (const char *const *) add,
                                   (const char *const *) iter->del);
         if (ret != EOK) {
             DEBUG(3, ("Failed to update memberships\n"));
@@ -1894,6 +1970,7 @@ errno_t save_rfc2307bis_user_memberships(
     DEBUG(7, ("Save parent groups to sysdb\n"));
     ret = sysdb_transaction_start(state->sysdb);
     if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
         goto error;
     }
     in_transaction = true;
@@ -1942,6 +2019,7 @@ errno_t save_rfc2307bis_user_memberships(
 
     ret = sysdb_transaction_commit(state->sysdb);
     if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
         goto error;
     }
     in_transaction = false;
@@ -2058,6 +2136,8 @@ struct tevent_req *rfc2307bis_nested_groups_send(
         }
     }
 
+    ret = EOK;
+
 done:
     if (ret == EOK) {
         /* All parent groups were already processed */
@@ -2078,6 +2158,7 @@ static errno_t rfc2307bis_nested_groups_step(struct tevent_req *req)
 {
     errno_t ret;
     TALLOC_CTX *tmp_ctx = NULL;
+    const char **attr_filter;
     char *clean_orig_dn;
     hash_key_t key;
     hash_value_t value;
@@ -2141,8 +2222,17 @@ static errno_t rfc2307bis_nested_groups_step(struct tevent_req *req)
         goto done;
     }
 
-    ret = build_attrs_from_map(state, state->opts->group_map,
-                               SDAP_OPTS_GROUP, &state->attrs);
+    attr_filter = talloc_array(state, const char *, 2);
+    if (!attr_filter) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    attr_filter[0] = state->opts->group_map[SDAP_AT_GROUP_MEMBER].name;
+    attr_filter[1] = NULL;
+
+    ret = build_attrs_from_map(state, state->opts->group_map, SDAP_OPTS_GROUP,
+                               attr_filter, &state->attrs, NULL);
     if (ret != EOK) {
         goto done;
     }
@@ -2477,8 +2567,8 @@ struct tevent_req *sdap_get_initgr_send(TALLOC_CTX *memctx,
         return NULL;
     }
 
-    ret = build_attrs_from_map(state, state->opts->user_map,
-                               SDAP_OPTS_USER, &state->user_attrs);
+    ret = build_attrs_from_map(state, state->opts->user_map, SDAP_OPTS_USER,
+                               NULL, &state->user_attrs, NULL);
     if (ret) {
         talloc_zfree(req);
         return NULL;
@@ -2548,8 +2638,12 @@ static void sdap_get_initgr_user(struct tevent_req *subreq)
     struct sysdb_attrs **usr_attrs;
     size_t count;
     int ret;
+    errno_t sret;
     const char *orig_dn;
     const char *cname;
+    bool in_transaction = false;
+    bool use_id_mapping =
+            dp_opt_get_bool(state->opts->basic, SDAP_ID_MAPPING);
 
     DEBUG(9, ("Receiving info for the user\n"));
 
@@ -2584,9 +2678,10 @@ static void sdap_get_initgr_user(struct tevent_req *subreq)
 
     ret = sysdb_transaction_start(state->sysdb);
     if (ret) {
-        tevent_req_error(req, ret);
-        return;
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
+        goto fail;
     }
+    in_transaction = true;
 
     DEBUG(9, ("Storing the user\n"));
 
@@ -2595,18 +2690,17 @@ static void sdap_get_initgr_user(struct tevent_req *subreq)
                          state->orig_user,
                          true, NULL, 0);
     if (ret) {
-        sysdb_transaction_cancel(state->sysdb);
-        tevent_req_error(req, ret);
-        return;
+        goto fail;
     }
 
     DEBUG(9, ("Commit change\n"));
 
     ret = sysdb_transaction_commit(state->sysdb);
     if (ret) {
-        tevent_req_error(req, ret);
-        return;
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
+        goto fail;
     }
+    in_transaction = false;
 
     ret = sysdb_get_real_name(state, state->sysdb, state->name, &cname);
     if (ret != EOK) {
@@ -2619,13 +2713,6 @@ static void sdap_get_initgr_user(struct tevent_req *subreq)
 
     switch (state->opts->schema_type) {
     case SDAP_SCHEMA_RFC2307:
-        ret = sdap_check_aliases(state->sysdb, state->orig_user, state->dom,
-                                 state->opts, false);
-        if (ret != EOK) {
-            tevent_req_error(req, ret);
-            return;
-        }
-
         subreq = sdap_initgr_rfc2307_send(state, state->ev, state->opts,
                                           state->sysdb, state->sh,
                                           cname);
@@ -2637,6 +2724,7 @@ static void sdap_get_initgr_user(struct tevent_req *subreq)
         break;
 
     case SDAP_SCHEMA_RFC2307BIS:
+    case SDAP_SCHEMA_AD:
         ret = sysdb_attrs_get_string(state->orig_user,
                                      SYSDB_ORIG_DN,
                                      &orig_dn);
@@ -2645,23 +2733,39 @@ static void sdap_get_initgr_user(struct tevent_req *subreq)
             return;
         }
 
-        subreq = sdap_initgr_rfc2307bis_send(
-                state, state->ev, state->opts, state->sysdb,
-                state->dom, state->sh,
-                cname, orig_dn);
+        if (use_id_mapping
+                && state->opts->dc_functional_level >= DS_BEHAVIOR_WIN2008) {
+            /* Take advantage of AD's tokenGroups mechanism to look up all
+             * parent groups in a single request.
+             */
+            subreq = sdap_get_ad_tokengroups_initgroups_send(
+                    state, state->ev, state->opts, state->sysdb,
+                    state->sh, cname, orig_dn, state->timeout);
+        } else if (state->opts->support_matching_rule
+                    && dp_opt_get_bool(state->opts->basic,
+                                       SDAP_AD_MATCHING_RULE_INITGROUPS)) {
+            /* Take advantage of AD's extensibleMatch filter to look up
+             * all parent groups in a single request.
+             */
+            subreq = sdap_get_ad_match_rule_initgroups_send(
+                    state, state->ev, state->opts, state->sysdb,
+                    state->sh, cname, orig_dn, state->timeout);
+        } else {
+            subreq = sdap_initgr_rfc2307bis_send(
+                    state, state->ev, state->opts, state->sysdb,
+                    state->dom, state->sh,
+                    cname, orig_dn);
+        }
         if (!subreq) {
             tevent_req_error(req, ENOMEM);
             return;
         }
+
         talloc_steal(subreq, orig_dn);
         tevent_req_set_callback(subreq, sdap_get_initgr_done, req);
         break;
-    case SDAP_SCHEMA_IPA_V1:
-    case SDAP_SCHEMA_AD:
-        /* TODO: AD uses a different member/memberof schema
-         *       We need an AD specific call that is able to unroll
-         *       nested groups by doing extensive recursive searches */
 
+    case SDAP_SCHEMA_IPA_V1:
         subreq = sdap_initgr_nested_send(state, state->ev, state->opts,
                                          state->sysdb, state->dom, state->sh,
                                          state->orig_user, state->grp_attrs);
@@ -2676,6 +2780,16 @@ static void sdap_get_initgr_user(struct tevent_req *subreq)
         tevent_req_error(req, EINVAL);
         return;
     }
+
+    return;
+fail:
+    if (in_transaction) {
+        sret = sysdb_transaction_cancel(state->sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to cancel transaction\n"));
+        }
+    }
+    tevent_req_error(req, ret);
 }
 
 static int sdap_initgr_rfc2307bis_recv(struct tevent_req *req);
@@ -2687,22 +2801,44 @@ static void sdap_get_initgr_done(struct tevent_req *subreq)
     struct sdap_get_initgr_state *state = tevent_req_data(req,
                                                struct sdap_get_initgr_state);
     int ret;
+    TALLOC_CTX *tmp_ctx;
     gid_t primary_gid;
     char *gid;
+    char *sid_str;
+    char *dom_sid_str;
+    char *group_sid_str;
+    struct sdap_options *opts = state->opts;
+    bool use_id_mapping = dp_opt_get_bool(opts->basic, SDAP_ID_MAPPING);
 
     DEBUG(9, ("Initgroups done\n"));
 
+    tmp_ctx = talloc_new(NULL);
+    if (!tmp_ctx) {
+        tevent_req_error(req, ENOMEM);
+        return;
+    }
+
     switch (state->opts->schema_type) {
     case SDAP_SCHEMA_RFC2307:
         ret = sdap_initgr_rfc2307_recv(subreq);
         break;
 
     case SDAP_SCHEMA_RFC2307BIS:
-        ret = sdap_initgr_rfc2307bis_recv(subreq);
+    case SDAP_SCHEMA_AD:
+        if (use_id_mapping
+                && state->opts->dc_functional_level >= DS_BEHAVIOR_WIN2008) {
+            ret = sdap_get_ad_tokengroups_initgroups_recv(subreq);
+        }
+        else if (state->opts->support_matching_rule
+                && dp_opt_get_bool(state->opts->basic,
+                                   SDAP_AD_MATCHING_RULE_INITGROUPS)) {
+            ret = sdap_get_ad_match_rule_initgroups_recv(subreq);
+        } else {
+            ret = sdap_initgr_rfc2307bis_recv(subreq);
+        }
         break;
 
     case SDAP_SCHEMA_IPA_V1:
-    case SDAP_SCHEMA_AD:
         ret = sdap_initgr_nested_recv(subreq);
         break;
 
@@ -2716,35 +2852,94 @@ static void sdap_get_initgr_done(struct tevent_req *subreq)
     if (ret) {
         DEBUG(9, ("Error in initgroups: [%d][%s]\n",
                   ret, strerror(ret)));
-        tevent_req_error(req, ret);
-        return;
+        goto fail;
     }
 
     /* We also need to update the user's primary group, since
      * the user may not be an explicit member of that group
      */
-    ret = sysdb_attrs_get_uint32_t(state->orig_user, SYSDB_GIDNUM, &primary_gid);
-    if (ret != EOK) {
-        DEBUG(6, ("Could not find user's primary GID\n"));
-        tevent_req_error(req, ret);
-        return;
+
+    if (use_id_mapping) {
+        DEBUG(SSSDBG_TRACE_LIBS,
+              ("Mapping primary group to unix ID\n"));
+
+        /* The primary group ID is just the RID part of the objectSID
+         * of the group. Generate the GID by adding this to the domain
+         * SID value.
+         */
+
+        /* Get the user SID so we can extract the domain SID
+         * from it.
+         */
+        ret = sdap_attrs_get_sid_str(
+                tmp_ctx, opts->idmap_ctx, state->orig_user,
+                opts->user_map[SDAP_AT_USER_OBJECTSID].sys_name,
+                &sid_str);
+        if (ret != EOK) goto fail;
+
+        /* Get the domain SID from the user SID */
+        ret = sdap_idmap_get_dom_sid_from_object(tmp_ctx, sid_str,
+                                                 &dom_sid_str);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  ("Could not parse domain SID from [%s]\n", sid_str));
+            goto fail;
+        }
+
+        ret = sysdb_attrs_get_uint32_t(
+                state->orig_user,
+                opts->user_map[SDAP_AT_USER_PRIMARY_GROUP].sys_name,
+                &primary_gid);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  ("no primary group ID provided\n"));
+            ret = EINVAL;
+            goto fail;
+        }
+
+        /* Add the RID to the end */
+        group_sid_str = talloc_asprintf(tmp_ctx, "%s-%lu",
+                                        dom_sid_str,
+                                        (unsigned long)primary_gid);
+        if (!group_sid_str) {
+            ret = ENOMEM;
+            goto fail;
+        }
+
+        /* Convert the SID into a UNIX group ID */
+        ret = sdap_idmap_sid_to_unix(opts->idmap_ctx, sid_str, &primary_gid);
+        if (ret != EOK) goto fail;
+    } else {
+        ret = sysdb_attrs_get_uint32_t(state->orig_user, SYSDB_GIDNUM,
+                                       &primary_gid);
+        if (ret != EOK) {
+            DEBUG(6, ("Could not find user's primary GID\n"));
+            goto fail;
+        }
     }
 
     gid = talloc_asprintf(state, "%lu", (unsigned long)primary_gid);
     if (gid == NULL) {
-        tevent_req_error(req, ENOMEM);
-        return;
+        ret = ENOMEM;
+        goto fail;
     }
 
     subreq = groups_get_send(req, state->ev, state->id_ctx, gid,
                              BE_FILTER_IDNUM, BE_ATTR_ALL);
     if (!subreq) {
-        tevent_req_error(req, ENOMEM);
-        return;
+        ret = ENOMEM;
+        goto fail;
     }
     tevent_req_set_callback(subreq, sdap_get_initgr_pgid, req);
 
+    talloc_free(tmp_ctx);
     tevent_req_done(req);
+    return;
+
+fail:
+    talloc_free(tmp_ctx);
+    tevent_req_error(req, ret);
+    return;
 }
 
 static void sdap_get_initgr_pgid(struct tevent_req *subreq)
@@ -2770,3 +2965,65 @@ int sdap_get_initgr_recv(struct tevent_req *req)
     return EOK;
 }
 
+errno_t get_sysdb_grouplist(TALLOC_CTX *mem_ctx,
+                            struct sysdb_ctx *sysdb,
+                            const char *name,
+                            char ***grouplist)
+{
+    errno_t ret;
+    const char *attrs[2];
+    struct ldb_message *msg;
+    TALLOC_CTX *tmp_ctx;
+    struct ldb_message_element *groups;
+    char **sysdb_grouplist = NULL;
+    unsigned int i;
+
+    attrs[0] = SYSDB_MEMBEROF;
+    attrs[1] = NULL;
+
+    tmp_ctx = talloc_new(NULL);
+    if (!tmp_ctx) return ENOMEM;
+
+    ret = sysdb_search_user_by_name(tmp_ctx, sysdb, name,
+                                    attrs, &msg);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Error searching user [%s] by name: [%s]\n",
+               name, strerror(ret)));
+        goto done;
+    }
+
+    groups = ldb_msg_find_element(msg, SYSDB_MEMBEROF);
+    if (!groups || groups->num_values == 0) {
+        /* No groups for this user in sysdb currently */
+        sysdb_grouplist = NULL;
+    } else {
+        sysdb_grouplist = talloc_array(tmp_ctx, char *, groups->num_values+1);
+        if (!sysdb_grouplist) {
+            ret = ENOMEM;
+            goto done;
+        }
+
+        /* Get a list of the groups by groupname only */
+        for (i=0; i < groups->num_values; i++) {
+            ret = sysdb_group_dn_name(sysdb,
+                                      sysdb_grouplist,
+                                      (const char *)groups->values[i].data,
+                                      &sysdb_grouplist[i]);
+            if (ret != EOK) {
+                DEBUG(SSSDBG_MINOR_FAILURE,
+                      ("Could not determine group name from [%s]: [%s]\n",
+                       (const char *)groups->values[i].data, strerror(ret)));
+                goto done;
+            }
+        }
+        sysdb_grouplist[groups->num_values] = NULL;
+    }
+
+    *grouplist = talloc_steal(mem_ctx, sysdb_grouplist);
+
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
