@@ -72,9 +72,14 @@ struct sdap_id_conn_data {
     int notify_lock;
     /* list of operations using connect */
     struct sdap_id_op *ops;
+    /* A flag which is signalizing that this
+     * connection will be disconnected and should
+     * not be used any more */
+    bool disconnecting;
 };
 
 static void sdap_id_conn_cache_be_offline_cb(void *pvt);
+static void sdap_id_conn_cache_fo_reconnect_cb(void *pvt);
 
 static void sdap_id_release_conn_data(struct sdap_id_conn_data *conn_data);
 static int sdap_id_conn_data_destroy(struct sdap_id_conn_data *conn_data);
@@ -118,6 +123,14 @@ int sdap_id_conn_cache_create(TALLOC_CTX *memctx,
         goto fail;
     }
 
+    ret = be_add_reconnect_cb(conn_cache, id_ctx->be,
+                              sdap_id_conn_cache_fo_reconnect_cb, conn_cache,
+                              NULL);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("be_add_reconnect_cb failed.\n"));
+        goto fail;
+    }
+
     *conn_cache_out = conn_cache;
     return EOK;
 
@@ -139,6 +152,18 @@ static void sdap_id_conn_cache_be_offline_cb(void *pvt)
     }
 }
 
+/* Callback for attempt to reconnect to primary server */
+static void sdap_id_conn_cache_fo_reconnect_cb(void *pvt)
+{
+    struct sdap_id_conn_cache *conn_cache = talloc_get_type(pvt, struct sdap_id_conn_cache);
+    struct sdap_id_conn_data *cached_connection = conn_cache->cached_connection;
+
+    /* Release any cached connection on going offline */
+    if (cached_connection != NULL) {
+        cached_connection->disconnecting = true;
+    }
+}
+
 /* Release sdap_id_conn_data and destroy it if no longer needed */
 static void sdap_id_release_conn_data(struct sdap_id_conn_data *conn_data)
 {
@@ -194,7 +219,8 @@ static bool sdap_can_reuse_connection(struct sdap_id_conn_data *conn_data)
 {
     int timeout;
 
-    if (!conn_data || !conn_data->sh || !conn_data->sh->connected) {
+    if (!conn_data || !conn_data->sh ||
+        !conn_data->sh->connected || conn_data->disconnecting) {
         return false;
     }
 
@@ -493,6 +519,8 @@ done:
     return ret;
 }
 
+static void sdap_id_op_connect_reinit_done(struct tevent_req *req);
+
 /* Subrequest callback for connection completion */
 static void sdap_id_op_connect_done(struct tevent_req *subreq)
 {
@@ -503,6 +531,8 @@ static void sdap_id_op_connect_done(struct tevent_req *subreq)
     struct sdap_server_opts *current_srv_opts = NULL;
     bool can_retry = false;
     bool is_offline = false;
+    struct tevent_req *reinit_req = NULL;
+    bool reinit = false;
     int ret;
 
     ret = sdap_cli_connect_recv(subreq, conn_data, &can_retry,
@@ -542,7 +572,10 @@ static void sdap_id_op_connect_done(struct tevent_req *subreq)
                 current_srv_opts->max_user_value = 0;
                 current_srv_opts->max_group_value = 0;
                 current_srv_opts->max_service_value = 0;
+                current_srv_opts->max_sudo_value = 0;
                 current_srv_opts->last_usn = srv_opts->last_usn;
+
+                reinit = true;
             }
         }
         ret = sdap_id_conn_data_set_expire_timer(conn_data);
@@ -667,6 +700,38 @@ static void sdap_id_op_connect_done(struct tevent_req *subreq)
 
         sdap_id_release_conn_data(conn_data);
     }
+
+    if (reinit) {
+        DEBUG(SSSDBG_TRACE_FUNC, ("Server reinitialization detected. "
+                                  "Cleaning cache.\n"));
+        reinit_req = sdap_reinit_cleanup_send(conn_cache->id_ctx->be,
+                                              conn_cache->id_ctx->be,
+                                              conn_cache->id_ctx);
+        if (reinit_req == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Unable to perform reinitialization "
+                                        "clean up.\n"));
+            return;
+        }
+
+        tevent_req_set_callback(reinit_req, sdap_id_op_connect_reinit_done,
+                                NULL);
+    }
+}
+
+static void sdap_id_op_connect_reinit_done(struct tevent_req *req)
+{
+    errno_t ret;
+
+    ret = sdap_reinit_cleanup_recv(req);
+    talloc_zfree(req);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Unable to perform reinitialization "
+              "clean up [%d]: %s\n", ret, strerror(ret)));
+        /* not fatal */
+        return;
+    }
+
+    DEBUG(SSSDBG_TRACE_FUNC, ("Reinitialization clean up completed\n"));
 }
 
 /* Mark operation connection request as complete */
@@ -720,6 +785,7 @@ int sdap_id_op_connect_recv(struct tevent_req *req, int *dp_error)
 int sdap_id_op_done(struct sdap_id_op *op, int retval, int *dp_err_out)
 {
     bool communication_error;
+    struct sdap_id_conn_data *current_conn = op->conn_data;
     switch (retval) {
         case EIO:
         case ETIMEDOUT:
@@ -732,8 +798,8 @@ int sdap_id_op_done(struct sdap_id_op *op, int retval, int *dp_err_out)
             break;
     }
 
-    if (communication_error && op->conn_data != 0
-            && op->conn_data == op->conn_cache->cached_connection) {
+    if (communication_error && current_conn != 0
+            && current_conn == op->conn_cache->cached_connection) {
         /* do not reuse failed connection */
         op->conn_cache->cached_connection = NULL;
 
@@ -773,7 +839,7 @@ int sdap_id_op_done(struct sdap_id_op *op, int retval, int *dp_err_out)
         op->reconnect_retry_count = 0;
     }
 
-    if (op->conn_data) {
+    if (current_conn) {
         DEBUG(9, ("releasing operation connection\n"));
         sdap_id_op_hook_conn_data(op, NULL);
     }
