@@ -62,7 +62,6 @@ static int get_pw_name(TALLOC_CTX *mem_ctx,
     pwd = talloc_zero(tmpctx, struct passwd);
     if (!pwd) {
         ret = ENOMEM;
-        status = NSS_STATUS_TRYAGAIN;
         goto done;
     }
 
@@ -70,7 +69,6 @@ static int get_pw_name(TALLOC_CTX *mem_ctx,
     buffer = talloc_size(tmpctx, buflen);
     if (!buffer) {
         ret = ENOMEM;
-        status = NSS_STATUS_TRYAGAIN;
         goto done;
     }
 
@@ -143,8 +141,8 @@ done:
     talloc_zfree(tmpctx);
     if (ret) {
         DEBUG(SSSDBG_OP_FAILURE,
-              ("proxy -> getpwnam_r failed for '%s' <%d>\n",
-               name, status));
+              ("proxy -> getpwnam_r failed for '%s' <%d>: %s\n",
+               name, ret, strerror(ret)));
     }
     return ret;
 }
@@ -334,7 +332,8 @@ done:
     talloc_zfree(tmpctx);
     if (ret) {
         DEBUG(SSSDBG_CRIT_FAILURE,
-              ("proxy -> getpwuid_r failed for '%d' <%d>\n", uid, status));
+              ("proxy -> getpwuid_r failed for '%d' <%d>: %s\n",
+               uid, ret, strerror(ret)));
     }
     return ret;
 }
@@ -354,8 +353,10 @@ static int enum_users(TALLOC_CTX *mem_ctx,
     char *buffer;
     char *newbuf;
     int ret;
+    errno_t sret;
+    bool again;
 
-    DEBUG(7, ("Enumerating users\n"));
+    DEBUG(SSSDBG_TRACE_LIBS, ("Enumerating users\n"));
 
     tmpctx = talloc_new(mem_ctx);
     if (!tmpctx) {
@@ -377,6 +378,7 @@ static int enum_users(TALLOC_CTX *mem_ctx,
 
     ret = sysdb_transaction_start(sysdb);
     if (ret) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
         goto done;
     }
     in_transaction = true;
@@ -387,81 +389,95 @@ static int enum_users(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-again:
-    /* always zero out the pwd structure */
-    memset(pwd, 0, sizeof(struct passwd));
+    do {
+        again = false;
 
-    /* get entry */
-    status = ctx->ops.getpwent_r(pwd, buffer, buflen, &ret);
+        /* always zero out the pwd structure */
+        memset(pwd, 0, sizeof(struct passwd));
 
-    switch (status) {
-    case NSS_STATUS_TRYAGAIN:
-        /* buffer too small ? */
-        if (buflen < MAX_BUF_SIZE) {
-            buflen *= 2;
-        }
-        if (buflen > MAX_BUF_SIZE) {
-            buflen = MAX_BUF_SIZE;
-        }
-        newbuf = talloc_realloc_size(tmpctx, buffer, buflen);
-        if (!newbuf) {
-            ret = ENOMEM;
-            goto done;
-        }
-        buffer = newbuf;
-        goto again;
+        /* get entry */
+        status = ctx->ops.getpwent_r(pwd, buffer, buflen, &ret);
 
-    case NSS_STATUS_NOTFOUND:
+        switch (status) {
+            case NSS_STATUS_TRYAGAIN:
+                /* buffer too small ? */
+                if (buflen < MAX_BUF_SIZE) {
+                    buflen *= 2;
+                }
+                if (buflen > MAX_BUF_SIZE) {
+                    buflen = MAX_BUF_SIZE;
+                }
+                newbuf = talloc_realloc_size(tmpctx, buffer, buflen);
+                if (!newbuf) {
+                    ret = ENOMEM;
+                    goto done;
+                }
+                buffer = newbuf;
+                again = true;
+                break;
 
-        /* we are done here */
-        DEBUG(7, ("Enumeration completed.\n"));
+            case NSS_STATUS_NOTFOUND:
 
-        ret = sysdb_transaction_commit(sysdb);
-        in_transaction = false;
-        break;
+                /* we are done here */
+                DEBUG(SSSDBG_TRACE_LIBS, ("Enumeration completed.\n"));
 
-    case NSS_STATUS_SUCCESS:
+                ret = sysdb_transaction_commit(sysdb);
+                if (ret != EOK) {
+                    DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
+                    goto done;
+                }
+                in_transaction = false;
+                break;
 
-        DEBUG(7, ("User found (%s, %d, %d)\n",
-                  pwd->pw_name, pwd->pw_uid, pwd->pw_gid));
+            case NSS_STATUS_SUCCESS:
 
-        /* uid=0 or gid=0 are invalid values */
-        /* also check that the id is in the valid range for this domain */
-        if (OUT_OF_ID_RANGE(pwd->pw_uid, dom->id_min, dom->id_max) ||
-            OUT_OF_ID_RANGE(pwd->pw_gid, dom->id_min, dom->id_max)) {
+                DEBUG(SSSDBG_TRACE_LIBS, ("User found (%s, %d, %d)\n",
+                            pwd->pw_name, pwd->pw_uid, pwd->pw_gid));
 
-            DEBUG(2, ("User [%s] filtered out! (id out of range)\n",
-                      pwd->pw_name));
+                /* uid=0 or gid=0 are invalid values */
+                /* also check that the id is in the valid range for this domain
+                 */
+                if (OUT_OF_ID_RANGE(pwd->pw_uid, dom->id_min, dom->id_max) ||
+                    OUT_OF_ID_RANGE(pwd->pw_gid, dom->id_min, dom->id_max)) {
 
-            goto again; /* skip */
-        }
+                    DEBUG(SSSDBG_OP_FAILURE, ("User [%s] filtered out! (id out"
+                        " of range)\n", pwd->pw_name));
 
-        ret = save_user(sysdb, !dom->case_sensitive, pwd,
+                    again = true;
+                    break;
+                }
+
+                ret = save_user(sysdb, !dom->case_sensitive, pwd,
                         pwd->pw_name, NULL, dom->user_timeout);
-        if (ret) {
-            /* Do not fail completely on errors.
-             * Just report the failure to save and go on */
-            DEBUG(2, ("Failed to store user %s. Ignoring.\n",
-                      pwd->pw_name));
+                if (ret) {
+                    /* Do not fail completely on errors.
+                     * Just report the failure to save and go on */
+                    DEBUG(SSSDBG_OP_FAILURE, ("Failed to store user %s."
+                                " Ignoring.\n", pwd->pw_name));
+                }
+                again = true;
+                break;
+
+            case NSS_STATUS_UNAVAIL:
+                /* "remote" backend unavailable. Enter offline mode */
+                ret = ENXIO;
+                break;
+
+            default:
+                ret = EIO;
+                DEBUG(SSSDBG_OP_FAILURE, ("proxy -> getpwent_r failed (%d)[%s]"
+                            "\n", ret, strerror(ret)));
+                break;
         }
-        goto again; /* next */
-
-    case NSS_STATUS_UNAVAIL:
-        /* "remote" backend unavailable. Enter offline mode */
-        ret = ENXIO;
-        break;
-
-    default:
-        ret = EIO;
-        DEBUG(2, ("proxy -> getpwent_r failed (%d)[%s]\n",
-                  ret, strerror(ret)));
-        break;
-    }
+    } while (again);
 
 done:
     talloc_zfree(tmpctx);
     if (in_transaction) {
-        sysdb_transaction_cancel(sysdb);
+        sret = sysdb_transaction_cancel(sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to cancel transaction\n"));
+        }
     }
     ctx->ops.endpwent();
     return ret;
@@ -488,6 +504,7 @@ done:
 
 
 static errno_t proxy_process_missing_users(struct sysdb_ctx *sysdb,
+                                           struct sysdb_attrs *group_attrs,
                                            struct group *grp,
                                            time_t now);
 static int save_group(struct sysdb_ctx *sysdb, struct sss_domain_info *dom,
@@ -510,7 +527,10 @@ static int save_group(struct sysdb_ctx *sysdb, struct sss_domain_info *dom,
     DEBUG_GR_MEM(7, grp);
 
     ret = sysdb_transaction_start(sysdb);
-    if (ret != EOK) goto done;
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
+        goto done;
+    }
     in_transaction = true;
 
     if (grp->gr_mem && grp->gr_mem[0]) {
@@ -529,8 +549,8 @@ static int save_group(struct sysdb_ctx *sysdb, struct sss_domain_info *dom,
             goto done;
         }
 
-        /* Create fake users if they don't already exist */
-        ret = proxy_process_missing_users(sysdb, grp, now);
+        /* Create ghost users */
+        ret = proxy_process_missing_users(sysdb, attrs, grp, now);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE, ("Could not add missing members\n"));
             goto done;
@@ -611,6 +631,7 @@ done:
 }
 
 static errno_t proxy_process_missing_users(struct sysdb_ctx *sysdb,
+                                           struct sysdb_attrs *group_attrs,
                                            struct group *grp,
                                            time_t now)
 {
@@ -635,15 +656,15 @@ static errno_t proxy_process_missing_users(struct sysdb_ctx *sysdb,
             talloc_zfree(msg);
             continue;
         } else if (ret == ENOENT) {
-            /* No entry for this user. Create a fake user */
+            /* No entry for this user. Create a ghost user */
             DEBUG(SSSDBG_TRACE_LIBS,
-                  ("Member [%s] not cached, creating fake user entry\n",
+                  ("Member [%s] not cached, creating ghost user entry\n",
                    grp->gr_mem[i]));
 
-            ret = sysdb_add_fake_user(sysdb, grp->gr_mem[i], NULL, now);
+            ret = sysdb_attrs_add_string(group_attrs, SYSDB_GHOST, grp->gr_mem[i]);
             if (ret != EOK) {
                 DEBUG(SSSDBG_MINOR_FAILURE,
-                      ("Cannot store fake user entry: [%d]: %s\n",
+                      ("Cannot store ghost user entry: [%d]: %s\n",
                        ret, strerror(ret)));
                 goto done;
             }
@@ -656,6 +677,7 @@ static errno_t proxy_process_missing_users(struct sysdb_ctx *sysdb,
         }
     }
 
+    ret = EOK;
 done:
     talloc_free(tmp_ctx);
     return ret;
@@ -848,8 +870,8 @@ done:
     talloc_zfree(tmpctx);
     if (ret) {
         DEBUG(SSSDBG_OP_FAILURE,
-              ("proxy -> getgrnam_r failed for '%s' <%d>\n",
-              name, status));
+              ("proxy -> getgrnam_r failed for '%s' <%d>: %s\n",
+              name, ret, strerror(ret)));
     }
     return ret;
 }
@@ -917,8 +939,8 @@ done:
     talloc_zfree(tmpctx);
     if (ret) {
         DEBUG(SSSDBG_OP_FAILURE,
-              ("proxy -> getgrgid_r failed for '%d' <%d>\n",
-               gid, status));
+              ("proxy -> getgrgid_r failed for '%d' <%d>: %s\n",
+               gid, ret, strerror(ret)));
     }
     return ret;
 }
@@ -938,8 +960,10 @@ static int enum_groups(TALLOC_CTX *mem_ctx,
     char *buffer;
     char *newbuf;
     int ret;
+    errno_t sret;
+    bool again;
 
-    DEBUG(7, ("Enumerating groups\n"));
+    DEBUG(SSSDBG_TRACE_LIBS, ("Enumerating groups\n"));
 
     tmpctx = talloc_new(mem_ctx);
     if (!tmpctx) {
@@ -961,6 +985,7 @@ static int enum_groups(TALLOC_CTX *mem_ctx,
 
     ret = sysdb_transaction_start(sysdb);
     if (ret) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
         goto done;
     }
     in_transaction = true;
@@ -971,79 +996,94 @@ static int enum_groups(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-again:
-    /* always zero out the grp structure */
-    memset(grp, 0, sizeof(struct group));
+    do {
+        again = false;
 
-    /* get entry */
-    status = ctx->ops.getgrent_r(grp, buffer, buflen, &ret);
+        /* always zero out the grp structure */
+        memset(grp, 0, sizeof(struct group));
 
-    switch (status) {
-    case NSS_STATUS_TRYAGAIN:
-        /* buffer too small ? */
-        if (buflen < MAX_BUF_SIZE) {
-            buflen *= 2;
+        /* get entry */
+        status = ctx->ops.getgrent_r(grp, buffer, buflen, &ret);
+
+        switch (status) {
+            case NSS_STATUS_TRYAGAIN:
+                /* buffer too small ? */
+                if (buflen < MAX_BUF_SIZE) {
+                    buflen *= 2;
+                }
+                if (buflen > MAX_BUF_SIZE) {
+                    buflen = MAX_BUF_SIZE;
+                }
+                newbuf = talloc_realloc_size(tmpctx, buffer, buflen);
+                if (!newbuf) {
+                    ret = ENOMEM;
+                    goto done;
+                }
+                buffer = newbuf;
+                again = true;
+                break;
+
+            case NSS_STATUS_NOTFOUND:
+
+                /* we are done here */
+                DEBUG(SSSDBG_TRACE_LIBS, ("Enumeration completed.\n"));
+
+                ret = sysdb_transaction_commit(sysdb);
+                if (ret != EOK) {
+                    DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
+                    goto done;
+                }
+                in_transaction = false;
+                break;
+
+            case NSS_STATUS_SUCCESS:
+
+                DEBUG(SSSDBG_OP_FAILURE, ("Group found (%s, %d)\n",
+                            grp->gr_name, grp->gr_gid));
+
+                /* gid=0 is an invalid value */
+                /* also check that the id is in the valid range for this domain
+                 */
+                if (OUT_OF_ID_RANGE(grp->gr_gid, dom->id_min, dom->id_max)) {
+
+                    DEBUG(SSSDBG_OP_FAILURE, ("Group [%s] filtered out! (id"
+                        "out of range)\n", grp->gr_name));
+
+                    again = true;
+                    break;
+                }
+
+                ret = save_group(sysdb, dom, grp, grp->gr_name,
+                        NULL, dom->group_timeout);
+                if (ret) {
+                    /* Do not fail completely on errors.
+                     * Just report the failure to save and go on */
+                    DEBUG(SSSDBG_OP_FAILURE, ("Failed to store group."
+                                "Ignoring\n"));
+                }
+                again = true;
+                break;
+
+            case NSS_STATUS_UNAVAIL:
+                /* "remote" backend unavailable. Enter offline mode */
+                ret = ENXIO;
+                break;
+
+            default:
+                ret = EIO;
+                DEBUG(SSSDBG_OP_FAILURE, ("proxy -> getgrent_r failed (%d)[%s]"
+                            "\n", ret, strerror(ret)));
+                break;
         }
-        if (buflen > MAX_BUF_SIZE) {
-            buflen = MAX_BUF_SIZE;
-        }
-        newbuf = talloc_realloc_size(tmpctx, buffer, buflen);
-        if (!newbuf) {
-            ret = ENOMEM;
-            goto done;
-        }
-        buffer = newbuf;
-        goto again;
-
-    case NSS_STATUS_NOTFOUND:
-
-        /* we are done here */
-        DEBUG(7, ("Enumeration completed.\n"));
-
-        ret = sysdb_transaction_commit(sysdb);
-        in_transaction = false;
-        break;
-
-    case NSS_STATUS_SUCCESS:
-
-        DEBUG(7, ("Group found (%s, %d)\n",
-                  grp->gr_name, grp->gr_gid));
-
-        /* gid=0 is an invalid value */
-        /* also check that the id is in the valid range for this domain */
-        if (OUT_OF_ID_RANGE(grp->gr_gid, dom->id_min, dom->id_max)) {
-
-                DEBUG(2, ("Group [%s] filtered out! (id out of range)\n",
-                          grp->gr_name));
-
-            goto again; /* skip */
-        }
-
-        ret = save_group(sysdb, dom, grp, grp->gr_name,
-                         NULL, dom->group_timeout);
-        if (ret) {
-            /* Do not fail completely on errors.
-             * Just report the failure to save and go on */
-            DEBUG(2, ("Failed to store group. Ignoring.\n"));
-        }
-        goto again; /* next */
-
-    case NSS_STATUS_UNAVAIL:
-        /* "remote" backend unavailable. Enter offline mode */
-        ret = ENXIO;
-        break;
-
-    default:
-        ret = EIO;
-        DEBUG(2, ("proxy -> getgrent_r failed (%d)[%s]\n",
-                  ret, strerror(ret)));
-        break;
-    }
+    } while (again);
 
 done:
     talloc_zfree(tmpctx);
     if (in_transaction) {
-        sysdb_transaction_cancel(sysdb);
+        sret = sysdb_transaction_cancel(sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to cancel transaction\n"));
+        }
     }
     ctx->ops.endgrent();
     return ret;
@@ -1071,6 +1111,7 @@ static int get_initgr(TALLOC_CTX *mem_ctx,
     char *buffer;
     size_t buflen;
     int ret;
+    errno_t sret;
     bool del_user;
     uid_t uid;
     struct ldb_result *cached_pwd = NULL;
@@ -1096,6 +1137,7 @@ static int get_initgr(TALLOC_CTX *mem_ctx,
 
     ret = sysdb_transaction_start(sysdb);
     if (ret) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
         goto fail;
     }
     in_transaction = true;
@@ -1193,7 +1235,10 @@ done:
 fail:
     talloc_zfree(tmpctx);
     if (in_transaction) {
-        sysdb_transaction_cancel(sysdb);
+        sret = sysdb_transaction_cancel(sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to cancel transaction\n"));
+        }
     }
     return ret;
 }
@@ -1223,32 +1268,34 @@ static int get_initgr_groups_process(TALLOC_CTX *memctx,
         return ENOMEM;
     }
 
-again:
     /* FIXME: should we move this call outside the transaction to keep the
      * transaction as short as possible ? */
-    status = ctx->ops.initgroups_dyn(pwd->pw_name, pwd->pw_gid, &num_gids,
-                                     &num, &gids, limit, &ret);
-    switch (status) {
-    case NSS_STATUS_TRYAGAIN:
-        /* buffer too small ? */
-        if (size < MAX_BUF_SIZE) {
-            num *= 2;
-            size = num*sizeof(gid_t);
-        }
-        if (size > MAX_BUF_SIZE) {
-            size = MAX_BUF_SIZE;
-            num = size/sizeof(gid_t);
-        }
-        limit = num;
-        gids = talloc_realloc_size(memctx, gids, size);
-        if (!gids) {
-            return ENOMEM;
-        }
-        goto again; /* retry with more memory */
+    do {
+        status = ctx->ops.initgroups_dyn(pwd->pw_name, pwd->pw_gid, &num_gids,
+                &num, &gids, limit, &ret);
 
+        if (status == NSS_STATUS_TRYAGAIN) {
+            /* buffer too small ? */
+            if (size < MAX_BUF_SIZE) {
+                num *= 2;
+                size = num*sizeof(gid_t);
+            }
+            if (size > MAX_BUF_SIZE) {
+                size = MAX_BUF_SIZE;
+                num = size/sizeof(gid_t);
+            }
+            limit = num;
+            gids = talloc_realloc_size(memctx, gids, size);
+            if (!gids) {
+                return ENOMEM;
+            }
+        }
+    } while(status == NSS_STATUS_TRYAGAIN);
+
+    switch (status) {
     case NSS_STATUS_SUCCESS:
-        DEBUG(4, ("User [%s] appears to be member of %lu groups\n",
-                  pwd->pw_name, num_gids));
+        DEBUG(SSSDBG_CONF_SETTINGS, ("User [%s] appears to be member of %lu"
+                    "groups\n", pwd->pw_name, num_gids));
 
         now = time(NULL);
         for (i = 0; i < num_gids; i++) {
