@@ -361,20 +361,22 @@ setautomntent_send(TALLOC_CTX *mem_ctx,
     if (!req) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               ("Could not create tevent request for setautomntent\n"));
-        goto fail;
+        return NULL;
     }
     state->cmdctx = cmdctx;
 
     dctx = talloc_zero(state, struct autofs_dom_ctx);
     if (!dctx) {
         DEBUG(SSSDBG_FATAL_FAILURE, ("Out of memory\n"));
+        ret = ENOMEM;
         goto fail;
     }
     dctx->cmd_ctx = state->cmdctx;
     state->dctx = dctx;
 
-    ret = sss_parse_name(state, client->rctx->names, rawname,
-                         &domname, &state->mapname);
+    ret = sss_parse_name_for_domains(state, client->rctx->domains,
+                                     client->rctx->default_domain, rawname,
+                                     &domname, &state->mapname);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               ("Invalid name received [%s]\n", rawname));
@@ -386,13 +388,15 @@ setautomntent_send(TALLOC_CTX *mem_ctx,
          state->mapname, domname?domname:"<ALL>"));
 
     if (domname) {
-        dctx->domain = responder_get_domain(client->rctx->domains, domname);
+        dctx->domain = responder_get_domain(dctx, client->rctx, domname);
         if (!dctx->domain) {
+            ret = EINVAL;
             goto fail;
         }
 
         client->automntmap_name = talloc_strdup(client, rawname);
         if (!client->automntmap_name) {
+            ret = ENOMEM;
             goto fail;
         }
     } else {
@@ -402,6 +406,7 @@ setautomntent_send(TALLOC_CTX *mem_ctx,
 
         client->automntmap_name = talloc_strdup(client, state->mapname);
         if (!client->automntmap_name) {
+            ret = ENOMEM;
             goto fail;
         }
     }
@@ -504,6 +509,7 @@ setautomntent_send(TALLOC_CTX *mem_ctx,
         } else if (ret != EOK) {
             DEBUG(SSSDBG_CRIT_FAILURE, ("Could not get data from cache\n"));
             talloc_free(state->map);
+            ret = ENOMEM;
             goto fail;
         }
 
@@ -520,8 +526,9 @@ setautomntent_send(TALLOC_CTX *mem_ctx,
     return req;
 
 fail:
-    talloc_free(req);
-    return NULL;
+    tevent_req_error(req, ret);
+    tevent_req_post(req, actx->rctx->ev);
+    return req;
 }
 
 static errno_t
@@ -559,8 +566,8 @@ lookup_automntmap_step(struct setautomntent_lookup_ctx *lookup_ctx)
 
         DEBUG(SSSDBG_TRACE_FUNC, ("Requesting info for [%s@%s]\n",
               lookup_ctx->mapname, dom->name));
-        ret = sysdb_get_ctx_from_list(lookup_ctx->rctx->db_list, dom, &sysdb);
-        if (ret != EOK) {
+        sysdb = dom->sysdb;
+        if (sysdb == NULL) {
             DEBUG(SSSDBG_FATAL_FAILURE,
                   ("Fatal: Sysdb CTX not found for this domain!\n"));
             return EIO;
@@ -585,7 +592,6 @@ lookup_automntmap_step(struct setautomntent_lookup_ctx *lookup_ctx)
                 }
                 else break;
             }
-            ret = EOK;
         }
 
         ret = get_autofs_map(lookup_ctx->actx, lookup_ctx->mapname, &map);
@@ -853,7 +859,7 @@ sss_autofs_cmd_getautomntent(struct cli_ctx *client)
 
     SAFEALIGN_COPY_UINT32_CHECK(&namelen, body+c, blen, &c);
 
-    if (namelen == 0 || namelen > blen - c) {
+    if (namelen == 0) {
         ret = EINVAL;
         goto done;
     }
@@ -1128,7 +1134,7 @@ sss_autofs_cmd_getautomntbyname(struct cli_ctx *client)
     /* FIXME - split out a function to get string from <len><str>\0 */
     SAFEALIGN_COPY_UINT32_CHECK(&namelen, body+c, blen, &c);
 
-    if (namelen == 0 || namelen > blen - c) {
+    if (namelen == 0) {
         ret = EINVAL;
         goto done;
     }
@@ -1152,7 +1158,7 @@ sss_autofs_cmd_getautomntbyname(struct cli_ctx *client)
     /* FIXME - split out a function to get string from <len><str>\0 */
     SAFEALIGN_COPY_UINT32_CHECK(&keylen, body+c, blen, &c);
 
-    if (keylen == 0 || keylen > blen - c) {
+    if (keylen == 0) {
         ret = EINVAL;
         goto done;
     }
