@@ -565,14 +565,14 @@ int sss_ncache_reset_permament(struct sss_nc_ctx *ctx)
 
 errno_t sss_ncache_prepopulate(struct sss_nc_ctx *ncache,
                                struct confdb_ctx *cdb,
-                               struct sss_names_ctx *names_ctx,
-                               struct sss_domain_info *domain_list)
+                               struct resp_ctx *rctx)
 {
     errno_t ret;
     bool filter_set = false;
     char **filter_list = NULL;
     char *name = NULL;
     struct sss_domain_info *dom = NULL;
+    struct sss_domain_info *domain_list = rctx->domains;
     char *domainname = NULL;
     char *conf_path = NULL;
     TALLOC_CTX *tmpctx = talloc_new(NULL);
@@ -596,8 +596,10 @@ errno_t sss_ncache_prepopulate(struct sss_nc_ctx *ncache,
         filter_set = true;
 
         for (i = 0; (filter_list && filter_list[i]); i++) {
-            ret = sss_parse_name(tmpctx, names_ctx, filter_list[i],
-                                 &domainname, &name);
+            ret = sss_parse_name_for_domains(tmpctx, domain_list,
+                                             rctx->default_domain,
+                                             filter_list[i],
+                                             &domainname, &name);
             if (ret != EOK) {
                 DEBUG(1, ("Invalid name in filterUsers list: [%s] (%d)\n",
                          filter_list[i], ret));
@@ -641,15 +643,16 @@ errno_t sss_ncache_prepopulate(struct sss_nc_ctx *ncache,
     else if (ret != EOK) goto done;
 
     for (i = 0; (filter_list && filter_list[i]); i++) {
-        ret = sss_parse_name(tmpctx, names_ctx, filter_list[i],
-                             &domainname, &name);
+        ret = sss_parse_name_for_domains(tmpctx, domain_list,
+                                         rctx->default_domain, filter_list[i],
+                                         &domainname, &name);
         if (ret != EOK) {
             DEBUG(1, ("Invalid name in filterUsers list: [%s] (%d)\n",
                      filter_list[i], ret));
             continue;
         }
         if (domainname) {
-            dom = responder_get_domain(domain_list, domainname);
+            dom = responder_get_domain(tmpctx, rctx, domainname);
             if (!dom) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       ("Invalid domain name [%s]\n", domainname));
@@ -693,7 +696,7 @@ errno_t sss_ncache_prepopulate(struct sss_nc_ctx *ncache,
         filter_set = true;
 
         for (i = 0; (filter_list && filter_list[i]); i++) {
-            ret = sss_parse_name(tmpctx, names_ctx, filter_list[i],
+            ret = sss_parse_name(tmpctx, dom->names, filter_list[i],
                                  &domainname, &name);
             if (ret != EOK) {
                 DEBUG(1, ("Invalid name in filterGroups list: [%s] (%d)\n",
@@ -738,15 +741,16 @@ errno_t sss_ncache_prepopulate(struct sss_nc_ctx *ncache,
     else if (ret != EOK) goto done;
 
     for (i = 0; (filter_list && filter_list[i]); i++) {
-        ret = sss_parse_name(tmpctx, names_ctx, filter_list[i],
-                             &domainname, &name);
+        ret = sss_parse_name_for_domains(tmpctx, domain_list,
+                                         rctx->default_domain, filter_list[i],
+                                         &domainname, &name);
         if (ret != EOK) {
             DEBUG(1, ("Invalid name in filterGroups list: [%s] (%d)\n",
                      filter_list[i], ret));
             continue;
         }
         if (domainname) {
-            dom = responder_get_domain(domain_list, domainname);
+            dom = responder_get_domain(tmpctx, rctx, domainname);
             if (!dom) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       ("Invalid domain name [%s]\n", domainname));
