@@ -45,6 +45,7 @@ struct sss_dp_req {
 
     hash_key_t *key;
 
+    struct tevent_timer *tev;
     struct sss_dp_callback *cb_list;
 
     dbus_uint16_t dp_err;
@@ -117,15 +118,6 @@ static int sss_dp_req_destructor(void *ptr)
     return 0;
 }
 
-static void sss_dp_req_timeout(struct tevent_context *ev,
-                               struct tevent_timer *te,
-                               struct timeval t, void *ptr)
-{
-    /* ptr is a pointer to sidereq */
-    /* Just free it to kill all waiting requests when the timeout fires */
-    talloc_zfree(ptr);
-}
-
 void handle_requests_after_reconnect(struct resp_ctx *rctx)
 {
     int ret;
@@ -252,8 +244,6 @@ sss_dp_issue_request(TALLOC_CTX *mem_ctx, struct resp_ctx *rctx,
     struct tevent_req *sidereq;
     struct sss_dp_req *sdp_req;
     struct sss_dp_callback *cb;
-    struct tevent_timer *te;
-    struct timeval tv;
     DBusMessage *msg;
     TALLOC_CTX *tmp_ctx = NULL;
     errno_t ret;
@@ -308,20 +298,6 @@ sss_dp_issue_request(TALLOC_CTX *mem_ctx, struct resp_ctx *rctx,
         }
         tevent_req_set_callback(sidereq, sss_dp_req_done, NULL);
 
-        /* add timeout handling so we do not hang forever should something
-         * go worng in the provider. Use 2 sec less than the idle timeout to
-         * give it a chance to reply to the client before closing the
-         * connection. */
-        tv = tevent_timeval_current_ofs(rctx->client_idle_timeout - 2, 0);
-        te = tevent_add_timer(rctx->ev, sidereq, tv,
-                              sss_dp_req_timeout, sidereq);
-        if (!te) {
-            /* Nothing much we can do */
-            DEBUG(SSSDBG_CRIT_FAILURE, ("Out of memory?!\n"));
-            ret = ENOMEM;
-            goto fail;
-        }
-
         /* We should now be able to find the sdp_req in the hash table */
         hret = hash_lookup(rctx->dp_request_table, key, &value);
         if (hret != HASH_SUCCESS) {
@@ -514,34 +490,24 @@ sss_dp_get_account_msg(void *pvt)
     uint32_t be_type;
     uint32_t attrs = BE_ATTR_CORE;
     char *filter;
-    const char *dp_method;
 
     info = talloc_get_type(pvt, struct sss_dp_account_info);
 
     switch (info->type) {
         case SSS_DP_USER:
             be_type = BE_REQ_USER;
-            dp_method = DP_METHOD_GETACCTINFO;
             break;
         case SSS_DP_GROUP:
             be_type = BE_REQ_GROUP;
-            dp_method = DP_METHOD_GETACCTINFO;
             break;
         case SSS_DP_INITGROUPS:
             be_type = BE_REQ_INITGROUPS;
-            dp_method = DP_METHOD_GETACCTINFO;
             break;
         case SSS_DP_NETGR:
             be_type = BE_REQ_NETGROUP;
-            dp_method = DP_METHOD_GETACCTINFO;
             break;
         case SSS_DP_SERVICES:
             be_type = BE_REQ_SERVICES;
-            dp_method = DP_METHOD_GETACCTINFO;
-            break;
-        case SSS_DP_HOST:
-            be_type = 0;
-            dp_method = DP_METHOD_HOSTHANDLER;
             break;
     }
 
@@ -574,7 +540,7 @@ sss_dp_get_account_msg(void *pvt)
     msg = dbus_message_new_method_call(NULL,
                                        DP_PATH,
                                        DP_INTERFACE,
-                                       dp_method);
+                                       DP_METHOD_GETACCTINFO);
     if (msg == NULL) {
         talloc_free(filter);
         DEBUG(SSSDBG_CRIT_FAILURE, ("Out of memory?!\n"));
@@ -590,6 +556,7 @@ sss_dp_get_account_msg(void *pvt)
                                      DBUS_TYPE_UINT32, &be_type,
                                      DBUS_TYPE_UINT32, &attrs,
                                      DBUS_TYPE_STRING, &filter,
+                                     DBUS_TYPE_STRING, &info->dom->name,
                                      DBUS_TYPE_INVALID);
     talloc_free(filter);
     if (!dbret) {
@@ -665,7 +632,7 @@ sss_dp_internal_get_send(struct resp_ctx *rctx,
      * in some pathological cases it may happen that nss starts up before
      * dp connection code is actually able to establish a connection.
      */
-    ret = sss_dp_get_domain_conn(rctx, dom->name, &be_conn);
+    ret = sss_dp_get_domain_conn(rctx, dom->conn_name, &be_conn);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               ("BUG: The Data Provider connection for %s is not available!",
@@ -786,12 +753,4 @@ static void sss_dp_internal_get_done(DBusPendingCall *pending, void *ptr)
      * This will clean up the hash table entry as well
      */
     talloc_zfree(sdp_req);
-
-    /* Free the sidereq to free the rest of the memory allocated with the
-     * internal dp request. */
-    if (ret == EOK) {
-        tevent_req_done(req);
-    } else {
-        tevent_req_error(req, ret);
-    }
 }
