@@ -39,6 +39,7 @@
 #define NSS_PACKET_MAX_RECV_SIZE 1024
 
 struct getent_ctx;
+struct sss_mc_ctx;
 
 struct nss_ctx {
     struct resp_ctx *rctx;
@@ -60,10 +61,16 @@ struct nss_ctx {
     char *pwfield;
 
     char *override_homedir;
+    char *fallback_homedir;
     char **allowed_shells;
+    char *override_shell;
     char **vetoed_shells;
     char **etc_shells;
     char *shell_fallback;
+    char *default_shell;
+
+    struct sss_mc_ctx *pwd_mc_ctx;
+    struct sss_mc_ctx *grp_mc_ctx;
 };
 
 struct nss_packet;
