@@ -197,7 +197,8 @@ static struct tevent_req *setnetgrent_send(TALLOC_CTX *mem_ctx,
     dctx = state->dctx;
     dctx->cmdctx = state->cmdctx;
 
-    ret = sss_parse_name_for_domains(state, client->rctx->domains, rawname,
+    ret = sss_parse_name_for_domains(state, client->rctx->domains,
+                                     client->rctx->default_domain, rawname,
                                      &domname, &state->netgr_shortname);
     if (ret != EOK) {
         DEBUG(2, ("Invalid name received [%s]\n", rawname));
