@@ -780,6 +780,7 @@ done:
 errno_t parse_getservbyname(TALLOC_CTX *mem_ctx,
                             uint8_t *body, size_t blen,
                             struct sss_domain_info *domains,
+                            char *default_domain,
                             char **domain_name,
                             char **service_name,
                             char **service_protocol);
@@ -821,6 +822,7 @@ int nss_cmd_getservbyname(struct cli_ctx *cctx)
 
     ret = parse_getservbyname(cmdctx, body, blen,
                               cctx->rctx->domains,
+                              cctx->rctx->default_domain,
                               &domname,
                               &service_name,
                               &service_protocol);
@@ -871,6 +873,7 @@ done:
 errno_t parse_getservbyname(TALLOC_CTX *mem_ctx,
                             uint8_t *body, size_t blen,
                             struct sss_domain_info *domains,
+                            char *default_domain,
                             char **domain_name,
                             char **service_name,
                             char **service_protocol)
@@ -959,7 +962,7 @@ errno_t parse_getservbyname(TALLOC_CTX *mem_ctx,
         }
     }
 
-    ret = sss_parse_name_for_domains(tmp_ctx, domains, rawname,
+    ret = sss_parse_name_for_domains(tmp_ctx, domains, default_domain, rawname,
                                      &domname, &svc_name);
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE,
