@@ -23,7 +23,6 @@
 #include <time.h>
 #include "util/util.h"
 #include "util/sss_selinux.h"
-#include "util/auth_utils.h"
 #include "db/sysdb.h"
 #include "confdb/confdb.h"
 #include "responder/common/responder_packet.h"
@@ -34,6 +33,9 @@
 #include "responder/pam/pam_helpers.h"
 #include "db/sysdb.h"
 #include "db/sysdb_selinux.h"
+#ifdef HAVE_SELINUX_LOGIN_DIR
+#include <selinux/selinux.h>
+#endif
 
 enum pam_verbosity {
     PAM_VERBOSITY_NO_MESSAGES = 0,
@@ -43,7 +45,6 @@ enum pam_verbosity {
 };
 
 #define DEFAULT_PAM_VERBOSITY PAM_VERBOSITY_IMPORTANT
-#define DEFAULT_PAM_PWD_EXPIRATION_WARNING 7
 
 static void pam_reply(struct pam_auth_req *preq);
 
@@ -117,9 +118,10 @@ static int pd_set_primary_name(const struct ldb_message *msg,struct pam_data *pd
     return EOK;
 }
 
-static int pam_parse_in_data_v2(struct sss_names_ctx *snctx,
-                             struct pam_data *pd,
-                             uint8_t *body, size_t blen)
+static int pam_parse_in_data_v2(struct sss_domain_info *domains,
+                                const char *default_domain,
+                                struct pam_data *pd,
+                                uint8_t *body, size_t blen)
 {
     size_t c;
     uint32_t type;
@@ -155,8 +157,9 @@ static int pam_parse_in_data_v2(struct sss_names_ctx *snctx,
                     ret = extract_string(&pam_user, size, body, blen, &c);
                     if (ret != EOK) return ret;
 
-                    ret = sss_parse_name(pd, snctx, pam_user,
-                                         &pd->domain, &pd->user);
+                    ret = sss_parse_name_for_domains(pd, domains,
+                                                     default_domain, pam_user,
+                                                     &pd->domain, &pd->user);
                     if (ret != EOK) return ret;
                     break;
                 case SSS_PAM_ITEM_SERVICE:
@@ -207,13 +210,14 @@ static int pam_parse_in_data_v2(struct sss_names_ctx *snctx,
 
 }
 
-static int pam_parse_in_data_v3(struct sss_names_ctx *snctx,
-                             struct pam_data *pd,
-                             uint8_t *body, size_t blen)
+static int pam_parse_in_data_v3(struct sss_domain_info *domains,
+                                const char *default_domain,
+                                struct pam_data *pd,
+                                uint8_t *body, size_t blen)
 {
     int ret;
 
-    ret = pam_parse_in_data_v2(snctx, pd, body, blen);
+    ret = pam_parse_in_data_v2(domains, default_domain, pd, body, blen);
     if (ret != EOK) {
         DEBUG(1, ("pam_parse_in_data_v2 failed.\n"));
         return ret;
@@ -227,7 +231,8 @@ static int pam_parse_in_data_v3(struct sss_names_ctx *snctx,
     return EOK;
 }
 
-static int pam_parse_in_data(struct sss_names_ctx *snctx,
+static int pam_parse_in_data(struct sss_domain_info *domains,
+                             const char *default_domain,
                              struct pam_data *pd,
                              uint8_t *body, size_t blen)
 {
@@ -243,7 +248,8 @@ static int pam_parse_in_data(struct sss_names_ctx *snctx,
     for (start = end; end < last; end++) if (body[end] == '\0') break;
     if (body[end++] != '\0') return EINVAL;
 
-    ret = sss_parse_name(pd, snctx, (char *)&body[start], &pd->domain, &pd->user);
+    ret = sss_parse_name_for_domains(pd, domains, default_domain,
+                                     (char *)&body[start], &pd->domain, &pd->user);
     if (ret != EOK) return ret;
 
     for (start = end; end < last; end++) if (body[end] == '\0') break;
@@ -332,10 +338,10 @@ static errno_t set_last_login(struct pam_auth_req *preq)
         goto fail;
     }
 
-    ret = sysdb_get_ctx_from_list(preq->cctx->rctx->db_list, preq->domain,
-                                  &dbctx);
-    if (ret != EOK) {
+    dbctx = preq->domain->sysdb;
+    if (dbctx == NULL) {
         DEBUG(0, ("Fatal: Sysdb context not found for this domain!\n"));
+        ret = EINVAL;
         goto fail;
     }
 
@@ -355,7 +361,130 @@ fail:
     return ret;
 }
 
-static errno_t get_selinux_string(struct pam_auth_req *preq)
+#ifdef HAVE_SELINUX_LOGIN_DIR
+
+#define ALL_SERVICES "*"
+#define selogin_path(mem_ctx, username) \
+    talloc_asprintf(mem_ctx, "%s/logins/%s", selinux_policy_root(), username)
+
+static errno_t write_selinux_login_file(const char *username, char *string)
+{
+    char *path = NULL;
+    char *tmp_path = NULL;
+    ssize_t written;
+    int len;
+    int fd = -1;
+    mode_t oldmask;
+    TALLOC_CTX *tmp_ctx;
+    char *full_string = NULL;
+    errno_t ret = EOK;
+
+    len = strlen(string);
+    if (len == 0) {
+        return EINVAL;
+    }
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("talloc_new() failed\n"));
+        return ENOMEM;
+    }
+
+    path = selogin_path(tmp_ctx, username);
+    if (path == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    tmp_path = talloc_asprintf(tmp_ctx, "%sXXXXXX", path);
+    if (tmp_path == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    oldmask = umask(022);
+    fd = mkstemp(tmp_path);
+    umask(oldmask);
+    if (fd < 0) {
+        DEBUG(SSSDBG_OP_FAILURE, ("creating the temp file for SELinux "
+                                  "data failed. %s", tmp_path));
+        ret = EIO;
+        goto done;
+    }
+
+    full_string = talloc_asprintf(tmp_ctx, "%s:%s", ALL_SERVICES, string);
+    if (full_string == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    len = strlen(full_string);
+
+    errno = 0;
+    written = sss_atomic_write_s(fd, full_string, len);
+    if (written == -1) {
+        ret = errno;
+        DEBUG(SSSDBG_OP_FAILURE, ("writing to SELinux data file %s"
+                                  "failed [%d]: %s", tmp_path, ret,
+                                  strerror(ret)));
+        goto done;
+    }
+
+    if (written != len) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Expected to write %d bytes, wrote %d",
+                                  written, len));
+        ret = EIO;
+        goto done;
+    }
+
+    errno = 0;
+    if (rename(tmp_path, path) < 0) {
+        ret = errno;
+    } else {
+        ret = EOK;
+    }
+    fd = -1;
+
+done:
+    if (fd != -1) {
+        close(fd);
+        if (unlink(tmp_path) < 0) {
+            DEBUG(SSSDBG_MINOR_FAILURE, ("Could not remove file [%s]",
+                                         tmp_path));
+        }
+    }
+
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
+static errno_t remove_selinux_login_file(const char *username)
+{
+    char *path;
+    errno_t ret;
+
+    path = selogin_path(NULL, username);
+    if (!path) return ENOMEM;
+
+    errno = 0;
+    ret = unlink(path);
+    if (ret < 0) {
+        ret = errno;
+        if (ret == ENOENT) {
+            /* Just return success if the file was not there */
+            ret = EOK;
+        } else {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  ("Could not remove login file %s [%d]: %s\n",
+                   path, ret, strerror(ret)));
+        }
+    }
+
+    talloc_free(path);
+    return ret;
+}
+
+static errno_t process_selinux_mappings(struct pam_auth_req *preq)
 {
     struct sysdb_ctx *sysdb;
     TALLOC_CTX *tmp_ctx;
@@ -367,10 +496,10 @@ static errno_t get_selinux_string(struct pam_auth_req *preq)
     const char *tmp_str;
     char *order = NULL;
     char **order_array;
-    errno_t ret;
+    errno_t ret, err;
     int i, j;
     size_t order_count;
-    size_t len;
+    size_t len = 0;
 
     tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) {
@@ -378,9 +507,11 @@ static errno_t get_selinux_string(struct pam_auth_req *preq)
         goto done;
     }
 
-    ret = sysdb_get_ctx_from_list(preq->cctx->rctx->db_list,
-                                  preq->domain, &sysdb);
-    if (ret != EOK) {
+    sysdb = preq->domain->sysdb;
+    if (sysdb == NULL) {
+        DEBUG(SSSDBG_FATAL_FAILURE, ("Fatal: Sysdb CTX not found for "
+                                     "domain [%s]!\n", preq->domain->name));
+        ret = EINVAL;
         goto done;
     }
 
@@ -393,30 +524,33 @@ static errno_t get_selinux_string(struct pam_auth_req *preq)
         goto done;
     }
 
-    /* We need two values from the config object:
-     * - default SELinux user in case no other is available
-     * - the order for fetched usermaps
-     */
-    for (i = 0; i < config->num_elements; i++) {
-        if (strcasecmp(config->elements[i].name, SYSDB_SELINUX_DEFAULT_USER) == 0) {
-            default_user = (const char *)config->elements[i].values[0].data;
-        } else if (strcasecmp(config->elements[i].name, SYSDB_SELINUX_DEFAULT_ORDER) == 0) {
-            tmp_str = (char *)config->elements[i].values[0].data;
-            len = config->elements[i].values[0].length;
-            order = talloc_strdup(tmp_ctx, tmp_str);
-            if (order == NULL) {
-                goto done;
-            }
-        }
+    default_user = ldb_msg_find_attr_as_string(config,
+                                               SYSDB_SELINUX_DEFAULT_USER,
+                                               NULL);
+    if (!default_user || default_user[0] == '\0') {
+        /* Skip creating the maps altogether if there is no default
+         * or empty default
+         */
+        ret = EOK;
+        goto done;
     }
 
-    if (default_user == NULL || order == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, ("No default SELinux user "
-                                  "or map order given!\n"));
+    tmp_str = ldb_msg_find_attr_as_string(config,
+                                          SYSDB_SELINUX_DEFAULT_ORDER,
+                                          NULL);
+    if (tmp_str == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, ("No map order given!\n"));
         ret = EINVAL;
         goto done;
     }
 
+    order = talloc_strdup(tmp_ctx, tmp_str);
+    if (order == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+    len = strlen(order);
+
     /* The "order" string contains one or more SELinux user records
      * separated by $. Now we need to create an array of string from
      * this one string. First find out how many elements in the array
@@ -453,59 +587,56 @@ static errno_t get_selinux_string(struct pam_auth_req *preq)
         goto done;
     }
 
-    if (ret == ENOENT) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("No user maps found, using default!"));
-        file_content = talloc_strdup(tmp_ctx, default_user);
-        if (file_content == NULL) {
-            ret = ENOMEM;
-            goto done;
-        }
-    } else {
-        file_content = talloc_strdup(tmp_ctx, "");
-        if (file_content == NULL) {
-            ret = ENOMEM;
-            goto done;
-        }
+    /* If no maps match, we'll use the default SELinux user from the
+     * config */
+    file_content = talloc_strdup(tmp_ctx, default_user);
+    if (file_content == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
 
-        /* Iterate through the order array and try to find SELinux users
-         * in fetched maps. The order array contains all SELinux users
-         * allowed in the domain in the same order they should appear
-         * in the SELinux config file. If any user from the order array
-         * is not in fetched user maps, it means it should not be allowed
-         * for the user who is just logging in.
-         *
-         * Right now we have empty content of the SELinux config file,
-         * we shall add only those SELinux users that are present both in
-         * the order array and user maps applicable to the user who is
-         * logging in.
-         */
-        for (i = 0; i < order_count; i++) {
-            for (j = 0; usermaps[j] != NULL; j++) {
-                tmp_str = sss_selinux_map_get_seuser(usermaps[j]);
+    /* Iterate through the order array and try to find SELinux users
+     * in fetched maps. The order array contains all SELinux users
+     * allowed in the domain in the same order they should appear
+     * in the SELinux config file. If any user from the order array
+     * is not in fetched user maps, it means it should not be allowed
+     * for the user who is just logging in.
+     *
+     * Right now we have empty content of the SELinux config file,
+     * we shall add only those SELinux users that are present both in
+     * the order array and user maps applicable to the user who is
+     * logging in.
+     */
+    for (i = 0; i < order_count; i++) {
+        for (j = 0; usermaps[j] != NULL; j++) {
+            tmp_str = sss_selinux_map_get_seuser(usermaps[j]);
 
-                if (tmp_str && !strcasecmp(tmp_str, order_array[i])) {
-                    file_content = talloc_asprintf_append(file_content, "%s\n",
-                                                          tmp_str);
-                    if (file_content == NULL) {
-                        ret = ENOMEM;
-                        goto done;
-                    }
-                    break;
+            if (tmp_str && !strcasecmp(tmp_str, order_array[i])) {
+                /* If file_content contained something, overwrite it.
+                 * This record has higher priority.
+                 */
+                talloc_zfree(file_content);
+                file_content = talloc_strdup(tmp_ctx, tmp_str);
+                if (file_content == NULL) {
+                    ret = ENOMEM;
+                    goto done;
                 }
+                break;
             }
         }
     }
 
-    len = strlen(file_content);
-    if (len > 0) {
-        ret = pam_add_response(pd, SSS_PAM_SELINUX_MAP, len,
-                               (uint8_t *)file_content);
-    }
-
+    ret = write_selinux_login_file(pd->user, file_content);
 done:
+    if (!file_content) {
+        err = remove_selinux_login_file(pd->user);
+        /* Don't overwrite original error condition if there was one */
+        if (ret == EOK) ret = err;
+    }
     talloc_free(tmp_ctx);
     return ret;
 }
+#endif
 
 static errno_t filter_responses(struct confdb_ctx *cdb,
                                 struct response_data *resp_list)
@@ -514,18 +645,9 @@ static errno_t filter_responses(struct confdb_ctx *cdb,
     struct response_data *resp;
     uint32_t user_info_type;
     int64_t expire_date;
-    uint32_t expire_warn;
-    TALLOC_CTX *tmp_ctx;
     int pam_verbosity;
-    int pam_expiration_warning;
 
-    tmp_ctx = talloc_new(NULL);
-    if (tmp_ctx == NULL) {
-        DEBUG(1, ("talloc_new failed.\n"));
-        return ENOMEM;
-    }
-
-    ret = confdb_get_int(cdb, tmp_ctx, CONFDB_PAM_CONF_ENTRY,
+    ret = confdb_get_int(cdb, CONFDB_PAM_CONF_ENTRY,
                          CONFDB_PAM_VERBOSITY, DEFAULT_PAM_VERBOSITY,
                          &pam_verbosity);
     if (ret != EOK) {
@@ -533,20 +655,7 @@ static errno_t filter_responses(struct confdb_ctx *cdb,
         pam_verbosity = DEFAULT_PAM_VERBOSITY;
     }
 
-
-    ret = confdb_get_int(cdb, tmp_ctx, CONFDB_PAM_CONF_ENTRY,
-                         CONFDB_PAM_PWD_EXPIRATION_WARNING,
-                         DEFAULT_PAM_PWD_EXPIRATION_WARNING,
-                         &pam_expiration_warning);
-    if (ret != EOK) {
-        DEBUG(1, ("Failed to read PAM expiration warning, not fatal.\n"));
-        pam_expiration_warning = DEFAULT_PAM_PWD_EXPIRATION_WARNING;
-    }
-
-    talloc_free(tmp_ctx);
-
     resp = resp_list;
-
     while(resp != NULL) {
         if (resp->type == SSS_PAM_USER_INFO) {
             if (resp->len < sizeof(uint32_t)) {
@@ -579,18 +688,6 @@ static errno_t filter_responses(struct confdb_ctx *cdb,
                         resp->do_not_send_to_client = true;
                     }
 
-                    break;
-                case SSS_PAM_USER_INFO_EXPIRE_WARN:
-                    if (resp->len != 2 * sizeof(uint32_t)) {
-                        DEBUG(1, ("User info expire warning entry is "
-                                  "too short.\n"));
-                        return EINVAL;
-                    }
-                    memcpy(&expire_warn, resp->data + sizeof(uint32_t),
-                           sizeof(uint32_t));
-                    if(expire_warn > pam_expiration_warning * (60 * 60 * 24)) {
-                        resp->do_not_send_to_client = true;
-                    }
                     break;
                 default:
                     DEBUG(7, ("User info type [%d] not filtered.\n"));
@@ -617,8 +714,9 @@ static void pam_reply_delay(struct tevent_context *ev, struct tevent_timer *te,
     pam_reply(preq);
 }
 
-static void pam_handle_cached_login(struct pam_auth_req *preq, int ret,
-                                    time_t expire_date, time_t delayed_until);
+static int pam_forwarder(struct cli_ctx *cctx, int pam_cmd);
+static void pam_cache_auth_done(struct pam_auth_req *preq, int ret,
+                                time_t expire_date, time_t delayed_until);
 
 static void pam_reply(struct pam_auth_req *preq)
 {
@@ -644,7 +742,8 @@ static void pam_reply(struct pam_auth_req *preq)
     pctx = talloc_get_type(preq->cctx->rctx->pvt_ctx, struct pam_ctx);
 
 
-    DEBUG(4, ("pam_reply get called.\n"));
+    DEBUG(SSSDBG_FUNC_DATA,
+          ("pam_reply called with result [%d].\n", pd->pam_status));
 
     if (pd->pam_status == PAM_AUTHINFO_UNAVAIL) {
         switch(pd->cmd) {
@@ -656,9 +755,8 @@ static void pam_reply(struct pam_auth_req *preq)
                     /* do auth with offline credentials */
                     pd->offline_auth = true;
 
-                    ret = sysdb_get_ctx_from_list(preq->cctx->rctx->db_list,
-                                                  preq->domain, &sysdb);
-                    if (ret != EOK) {
+                    sysdb = preq->domain->sysdb;
+                    if (sysdb == NULL) {
                         DEBUG(0, ("Fatal: Sysdb CTX not found for "
                                   "domain [%s]!\n", preq->domain->name));
                         goto done;
@@ -669,7 +767,7 @@ static void pam_reply(struct pam_auth_req *preq)
                                            pctx->rctx->cdb, false,
                                            &exp_date, &delay_until);
 
-                    pam_handle_cached_login(preq, ret, exp_date, delay_until);
+                    pam_cache_auth_done(preq, ret, exp_date, delay_until);
                     return;
             }
             break;
@@ -735,15 +833,17 @@ static void pam_reply(struct pam_auth_req *preq)
         return;
     }
 
-    if (pd->cmd == SSS_PAM_OPEN_SESSION &&
+#ifdef HAVE_SELINUX_LOGIN_DIR
+    if (pd->cmd == SSS_PAM_ACCT_MGMT &&
         pd->pam_status == PAM_SUCCESS) {
         /* Try to fetch data from sysdb
          * (auth already passed -> we should have them) */
-        ret = get_selinux_string(preq);
+        ret = process_selinux_mappings(preq);
         if (ret != EOK) {
             pd->pam_status = PAM_SYSTEM_ERR;
         }
     }
+#endif
 
     ret = sss_packet_new(cctx->creq, 0, sss_packet_get_cmd(cctx->creq->in),
                          &cctx->creq->out);
@@ -812,18 +912,18 @@ done:
     sss_cmd_done(cctx, preq);
 }
 
-static void pam_handle_cached_login(struct pam_auth_req *preq, int ret,
-                                    time_t expire_date, time_t delayed_until)
+static void pam_cache_auth_done(struct pam_auth_req *preq, int ret,
+                                time_t expire_date, time_t delayed_until)
 {
     uint32_t resp_type;
     size_t resp_len;
     uint8_t *resp;
     int64_t dummy;
 
-    preq->pd->pam_status = cached_login_pam_status(ret);
+    switch (ret) {
+        case EOK:
+            preq->pd->pam_status = PAM_SUCCESS;
 
-    switch (preq->pd->pam_status) {
-        case PAM_SUCCESS:
             resp_type = SSS_PAM_USER_INFO_OFFLINE_AUTH;
             resp_len = sizeof(uint32_t) + sizeof(int64_t);
             resp = talloc_size(preq->pd, resp_len);
@@ -840,7 +940,14 @@ static void pam_handle_cached_login(struct pam_auth_req *preq, int ret,
                 }
             }
             break;
-        case PAM_PERM_DENIED:
+        case ENOENT:
+            preq->pd->pam_status = PAM_AUTHINFO_UNAVAIL;
+            break;
+        case EINVAL:
+            preq->pd->pam_status = PAM_AUTH_ERR;
+            break;
+        case EACCES:
+            preq->pd->pam_status = PAM_PERM_DENIED;
             if (delayed_until >= 0) {
                 resp_type = SSS_PAM_USER_INFO_OFFLINE_AUTH_DELAYED;
                 resp_len = sizeof(uint32_t) + sizeof(int64_t);
@@ -860,14 +967,14 @@ static void pam_handle_cached_login(struct pam_auth_req *preq, int ret,
             }
             break;
         default:
-            DEBUG(SSSDBG_TRACE_LIBS,
-                  ("cached login returned: %d\n", preq->pd->pam_status));
+            preq->pd->pam_status = PAM_SYSTEM_ERR;
     }
 
     pam_reply(preq);
     return;
 }
 
+static void pam_forwarder_cb(struct tevent_req *req);
 static void pam_check_user_dp_callback(uint16_t err_maj, uint32_t err_min,
                                        const char *err_msg, void *ptr);
 static int pam_check_user_search(struct pam_auth_req *preq);
@@ -878,18 +985,58 @@ static void pam_dom_forwarder(struct pam_auth_req *preq);
  * PAM_ENVIRONMENT, so that we can save performing some calls and cache
  * data. */
 
+errno_t pam_forwarder_parse_data(struct cli_ctx *cctx, struct pam_data *pd)
+{
+    uint8_t *body;
+    size_t blen;
+    errno_t ret;
+    uint32_t terminator = SSS_END_OF_PAM_REQUEST;
+
+    sss_packet_get_body(cctx->creq->in, &body, &blen);
+    if (blen >= sizeof(uint32_t) &&
+        memcmp(&body[blen - sizeof(uint32_t)], &terminator, sizeof(uint32_t)) != 0) {
+        DEBUG(1, ("Received data not terminated.\n"));
+        ret = EINVAL;
+        goto done;
+    }
+
+    switch (cctx->cli_protocol_version->version) {
+        case 1:
+            ret = pam_parse_in_data(cctx->rctx->domains,
+                                    cctx->rctx->default_domain, pd,
+                                    body, blen);
+            break;
+        case 2:
+            ret = pam_parse_in_data_v2(cctx->rctx->domains,
+                                       cctx->rctx->default_domain, pd,
+                                       body, blen);
+            break;
+        case 3:
+            ret = pam_parse_in_data_v3(cctx->rctx->domains,
+                                       cctx->rctx->default_domain, pd,
+                                       body, blen);
+            break;
+        default:
+            DEBUG(1, ("Illegal protocol version [%d].\n",
+                      cctx->cli_protocol_version->version));
+            ret = EINVAL;
+    }
+
+done:
+    return ret;
+}
+
 static int pam_forwarder(struct cli_ctx *cctx, int pam_cmd)
 {
     struct sss_domain_info *dom;
     struct pam_auth_req *preq;
     struct pam_data *pd;
-    uint8_t *body;
-    size_t blen;
     int ret;
     errno_t ncret;
     struct pam_ctx *pctx =
             talloc_get_type(cctx->rctx->pvt_ctx, struct pam_ctx);
-    uint32_t terminator = SSS_END_OF_PAM_REQUEST;
+    struct tevent_req *req;
+
     preq = talloc_zero(cctx, struct pam_auth_req);
     if (!preq) {
         return ENOMEM;
@@ -903,46 +1050,32 @@ static int pam_forwarder(struct cli_ctx *cctx, int pam_cmd)
     }
     pd = preq->pd;
 
-    sss_packet_get_body(cctx->creq->in, &body, &blen);
-    if (blen >= sizeof(uint32_t) &&
-        memcmp(&body[blen - sizeof(uint32_t)], &terminator, sizeof(uint32_t)) != 0) {
-        DEBUG(1, ("Received data not terminated.\n"));
-        ret = EINVAL;
-        goto done;
-    }
-
     pd->cmd = pam_cmd;
     pd->priv = cctx->priv;
 
-    switch (cctx->cli_protocol_version->version) {
-        case 1:
-            ret = pam_parse_in_data(cctx->rctx->names, pd, body, blen);
-            break;
-        case 2:
-            ret = pam_parse_in_data_v2(cctx->rctx->names, pd, body, blen);
-            break;
-        case 3:
-            ret = pam_parse_in_data_v3(cctx->rctx->names, pd, body, blen);
-            break;
-        default:
-            DEBUG(1, ("Illegal protocol version [%d].\n",
-                      cctx->cli_protocol_version->version));
-            ret = EINVAL;
-    }
-    if (ret != EOK) {
+    ret = pam_forwarder_parse_data(cctx, pd);
+    if (ret == EAGAIN) {
+        req = sss_dp_get_domains_send(cctx->rctx, cctx->rctx, true, pd->domain);
+        if (req == NULL) {
+            ret = ENOMEM;
+        } else {
+            tevent_req_set_callback(req, pam_forwarder_cb, preq);
+            ret = EAGAIN;
+        }
+        goto done;
+    } else if (ret != EOK) {
         ret = EINVAL;
         goto done;
     }
 
     /* now check user is valid */
     if (pd->domain) {
-        preq->domain = responder_get_domain(cctx->rctx->domains, pd->domain);
+        preq->domain = responder_get_domain(preq, cctx->rctx, pd->domain);
         if (!preq->domain) {
             ret = ENOENT;
             goto done;
         }
-    }
-    else {
+    } else {
         for (dom = preq->cctx->rctx->domains; dom; dom = dom->next) {
             if (dom->fqnames) continue;
 
@@ -984,12 +1117,50 @@ done:
     return pam_check_user_done(preq, ret);
 }
 
+static void pam_forwarder_cb(struct tevent_req *req)
+{
+    struct pam_auth_req *preq = tevent_req_callback_data(req,
+                                                         struct pam_auth_req);
+    struct cli_ctx *cctx = preq->cctx;
+    struct pam_data *pd;
+    errno_t ret = EOK;
+
+    ret = sss_dp_get_domains_recv(req);
+    talloc_free(req);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    pd = preq->pd;
+
+    ret = pam_forwarder_parse_data(cctx, pd);
+    if (ret != EOK) {
+        ret = EINVAL;
+        goto done;
+    }
+
+    if (preq->pd->domain) {
+        preq->domain = responder_get_domain(preq, cctx->rctx, preq->pd->domain);
+        if (preq->domain == NULL) {
+            ret = ENOENT;
+            goto done;
+        }
+    }
+
+    ret = pam_check_user_search(preq);
+    if (ret == EOK) {
+        pam_dom_forwarder(preq);
+    }
+
+done:
+    pam_check_user_done(preq, ret);
+}
+
 static void pam_dp_send_acct_req_done(struct tevent_req *req);
 
 static int pam_check_user_search(struct pam_auth_req *preq)
 {
     struct sss_domain_info *dom = preq->domain;
-    struct cli_ctx *cctx = preq->cctx;
     char *name = NULL;
     struct sysdb_ctx *sysdb;
     time_t cacheExpire;
@@ -1044,8 +1215,8 @@ static int pam_check_user_search(struct pam_auth_req *preq)
 
         DEBUG(4, ("Requesting info for [%s@%s]\n", name, dom->name));
 
-        ret = sysdb_get_ctx_from_list(cctx->rctx->db_list, dom, &sysdb);
-        if (ret != EOK) {
+        sysdb = dom->sysdb;
+        if (sysdb == NULL) {
             DEBUG(0, ("Fatal: Sysdb CTX not found for this domain!\n"));
             preq->pd->pam_status = PAM_SYSTEM_ERR;
             return EFAULT;
@@ -1136,7 +1307,8 @@ static int pam_check_user_search(struct pam_auth_req *preq)
         return EAGAIN;
     }
 
-    DEBUG(2, ("No matching domain found for [%s], fail!\n", name));
+    DEBUG(SSSDBG_MINOR_FAILURE,
+          ("No matching domain found for [%s], fail!\n", preq->pd->user));
     return ENOENT;
 }
 
