@@ -31,11 +31,13 @@
 
 struct ssh_ctx {
     struct resp_ctx *rctx;
+
+    bool hash_known_hosts;
+    int known_hosts_timeout;
 };
 
 struct ssh_cmd_ctx {
     struct cli_ctx *cctx;
-    enum sss_dp_acct_type type;
     char *name;
     char *alias;
     char *domname;
@@ -43,10 +45,24 @@ struct ssh_cmd_ctx {
     struct sss_domain_info *domain;
     bool check_next;
 
-    struct ldb_message **results;
-    size_t results_len;
+    struct ldb_message *result;
 };
 
 struct sss_cmd_table *get_ssh_cmds(void);
 
+struct tevent_req *
+sss_dp_get_ssh_host_send(TALLOC_CTX *mem_ctx,
+                         struct resp_ctx *rctx,
+                         struct sss_domain_info *dom,
+                         bool fast_reply,
+                         const char *name,
+                         const char *alias);
+
+errno_t
+sss_dp_get_ssh_host_recv(TALLOC_CTX *mem_ctx,
+                         struct tevent_req *req,
+                         dbus_uint16_t *dp_err,
+                         dbus_uint32_t *dp_ret,
+                         char **err_msg);
+
 #endif /* _SSHSRV_PRIVATE_H_ */
