@@ -114,7 +114,7 @@ int sudo_process_init(TALLOC_CTX *mem_ctx,
     sudo_ctx->rctx->pvt_ctx = sudo_ctx;
 
     /* Enable automatic reconnection to the Data Provider */
-    ret = confdb_get_int(sudo_ctx->rctx->cdb, sudo_ctx->rctx,
+    ret = confdb_get_int(sudo_ctx->rctx->cdb,
                          CONFDB_SUDO_CONF_ENTRY,
                          CONFDB_SERVICE_RECON_RETRIES,
                          3, &max_retries);
@@ -131,19 +131,8 @@ int sudo_process_init(TALLOC_CTX *mem_ctx,
 
     /* Get responder options */
 
-    /* Get cache_timeout option */
-    ret = confdb_get_int(sudo_ctx->rctx->cdb, sudo_ctx,
-                         CONFDB_SUDO_CONF_ENTRY, CONFDB_SUDO_CACHE_TIMEOUT,
-                         CONFDB_DEFAULT_SUDO_CACHE_TIMEOUT,
-                         &sudo_ctx->cache_timeout);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_FATAL_FAILURE, ("Error reading from confdb (%d) [%s]\n",
-              ret, strerror(ret)));
-        return ret;
-    }
-
     /* Get sudo_timed option */
-    ret = confdb_get_bool(sudo_ctx->rctx->cdb, sudo_ctx,
+    ret = confdb_get_bool(sudo_ctx->rctx->cdb,
                           CONFDB_SUDO_CONF_ENTRY, CONFDB_SUDO_TIMED,
                           CONFDB_DEFAULT_SUDO_TIMED,
                           &sudo_ctx->timed);
@@ -153,14 +142,6 @@ int sudo_process_init(TALLOC_CTX *mem_ctx,
         return ret;
     }
 
-    /* Initialize in-memory cache */
-    ret = sudosrv_cache_init(sudo_ctx, 10, &sudo_ctx->cache);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_FATAL_FAILURE,
-              ("Could not create hash table: [%s]", strerror(ret)));
-        return ret;
-    }
-
     DEBUG(SSSDBG_TRACE_FUNC, ("SUDO Initialization complete\n"));
 
     return EOK;
