@@ -78,8 +78,14 @@ static errno_t sudosrv_cmd_send_error(TALLOC_CTX *mem_ctx,
     size_t response_len = 0;
     int ret = EOK;
 
-    ret = sudosrv_response_append_uint32(mem_ctx, error,
-                                         &response_body, &response_len);
+    if (error == EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE, ("Everything is fine but we are "
+              "returning error?\n"));
+        return EFAULT;
+    }
+
+    ret = sudosrv_build_response(mem_ctx, error, NULL, 0, NULL,
+                                 &response_body, &response_len);
     if (ret != EOK) {
         return ret;
     }
@@ -87,25 +93,22 @@ static errno_t sudosrv_cmd_send_error(TALLOC_CTX *mem_ctx,
     return sudosrv_cmd_send_reply(cmd_ctx, response_body, response_len);
 }
 
-errno_t sudosrv_cmd_done(struct sudo_dom_ctx *dctx, int ret)
+errno_t sudosrv_cmd_done(struct sudo_cmd_ctx *cmd_ctx, int ret)
 {
     uint8_t *response_body = NULL;
     size_t response_len = 0;
-    size_t num_rules = dctx->res_count;
-    struct sysdb_attrs **rules = dctx->res;
+    size_t num_rules = cmd_ctx->num_rules;
+    struct sysdb_attrs **rules = cmd_ctx->rules;
 
     switch (ret) {
     case EOK:
         /*
-         * Parent of dctx->res is in-memory cache, we must not talloc_free it!
+         * Parent of cmd_ctx->rules is in-memory cache, we must not talloc_free it!
          */
-        if (!dctx->cmd_ctx->sudo_ctx->timed) {
-            num_rules = dctx->res_count;
-            rules = dctx->res;
-        } else {
+        if (cmd_ctx->sudo_ctx->timed) {
             /* filter rules by time */
-            ret = sysdb_sudo_filter_rules_by_time(dctx, dctx->res_count,
-                                                  dctx->res, 0,
+            ret = sysdb_sudo_filter_rules_by_time(cmd_ctx, cmd_ctx->num_rules,
+                                                  cmd_ctx->rules, 0,
                                                   &num_rules, &rules);
             if (ret != EOK) {
                 return EFAULT;
@@ -113,14 +116,14 @@ errno_t sudosrv_cmd_done(struct sudo_dom_ctx *dctx, int ret)
         }
 
         /* send result */
-        ret = sudosrv_get_sudorules_build_response(dctx->cmd_ctx, SSS_SUDO_ERROR_OK,
-                                                   num_rules, rules,
-                                                   &response_body, &response_len);
+        ret = sudosrv_build_response(cmd_ctx, SSS_SUDO_ERROR_OK,
+                                     cmd_ctx->domain->name, num_rules, rules,
+                                     &response_body, &response_len);
         if (ret != EOK) {
             return EFAULT;
         }
 
-        ret = sudosrv_cmd_send_reply(dctx->cmd_ctx, response_body, response_len);
+        ret = sudosrv_cmd_send_reply(cmd_ctx, response_body, response_len);
         break;
 
     case EAGAIN:
@@ -139,55 +142,55 @@ errno_t sudosrv_cmd_done(struct sudo_dom_ctx *dctx, int ret)
 
     default:
         /* send error */
-        ret = sudosrv_cmd_send_error(dctx->cmd_ctx, dctx->cmd_ctx, ret);
+        ret = sudosrv_cmd_send_error(cmd_ctx, cmd_ctx, ret);
         break;
     }
 
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("Fatal error, killing connection!\n"));
-        talloc_free(dctx->cmd_ctx->cli_ctx);
+        talloc_free(cmd_ctx->cli_ctx);
         return EFAULT;
     }
 
     return EOK;
 }
 
-static int sudosrv_cmd_get_sudorules(struct cli_ctx *cli_ctx)
+static int sudosrv_cmd(enum sss_sudo_type type, struct cli_ctx *cli_ctx)
 {
-    char *rawname = NULL;
-    char *domname = NULL;
+    struct sudo_cmd_ctx *cmd_ctx = NULL;
+    struct sudo_dom_ctx *dom_ctx = NULL;
     uint8_t *query_body = NULL;
     size_t query_len = 0;
-    int ret = EOK;
-    struct sudo_cmd_ctx *cmd_ctx = NULL;
-    struct sudo_dom_ctx *dctx = NULL;
+    errno_t ret;
+
+    /* create cmd_ctx */
 
     cmd_ctx = talloc_zero(cli_ctx, struct sudo_cmd_ctx);
-    if (!cmd_ctx) {
+    if (cmd_ctx == NULL) {
         /* kill the connection here as we have no context for reply */
         DEBUG(SSSDBG_FATAL_FAILURE, ("Out of memory?\n"));
         return ENOMEM;
     }
-    cmd_ctx->cli_ctx = cli_ctx;
-    cmd_ctx->type = SSS_DP_SUDO_USER;
 
-    /* get responder ctx */
+    cmd_ctx->domain = NULL;
+    cmd_ctx->cli_ctx = cli_ctx;
+    cmd_ctx->type = type;
     cmd_ctx->sudo_ctx = talloc_get_type(cli_ctx->rctx->pvt_ctx, struct sudo_ctx);
-    if (!cmd_ctx->sudo_ctx) {
+    if (cmd_ctx->sudo_ctx == NULL) {
         DEBUG(SSSDBG_FATAL_FAILURE, ("sudo_ctx not set, killing connection!\n"));
         return EFAULT;
     }
 
-    /* create domain ctx */
-    dctx = talloc_zero(cmd_ctx, struct sudo_dom_ctx);
-    if (!dctx) {
-        return sudosrv_cmd_send_error(cmd_ctx, cmd_ctx, ENOMEM);
+    /* if protocol is invalid return */
+    if (cli_ctx->cli_protocol_version->version != 1) {
+        DEBUG(SSSDBG_FATAL_FAILURE, ("Invalid protocol! [%d]\n",
+              cli_ctx->cli_protocol_version->version));
+        ret = EFAULT;
+        goto done;
     }
-    dctx->cmd_ctx = cmd_ctx;
-    dctx->orig_username = NULL;
-    dctx->cased_username = NULL;
 
-    /* get query */
+    /* parse query */
+
     sss_packet_get_body(cli_ctx->creq->in, &query_body, &query_len);
     if (query_len <= 0 || query_body == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("Query is empty\n"));
@@ -195,133 +198,61 @@ static int sudosrv_cmd_get_sudorules(struct cli_ctx *cli_ctx)
         goto done;
     }
 
-    /* If the body isn't valid UTF-8, fail */
-    if (!sss_utf8_check(query_body, query_len - 1)) {
-        DEBUG(SSSDBG_CRIT_FAILURE, ("Supplied data is not valid UTF-8 string\n"));
-        ret = EINVAL;
-        goto done;
-    }
-
-    /* parse query */
-    rawname = sudosrv_get_sudorules_parse_query(cmd_ctx,
-                                                (const char*)query_body,
-                                                 query_len);
-    if (rawname == NULL) {
-        ret = ENOMEM;
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              ("Unable to parse query: %s\n", strerror(ret)));
-        goto done;
-    }
-
-    domname = NULL;
-    ret = sss_parse_name(cmd_ctx, cli_ctx->rctx->names, rawname,
-                         &domname, &cmd_ctx->username);
+    ret = sudosrv_parse_query(cmd_ctx, cli_ctx->rctx,
+                              query_body, query_len,
+                              &cmd_ctx->uid, &cmd_ctx->username, &cmd_ctx->domain);
     if (ret != EOK) {
-        DEBUG(2, ("Invalid name received [%s]\n", rawname));
-        ret = ENOENT;
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Invalid query: %s\n", strerror(ret)));
         goto done;
     }
 
-    DEBUG(SSSDBG_FUNC_DATA, ("Requesting sudo rules for [%s] from [%s]\n",
-          cmd_ctx->username, domname ? domname : "<ALL>"));
+    cmd_ctx->check_next = cmd_ctx->domain == NULL;
 
-    if (domname) {
-        dctx->domain = responder_get_domain(cli_ctx->rctx->domains, domname);
-        if (!dctx->domain) {
-            ret = ENOENT;
-            goto done;
-        }
-    } else {
-        /* this is a multidomain search */
-        dctx->domain = cli_ctx->rctx->domains;
-        cmd_ctx->check_next = true;
+    switch (cmd_ctx->type) {
+        case SSS_SUDO_DEFAULTS:
+            DEBUG(SSSDBG_FUNC_DATA, ("Requesting default options "
+                  "for [%s] from [%s]\n", cmd_ctx->username,
+                  cmd_ctx->domain ? cmd_ctx->domain->name : "<ALL>"));
+            break;
+        case SSS_SUDO_USER:
+            DEBUG(SSSDBG_FUNC_DATA, ("Requesting rules "
+                  "for [%s] from [%s]\n", cmd_ctx->username,
+                  cmd_ctx->domain ? cmd_ctx->domain->name : "<ALL>"));
+            break;
     }
 
-    /* try to find rules in in-memory cache */
-    ret = sudosrv_cache_lookup(cmd_ctx->sudo_ctx->cache, dctx,
-                               cmd_ctx->check_next, cmd_ctx->username,
-                               &dctx->res_count, &dctx->res);
-    if (ret == EOK) {
-        /* cache hit */
-        DEBUG(SSSDBG_FUNC_DATA, ("Returning rules for [%s@%s] "
-              "from in-memory cache\n", cmd_ctx->username, dctx->domain->name));
-    } else if (ret == ENOENT) {
-        /* cache expired or missed */
-        ret = sudosrv_get_sudorules(dctx);
-    } /* else error */
+    /* create domain ctx */
+
+    dom_ctx = talloc_zero(cmd_ctx, struct sudo_dom_ctx);
+    if (dom_ctx == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+    dom_ctx->cmd_ctx = cmd_ctx;
+    dom_ctx->domain = cmd_ctx->domain != NULL ? cmd_ctx->domain
+                                              : cli_ctx->rctx->domains;
+
+    ret = sudosrv_get_sudorules(dom_ctx);
 
 done:
-    return sudosrv_cmd_done(dctx, ret);
+    return sudosrv_cmd_done(cmd_ctx, ret);
+}
+
+
+static int sudosrv_cmd_get_sudorules(struct cli_ctx *cli_ctx)
+{
+    return sudosrv_cmd(SSS_SUDO_USER, cli_ctx);
 }
 
 static int sudosrv_cmd_get_defaults(struct cli_ctx *cli_ctx)
 {
-    int ret = EOK;
-    struct sudo_cmd_ctx *cmd_ctx = NULL;
-    struct sudo_dom_ctx *dctx = NULL;
-
-    cmd_ctx = talloc_zero(cli_ctx, struct sudo_cmd_ctx);
-    if (!cmd_ctx) {
-        /* kill the connection here as we have no context for reply */
-        DEBUG(SSSDBG_FATAL_FAILURE, ("Out of memory?\n"));
-        return ENOMEM;
-    }
-    cmd_ctx->cli_ctx = cli_ctx;
-    cmd_ctx->type = SSS_DP_SUDO_DEFAULTS;
-    cmd_ctx->username = NULL;
-    cmd_ctx->check_next = false;
-
-    /* get responder ctx */
-    cmd_ctx->sudo_ctx = talloc_get_type(cli_ctx->rctx->pvt_ctx, struct sudo_ctx);
-    if (!cmd_ctx->sudo_ctx) {
-        DEBUG(SSSDBG_FATAL_FAILURE, ("sudo_ctx not set, killing connection!\n"));
-        return EFAULT;
-    }
-
-    /* create domain ctx */
-    dctx = talloc_zero(cmd_ctx, struct sudo_dom_ctx);
-    if (!dctx) {
-        return sudosrv_cmd_send_error(cmd_ctx, cmd_ctx, ENOMEM);
-    }
-    dctx->cmd_ctx = cmd_ctx;
-    dctx->orig_username = NULL;
-    dctx->cased_username = NULL;
-
-    DEBUG(SSSDBG_FUNC_DATA, ("Requesting cn=defaults\n"));
-
-    /* sudo currently does not support domain selection
-     * so find first available domain
-     * TODO - support domain selection */
-    dctx->domain = cli_ctx->rctx->domains;
-    while (dctx->domain && dctx->domain->fqnames) {
-        dctx->domain = dctx->domain->next;
-    }
-    if (!dctx->domain) {
-        DEBUG(SSSDBG_MINOR_FAILURE, ("No valid domain found\n"));
-        ret = ENOENT;
-        goto done;
-    }
-
-    /* try to find rules in in-memory cache */
-    ret = sudosrv_cache_lookup(cmd_ctx->sudo_ctx->cache, dctx,
-                               cmd_ctx->check_next, cmd_ctx->username,
-                               &dctx->res_count, &dctx->res);
-    if (ret == EOK) {
-        /* cache hit */
-        DEBUG(SSSDBG_FUNC_DATA, ("Returning defaults settings for [%s] "
-                                 "from in-memory cache\n", dctx->domain->name));
-    } else if (ret == ENOENT) {
-        /* cache expired or missed */
-        ret = sudosrv_get_rules(dctx);
-    } /* else error */
-
-done:
-    return sudosrv_cmd_done(dctx, ret);
+    return sudosrv_cmd(SSS_SUDO_DEFAULTS, cli_ctx);
 }
 
 struct cli_protocol_version *register_cli_protocol_version(void)
 {
     static struct cli_protocol_version sudo_cli_protocol_version[] = {
+		{1, "2012-05-14", "require uid and domain"},
         {0, NULL, NULL}
     };
 
