@@ -41,12 +41,10 @@
 #include <security/pam_modules.h>
 #include <security/pam_ext.h>
 #include <security/pam_modutil.h>
-#ifdef HAVE_SELINUX
-#include <selinux/selinux.h>
-#endif
 #include "sss_pam_macros.h"
 
 #include "sss_cli.h"
+#include "util/atomic_io.h"
 
 #include <libintl.h>
 #define _(STRING) dgettext (PACKAGE, STRING)
@@ -56,8 +54,6 @@
 #define FLAGS_USE_AUTHTOK    (1 << 2)
 
 #define PWEXP_FLAG "pam_sss:password_expired_flag"
-#define ALL_SERVICES "*:"
-#define ALL_SERVICES_LEN 2
 
 #define PW_RESET_MSG_FILENAME_TEMPLATE SSSD_CONF_DIR"/customize/%s/pam_sss_pw_reset_message.%s"
 #define PW_RESET_MSG_MAX_SIZE 4096
@@ -86,7 +82,6 @@ struct pam_items {
     pid_t cli_pid;
     const char *login_name;
     char *domain_name;
-    char *selinux_user;
 };
 
 #define DEBUG_MGS_LEN 1024
@@ -501,17 +496,12 @@ static errno_t display_pw_reset_message(pam_handle_t *pamh,
         goto done;
     }
 
-
-    total_len = 0;
-    while (total_len < stat_buf.st_size) {
-        ret = read(fd, msg_buf + total_len, stat_buf.st_size - total_len);
-        if (ret == -1) {
-            if (errno == EINTR || errno == EAGAIN) continue;
-            ret = errno;
-            D(("read failed [%d][%s].", ret, strerror(ret)));
-            goto done;
-        }
-        total_len += ret;
+    errno = 0;
+    total_len = sss_atomic_read_s(fd, msg_buf, stat_buf.st_size);
+    if (ret == -1) {
+        ret = errno;
+        D(("read failed [%d][%s].", ret, strerror(ret)));
+        goto done;
     }
 
     ret = close(fd);
@@ -643,7 +633,7 @@ static int user_info_grace_login(pam_handle_t *pamh,
     memcpy(&grace, buf + sizeof(uint32_t), sizeof(uint32_t));
     ret = snprintf(user_msg, sizeof(user_msg),
                    _("Your password has expired. "
-                     "You have %d grace login(s) remaining."),
+                     "You have %1$d grace login(s) remaining."),
                    grace);
     if (ret < 0 || ret >= sizeof(user_msg)) {
         D(("snprintf failed."));
@@ -688,7 +678,7 @@ static int user_info_expire_warn(pam_handle_t *pamh,
     }
 
     ret = snprintf(user_msg, sizeof(user_msg),
-                   _("Your password will expire in %d %s."), expire, unit);
+                   _("Your password will expire in %1$d %2$s."), expire, unit);
     if (ret < 0 || ret >= sizeof(user_msg)) {
         D(("snprintf failed."));
         return PAM_SYSTEM_ERR;
@@ -971,17 +961,6 @@ static int eval_response(pam_handle_t *pamh, size_t buflen, uint8_t *buf,
                     D(("do_pam_conversation failed."));
                 }
                 break;
-            case SSS_PAM_SELINUX_MAP:
-                if (pi->selinux_user) {
-                    free(pi->selinux_user);
-                }
-                pi->selinux_user = (char *)malloc(len + 1);
-                if (!pi->selinux_user) {
-                    D(("Insufficient memory."));
-                    return PAM_SYSTEM_ERR;
-                }
-                memcpy(pi->selinux_user, &buf[p], len + 1);
-                break;
             default:
                 D(("Unknown response type [%d]", type));
         }
@@ -1003,7 +982,6 @@ static int get_pam_items(pam_handle_t *pamh, struct pam_items *pi)
     pi->pam_newauthtok_type = SSS_AUTHTOK_TYPE_EMPTY;
     pi->pam_newauthtok = NULL;
     pi->pam_newauthtok_size = 0;
-    pi->selinux_user = NULL;
 
     ret = pam_get_item(pamh, PAM_SERVICE, (const void **) &(pi->pam_service));
     if (ret != PAM_SUCCESS) return ret;
@@ -1087,15 +1065,6 @@ static int send_and_receive(pam_handle_t *pamh, struct pam_items *pi,
     size_t replen;
     int pam_status = PAM_SYSTEM_ERR;
 
-#ifdef HAVE_SELINUX
-    char *path = NULL;
-    char *tmp_path = NULL;
-    int pos, len;
-    char *services = NULL;
-    int fd;
-    mode_t oldmask;
-#endif /* HAVE_SELINUX */
-
     print_pam_items(pi);
 
     ret = pack_message_v3(pi, &rd.len, &buf);
@@ -1183,74 +1152,7 @@ static int send_and_receive(pam_handle_t *pamh, struct pam_items *pi,
                           pi->pam_user, pam_status,
                           pam_strerror(pamh,pam_status));
                 }
-            } else {
-	        if (pi->selinux_user == NULL) {
-		    pam_status = PAM_SUCCESS;
-		    break;
-		}
-
-#ifdef HAVE_SELINUX
-		if (asprintf(&path, "%s/logins/%s", selinux_policy_root(),
-			     pi->pam_user) <  0 ||
-		    asprintf(&tmp_path, "%sXXXXXX", path) < 0) {
-		    pam_status = PAM_SYSTEM_ERR;
-		    goto done;
-		}
-
-		oldmask = umask(022);
-		fd = mkstemp(tmp_path);
-		umask(oldmask);
-		if (fd < 0) {
-		    logger(pamh, LOG_ERR, "creating the temp file for SELinux "
-			   "data failed. %s", tmp_path);
-		    pam_status = PAM_SYSTEM_ERR;
-		    goto done;
-		}
-
-		/* First write filter for all services */
-		services = strdup(ALL_SERVICES);
-		if (services == NULL) {
-		    pam_status = PAM_SYSTEM_ERR;
-		    goto done;
-		}
-
-		pos = 0;
-		len = ALL_SERVICES_LEN;
-		while (pos < len) {
-		    errno = 0;
-		    ret = write(fd, services + pos, len-pos);
-		    if (ret < 0) {
-		        if (errno != EINTR) {
-			    logger(pamh, LOG_ERR, "writing to SELinux data file "
-				   "failed. %s", tmp_path);
-			    pam_status = PAM_SYSTEM_ERR;
-			    goto done;
-			}
-			continue;
-		    }
-		    pos += ret;
-		}
-
-		pos = 0;
-		len = strlen(pi->selinux_user);
-		while (pos < len) {
-		    ret = write(fd, pi->selinux_user + pos, len-pos);
-		    if (ret < 0) {
-		        if (errno != EINTR) {
-			  logger(pamh, LOG_ERR, "writing to SELinux data file "
-				 "failed. %s", tmp_path);
-			  pam_status = PAM_SYSTEM_ERR;
-			  goto done;
-			}
-			continue;
-		    }
-		    pos += ret;
-		}
-		close(fd);
-
-		rename(tmp_path, path);
-#endif /* HAVE_SELINUX */
-	    }
+            }
             break;
         case SSS_PAM_OPEN_SESSION:
         case SSS_PAM_SETCRED:
@@ -1267,11 +1169,6 @@ done:
         free(buf);
     }
     free(repbuf);
-#ifdef HAVE_SELINUX
-    free(path);
-    free(tmp_path);
-    free(services);
-#endif /* HAVE_SELINUX */
 
     return pam_status;
 }
