@@ -114,11 +114,11 @@ START_TEST(test_fo_new_service)
         sprintf(buf, "service_%d", i);
 
         check_leaks_push(ctx);
-        ret = fo_new_service(ctx->fo_ctx, buf, &services[i]);
+        ret = fo_new_service(ctx->fo_ctx, buf, NULL, &services[i]);
         fail_if(ret != EOK);
     }
 
-    ret = fo_new_service(ctx->fo_ctx, "service_3", &service);
+    ret = fo_new_service(ctx->fo_ctx, "service_3", NULL, &service);
     fail_if(ret != EEXIST);
 
     for (i = 9; i >= 0; i--) {
@@ -223,21 +223,22 @@ START_TEST(test_fo_resolve_service)
     fail_if(ctx == NULL);
 
     /* Add service. */
-    fail_if(fo_new_service(ctx->fo_ctx, "http", &service[0]) != EOK);
+    fail_if(fo_new_service(ctx->fo_ctx, "http", NULL, &service[0]) != EOK);
 
-    fail_if(fo_new_service(ctx->fo_ctx, "ldap", &service[1]) != EOK);
+    fail_if(fo_new_service(ctx->fo_ctx, "ldap", NULL, &service[1]) != EOK);
 
-    fail_if(fo_new_service(ctx->fo_ctx, "ntp", &service[2]) != EOK);
+    fail_if(fo_new_service(ctx->fo_ctx, "ntp", NULL, &service[2]) != EOK);
 
     /* Add servers. */
-    fail_if(fo_add_server(service[0], "localhost", 20, NULL) != EOK);
-    fail_if(fo_add_server(service[0], "127.0.0.1", 80, NULL) != EOK);
+    fail_if(fo_add_server(service[0], "localhost", 20, NULL, true) != EOK);
+    fail_if(fo_add_server(service[0], "127.0.0.1", 80, NULL, false) != EOK);
 
-    fail_if(fo_add_server(service[1], "localhost", 30, NULL) != EOK);
-    fail_if(fo_add_server(service[1], "127.0.0.1", 389, NULL) != EOK);
-    fail_if(fo_add_server(service[1], "127.0.0.1", 389, NULL) != EEXIST);
+    fail_if(fo_add_server(service[1], "localhost", 30, NULL, false) != EOK);
+    fail_if(fo_add_server(service[1], "127.0.0.1", 389, NULL, true) != EOK);
+    fail_if(fo_add_server(service[1], "127.0.0.1", 389, NULL, true) != EEXIST);
+    fail_if(fo_add_server(service[1], "127.0.0.1", 389, NULL, false) != EEXIST);
 
-    fail_if(fo_add_server(service[2], NULL, 123, NULL) != EOK);
+    fail_if(fo_add_server(service[2], NULL, 123, NULL, true) != EOK);
 
     /* Make requests. */
     get_request(ctx, service[0], EOK, 20, PORT_WORKING, -1);
@@ -283,11 +284,10 @@ main(int argc, const char *argv[])
     int failure_count;
     Suite *suite;
     SRunner *sr;
-    int debug = 0;
 
     struct poptOption long_options[] = {
         POPT_AUTOHELP
-        { "debug-level", 'd', POPT_ARG_INT, &debug, 0, "Set debug level", NULL },
+        { "debug-level", 'd', POPT_ARG_INT, &debug_level, 0, "Set debug level", NULL },
         { "use-net-test", 'n', POPT_ARG_NONE, 0, 'n', "Run tests that need an active internet connection", NULL },
         POPT_TABLEEND
     };
