@@ -46,7 +46,9 @@ int main(int argc, const char **argv)
     poptContext pc = NULL;
     struct tools_ctx *tctx = NULL;
     int ret = EXIT_SUCCESS;
+    errno_t sret;
     const char *pc_groupname = NULL;
+    bool in_transaction = false;
 
     debug_prg_name = argv[0];
 
@@ -106,20 +108,32 @@ int main(int argc, const char **argv)
 
     tctx->error = sysdb_transaction_start(tctx->sysdb);
     if (tctx->error != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
         goto done;
     }
+    in_transaction = true;
 
     /* groupadd */
     tctx->error = groupadd(tctx->sysdb, tctx->octx);
     if (tctx->error) {
-        /* cancel transaction */
-        sysdb_transaction_cancel(tctx->sysdb);
         goto done;
     }
 
     tctx->error = sysdb_transaction_commit(tctx->sysdb);
+    if (tctx->error != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
+        goto done;
+    }
+    in_transaction = false;
 
 done:
+    if (in_transaction) {
+        sret = sysdb_transaction_cancel(tctx->sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to cancel transaction\n"));
+        }
+    }
+
     if (tctx->error) {
         ret = tctx->error;
         switch (ret) {
