@@ -31,7 +31,7 @@
 
 #define PADDING_SPACES   4
 #define GROUP_SHOW_ATTRS { SYSDB_MEMBEROF, SYSDB_GIDNUM, \
-                           SYSDB_MEMBER, SYSDB_NAME, \
+                           SYSDB_MEMBER, SYSDB_GHOST, SYSDB_NAME, \
                            NULL }
 #define GROUP_SHOW_MPG_ATTRS { SYSDB_MEMBEROF, SYSDB_UIDNUM, \
                                 SYSDB_NAME, NULL }
@@ -189,8 +189,8 @@ static int parse_members(TALLOC_CTX *mem_ctx,
     }
 
     *user_members = um;
-    *group_members = gm;
-    *num_group_members = gm_index;
+    if (group_members) *group_members = gm;
+    if (num_group_members) *num_group_members = gm_index;
     talloc_zfree(tmp_ctx);
     return EOK;
 
@@ -211,8 +211,10 @@ static int process_group(TALLOC_CTX *mem_ctx,
                          int    *num_group_members)
 {
     struct ldb_message_element *el;
-    int ret;
+    int ret, i, j;
+    int count = 0;
     struct group_info *gi = NULL;
+    const char **user_members;
 
     DEBUG(6, ("Found entry %s\n", ldb_dn_get_linearized(msg->dn)));
 
@@ -245,6 +247,40 @@ static int process_group(TALLOC_CTX *mem_ctx,
         if (ret != EOK) {
             goto done;
         }
+        if (gi->user_members == NULL) {
+            count = 0;
+        } else {
+            for (count = 0; gi->user_members[count]; count++) ;
+        }
+    }
+    el = ldb_msg_find_element(msg, SYSDB_GHOST);
+    if (el) {
+        ret = parse_members(gi, ldb, domain, el,
+                            parent_name,
+                            &user_members,
+                            NULL, NULL);
+        if (ret != EOK) {
+            goto done;
+        }
+
+        if (user_members != NULL) {
+            i = count;
+            for (count = 0; user_members[count]; count++) ;
+            gi->user_members = talloc_realloc(gi, gi->user_members,
+                                              const char *,
+                                              i + count + 1);
+            if (gi->user_members == NULL) {
+                ret = ENOMEM;
+                goto done;
+            }
+            for (j = 0; j < count; j++, i++) {
+                gi->user_members[i] = talloc_steal(gi->user_members,
+                                                   user_members[j]);
+            }
+            gi->user_members[i] = NULL;
+
+            talloc_zfree(user_members);
+        }
     }
 
     /* list memberofs */
@@ -387,7 +423,7 @@ static int group_show_trim_memberof(TALLOC_CTX *mem_ctx,
     int ret;
     int i;
 
-    dn = sysdb_group_dn(sysdb, mem_ctx, domain->name, name);
+    dn = sysdb_group_dn(sysdb, mem_ctx, name);
     if (!dn) {
         return ENOMEM;
     }
@@ -559,26 +595,26 @@ static void print_group_info(struct group_info *g, int level)
     snprintf(fmt, 8, "%%%ds", level*PADDING_SPACES);
     snprintf(padding, 512, fmt, "");
 
-    printf(_("%s%sGroup: %s\n"), padding,
+    printf(_("%1$s%2$sGroup: %3$s\n"), padding,
                                  g->mpg ? _("Magic Private ") : "",
                                  g->name);
-    printf(_("%sGID number: %d\n"), padding, g->gid);
+    printf(_("%1$sGID number: %2$d\n"), padding, g->gid);
 
-    printf(_("%sMember users: "), padding);
+    printf(_("%1$sMember users: "), padding);
     if (g->user_members) {
         for (i=0; g->user_members[i]; ++i) {
             printf("%s%s", i>0 ? "," : "",
                            g->user_members[i]);
         }
     }
-    printf(_("\n%sIs a member of: "), padding);
+    printf(_("\n%1$sIs a member of: "), padding);
     if (g->memberofs) {
         for (i=0; g->memberofs[i]; ++i) {
             printf("%s%s", i>0 ? "," : "",
                            g->memberofs[i]);
         }
     }
-    printf(_("\n%sMember groups: "), padding);
+    printf(_("\n%1$sMember groups: "), padding);
 }
 
 static void print_recursive(struct group_info **group_members, int level)
