@@ -61,7 +61,6 @@ static int mod_groups_member(struct sysdb_ctx *sysdb,
     struct ldb_dn *parent_dn;
     int ret;
     int i;
-    struct sss_domain_info *domain = sysdb_ctx_get_domain(sysdb);
 
     tmpctx = talloc_new(NULL);
     if (!tmpctx) {
@@ -72,7 +71,6 @@ static int mod_groups_member(struct sysdb_ctx *sysdb,
     for (i = 0; grouplist[i]; i++) {
 
         parent_dn = sysdb_group_dn(sysdb, tmpctx,
-                                   domain->name,
                                    grouplist[i]);
         if (!parent_dn) {
             ret = ENOMEM;
@@ -201,8 +199,7 @@ int usermod(TALLOC_CTX *mem_ctx,
     int ret;
 
     if (data->addgroups || data->rmgroups) {
-        member_dn = sysdb_user_dn(sysdb, mem_ctx,
-                                  data->domain->name, data->name);
+        member_dn = sysdb_user_dn(sysdb, mem_ctx, data->name);
         if (!member_dn) {
             return ENOMEM;
         }
@@ -259,8 +256,7 @@ int groupmod(TALLOC_CTX *mem_ctx,
     int ret;
 
     if (data->addgroups || data->rmgroups) {
-        member_dn = sysdb_group_dn(sysdb, mem_ctx,
-                                   data->domain->name, data->name);
+        member_dn = sysdb_group_dn(sysdb, mem_ctx, data->name);
         if (!member_dn) {
             return ENOMEM;
         }
@@ -317,7 +313,7 @@ int userdel_defaults(TALLOC_CTX *mem_ctx,
 
     /* remove homedir on user creation? */
     if (!remove_home) {
-        ret = confdb_get_bool(confdb, mem_ctx,
+        ret = confdb_get_bool(confdb,
                              conf_path, CONFDB_LOCAL_REMOVE_HOMEDIR,
                              DFL_REMOVE_HOMEDIR, &dfl_remove_home);
         if (ret != EOK) {
@@ -408,7 +404,7 @@ int useradd_defaults(TALLOC_CTX *mem_ctx,
 
     /* create homedir on user creation? */
     if (!create_home) {
-        ret = confdb_get_bool(confdb, mem_ctx,
+        ret = confdb_get_bool(confdb,
                              conf_path, CONFDB_LOCAL_CREATE_HOMEDIR,
                              DFL_CREATE_HOMEDIR, &data->create_homedir);
         if (ret != EOK) {
@@ -420,7 +416,7 @@ int useradd_defaults(TALLOC_CTX *mem_ctx,
     DEBUG(7, ("Auto create homedir: %s\n", data->create_homedir?"True":"False"));
 
     /* umask to create homedirs */
-    ret = confdb_get_int(confdb, mem_ctx,
+    ret = confdb_get_int(confdb,
                          conf_path, CONFDB_LOCAL_UMASK,
                          DFL_UMASK, (int *) &data->umask);
     if (ret != EOK) {
@@ -479,8 +475,7 @@ int useradd(TALLOC_CTX *mem_ctx,
     if (data->addgroups) {
         struct ldb_dn *member_dn;
 
-        member_dn = sysdb_user_dn(sysdb, mem_ctx,
-                                  data->domain->name, data->name);
+        member_dn = sysdb_user_dn(sysdb, mem_ctx, data->name);
         if (!member_dn) {
             ret = ENOMEM;
             goto done;
@@ -509,8 +504,7 @@ int userdel(TALLOC_CTX *mem_ctx,
     struct ldb_dn *user_dn;
     int ret;
 
-    user_dn = sysdb_user_dn(sysdb, mem_ctx,
-                            data->domain->name, data->name);
+    user_dn = sysdb_user_dn(sysdb, mem_ctx, data->name);
     if (!user_dn) {
         DEBUG(1, ("Could not construct a user DN\n"));
         return ENOMEM;
@@ -552,8 +546,7 @@ int groupdel(TALLOC_CTX *mem_ctx,
     struct ldb_dn *group_dn;
     int ret;
 
-    group_dn = sysdb_group_dn(sysdb, mem_ctx,
-                              data->domain->name, data->name);
+    group_dn = sysdb_group_dn(sysdb, mem_ctx, data->name);
     if (group_dn == NULL) {
         DEBUG(1, ("Could not construct a group DN\n"));
         return ENOMEM;
