@@ -53,14 +53,8 @@ static int setup_db(struct tools_ctx *ctx)
         return ret;
     }
 
-    ret = confdb_get_domain(ctx->confdb, "local", &ctx->local);
-    if (ret != EOK) {
-        DEBUG(1, ("Could not get 'local' domain: [%d] [%s]\n", ret, strerror(ret)));
-        return ret;
-    }
-
-    /* open 'local' sysdb at default path */
-    ret = sysdb_domain_init(ctx, ctx->local, DB_PATH, &ctx->sysdb);
+    ret = sysdb_init_domain_and_sysdb(ctx, ctx->confdb, "local", DB_PATH,
+                                      &ctx->local, &ctx->sysdb);
     if (ret != EOK) {
         DEBUG(1, ("Could not initialize connection to the sysdb\n"));
         return ret;
@@ -274,7 +268,7 @@ int init_sss_tools(struct tools_ctx **_tctx)
         goto fini;
     }
 
-    ret = sss_names_init(tctx, tctx->confdb, &tctx->snctx);
+    ret = sss_names_init(tctx, tctx->confdb, tctx->local->name, &tctx->snctx);
     if (ret != EOK) {
         DEBUG(1, ("Could not set up parsing\n"));
         goto fini;
@@ -472,7 +466,33 @@ int create_homedir(const char *skeldir,
 
     selinux_file_context(homedir);
 
-    ret = copy_tree(skeldir, homedir, 0777 & ~default_umask, uid, gid);
+    ret = mkdir(homedir, 0);
+    if (ret != 0) {
+        ret = errno;
+        DEBUG(1, ("Cannot create user's home directory: [%d][%s].\n",
+                  ret, strerror(ret)));
+        goto done;
+    }
+
+    ret = chown(homedir, uid, gid);
+    if (ret != 0) {
+        ret = errno;
+        DEBUG(1, ("Cannot chown user's home directory: [%d][%s].\n",
+                  ret, strerror(ret)));
+        goto done;
+    }
+
+    ret = chmod(homedir, 0777 & ~default_umask);
+    if (ret != 0) {
+        ret = errno;
+        DEBUG(1, ("Cannot chmod user's home directory: [%d][%s].\n",
+                  ret, strerror(ret)));
+        goto done;
+    }
+
+    reset_selinux_file_context();
+
+    ret = copy_tree(skeldir, homedir, uid, gid);
     if (ret != EOK) {
         DEBUG(1, ("Cannot populate user's home directory: [%d][%s].\n",
                   ret, strerror(ret)));
@@ -558,3 +578,96 @@ done:
     talloc_free(conf_path);
     return ret;
 }
+
+static pid_t parse_pid(const char *strpid)
+{
+    long value;
+    char *endptr;
+
+    errno = 0;
+    value = strtol(strpid, &endptr, 10);
+    if ((errno != 0) || (endptr == strpid)
+        || ((*endptr != '\0') && (*endptr != '\n'))) {
+        return 0;
+    }
+
+    return value;
+}
+
+static errno_t get_sssd_pid(pid_t *out_pid)
+{
+    int ret;
+    size_t fsize;
+    FILE *pid_file = NULL;
+    char pid_str[MAX_PID_LENGTH] = {'\0'};
+
+    *out_pid = 0;
+
+    errno = 0;
+    pid_file = fopen(SSSD_PIDFILE, "r");
+    if (pid_file == NULL) {
+        ret = errno;
+        DEBUG(SSSDBG_MINOR_FAILURE, ("Unable to open pid file \"%s\": %s\n",
+              SSSD_PIDFILE, strerror(ret)));
+        goto done;
+    }
+
+    fsize = fread(pid_str, sizeof(char), MAX_PID_LENGTH * sizeof(char),
+                  pid_file);
+    if (!feof(pid_file)) {
+        /* eof not reached */
+        ret = ferror(pid_file);
+        if (ret != 0) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Unable to read from file \"%s\": %s\n",
+                  SSSD_PIDFILE, strerror(ret)));
+        } else {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("File \"%s\" contains invalid pid.\n",
+                  SSSD_PIDFILE));
+        }
+        goto done;
+    }
+    if (fsize == 0) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("File \"%s\" contains no pid.\n",
+              SSSD_PIDFILE));
+        ret = EINVAL;
+        goto done;
+    }
+
+    pid_str[MAX_PID_LENGTH-1] = '\0';
+    *out_pid = parse_pid(pid_str);
+    if (*out_pid == 0) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("File \"%s\" contains invalid pid.\n", SSSD_PIDFILE));
+        ret = EINVAL;
+        goto done;
+    }
+
+    ret = EOK;
+
+done:
+    if (pid_file != NULL) {
+        fclose(pid_file);
+    }
+    return ret;
+}
+
+errno_t signal_sssd(int signum)
+{
+    int ret;
+    pid_t pid;
+
+    ret = get_sssd_pid(&pid);
+    if (ret != EOK) {
+        return ret;
+    }
+
+    if (kill(pid, signum) != 0) {
+        ret = errno;
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("Could not send signal %d to process %d: %s\n",
+              signum, pid, strerror(errno)));
+        return ret;
+    }
+
+    return EOK;
+}
