@@ -27,6 +27,9 @@
 
 #include "util/util.h"
 
+#define SSSD_PIDFILE ""PID_PATH"/sssd.pid"
+#define MAX_PID_LENGTH 10
+
 #define BAD_POPT_PARAMS(pc, msg, val, label) do { \
         usage(pc, msg);                           \
         val = EXIT_FAILURE;                       \
@@ -37,7 +40,7 @@
     val = getuid(); \
     if (val != 0) { \
         DEBUG(1, ("Running under %d, must be root\n", val)); \
-        ERROR("%s must be run as root\n", prg_name); \
+        ERROR("%1$s must be run as root\n", prg_name); \
         val = EXIT_FAILURE; \
         goto fini; \
     } \
@@ -99,11 +102,14 @@ int remove_homedir(TALLOC_CTX *mem_ctx,
 
 int run_userdel_cmd(struct tools_ctx *tctx);
 
+errno_t signal_sssd(int signum);
+
 /* from files.c */
 int remove_tree(const char *root);
 
-int copy_tree(const char *src_root, const char *dst_root,
-              mode_t mode_root, uid_t uid, gid_t gid);
+int copy_tree(const char *src_root,
+              const char *dst_root,
+              uid_t uid, gid_t gid);
 
 /* from nscd.c */
 enum nscd_db {
