@@ -10,13 +10,14 @@
 #include <stdint.h>
 #include <endian.h>
 #include <string.h>
+#include <byteswap.h>
 
 /* support RHEL5 lack of definitions */
 #ifndef le32toh
 # if __BYTE_ORDER == __LITTLE_ENDIAN
 #  define le32toh(x) (x)
 # else
-#  define le32toh(x) __bswap_32 (x)
+#  define le32toh(x) bswap_32 (x)
 # endif
 #endif
 
