@@ -117,51 +117,36 @@ int pidfile(const char *path, const char *name)
     fd = open(file, O_RDONLY, 0644);
     err = errno;
     if (fd != -1) {
-        len = 0;
-        while ((ret = read(fd, pid_str + len, pidlen - len)) != 0) {
-            if (ret == -1) {
-                if (errno == EINTR || errno == EAGAIN) {
-                    continue;
-                }
-                DEBUG(1, ("read failed [%d][%s].\n", errno, strerror(errno)));
-                break;
-            } else if (ret > 0) {
-                len += ret;
-                if (len > pidlen) {
-                    DEBUG(1, ("read too much, this should never happen.\n"));
-                    close(fd);
-                    talloc_free(file);
-                    return EINVAL;
-                }
-                continue;
-            } else {
-                DEBUG(1, ("unexpected return code of read [%d].\n", ret));
-                break;
-            }
+        errno = 0;
+        len = sss_atomic_read_s(fd, pid_str, pidlen);
+        ret = errno;
+        if (len == -1) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  ("read failed [%d][%s].\n", ret, strerror(ret)));
+            close(fd);
+            talloc_free(file);
+            return EINVAL;
         }
 
         /* Ensure NULL-termination */
         pid_str[len] = '\0';
 
-        if (ret == 0) {
-            /* let's check the pid */
-
-            pid = (pid_t)atoi(pid_str);
-            if (pid != 0) {
-                errno = 0;
-                ret = kill(pid, 0);
-                /* succeeded in signaling the process -> another sssd process */
-                if (ret == 0) {
-                    close(fd);
-                    talloc_free(file);
-                    return EEXIST;
-                }
-                if (ret != 0 && errno != ESRCH) {
-                    err = errno;
-                    close(fd);
-                    talloc_free(file);
-                    return err;
-                }
+        /* let's check the pid */
+        pid = (pid_t)atoi(pid_str);
+        if (pid != 0) {
+            errno = 0;
+            ret = kill(pid, 0);
+            /* succeeded in signaling the process -> another sssd process */
+            if (ret == 0) {
+                close(fd);
+                talloc_free(file);
+                return EEXIST;
+            }
+            if (ret != 0 && errno != ESRCH) {
+                err = errno;
+                close(fd);
+                talloc_free(file);
+                return err;
             }
         }
 
@@ -188,25 +173,20 @@ int pidfile(const char *path, const char *name)
     snprintf(pid_str, sizeof(pid_str) -1, "%u\n", (unsigned int) getpid());
     size = strlen(pid_str);
 
-    written = 0;
-    while (written < size) {
-        ret = write(fd, pid_str+written, size-written);
-        if (ret == -1) {
-            err = errno;
-            if (err == EINTR || err == EAGAIN) {
-                continue;
-            }
-            DEBUG(1, ("write failed [%d][%s]\n", err, strerror(err)));
-            break;
-        }
-        else {
-            written += ret;
-        }
+    errno = 0;
+    written = sss_atomic_write_s(fd, pid_str, size);
+    if (written == -1) {
+        err = errno;
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("write failed [%d][%s]\n", err, strerror(err)));
+        return err;
     }
 
     if (written != size) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("Wrote %d bytes expected %d\n", written, size));
         close(fd);
-        return err;
+        return EIO;
     }
 
     close(fd);
@@ -317,8 +297,10 @@ static void server_stdin_handler(struct tevent_context *event_ctx,
 {
 	const char *binary_name = (const char *)private;
 	uint8_t c;
-	if (read(0, &c, 1) == 0) {
-		DEBUG(0,("%s: EOF on stdin - terminating\n", binary_name));
+
+        errno = 0;
+        if (sss_atomic_read_s(0, &c, 1) == 0) {
+		DEBUG(SSSDBG_CRIT_FAILURE,("%s: EOF on stdin - terminating\n", binary_name));
 #if HAVE_GETPGRP
 		if (getpgrp() == getpid()) {
 			kill(-getpgrp(), SIGTERM);
@@ -409,8 +391,8 @@ int server_setup(const char *name, int flags,
     if (flags & FLAGS_PID_FILE) {
         ret = pidfile(PID_PATH, name);
         if (ret != EOK) {
-            DEBUG(0, ("Error creating pidfile! (%d [%s])\n",
-                      ret, strerror(ret)));
+            DEBUG(SSSDBG_FATAL_FAILURE, ("Error creating pidfile: %s/%s! (%d [%s])\n",
+                                         PID_PATH, name, ret, strerror(ret)));
             return ret;
         }
     }
@@ -464,7 +446,7 @@ int server_setup(const char *name, int flags,
 
     if (debug_level == SSSDBG_UNRESOLVED) {
         /* set debug level if any in conf_entry */
-        ret = confdb_get_int(ctx->confdb_ctx, ctx, conf_entry,
+        ret = confdb_get_int(ctx->confdb_ctx, conf_entry,
                              CONFDB_SERVICE_DEBUG_LEVEL,
                              SSSDBG_DEFAULT,
                              &debug_level);
@@ -479,7 +461,7 @@ int server_setup(const char *name, int flags,
 
     /* same for debug timestamps */
     if (debug_timestamps == SSSDBG_TIMESTAMP_UNRESOLVED) {
-        ret = confdb_get_bool(ctx->confdb_ctx, ctx, conf_entry,
+        ret = confdb_get_bool(ctx->confdb_ctx, conf_entry,
                               CONFDB_SERVICE_DEBUG_TIMESTAMPS,
                               SSSDBG_TIMESTAMP_DEFAULT,
                               &dt);
@@ -494,7 +476,7 @@ int server_setup(const char *name, int flags,
 
     /* same for debug microseconds */
     if (debug_microseconds == SSSDBG_MICROSECONDS_UNRESOLVED) {
-        ret = confdb_get_bool(ctx->confdb_ctx, ctx, conf_entry,
+        ret = confdb_get_bool(ctx->confdb_ctx, conf_entry,
                               CONFDB_SERVICE_DEBUG_MICROSECONDS,
                               SSSDBG_MICROSECONDS_DEFAULT,
                               &dm);
@@ -509,7 +491,7 @@ int server_setup(const char *name, int flags,
 
     /* same for debug to file */
     dl = (debug_to_file != 0);
-    ret = confdb_get_bool(ctx->confdb_ctx, ctx, conf_entry,
+    ret = confdb_get_bool(ctx->confdb_ctx, conf_entry,
                           CONFDB_SERVICE_DEBUG_TO_FILES,
                           dl, &dl);
     if (ret != EOK) {
@@ -544,7 +526,7 @@ int server_setup(const char *name, int flags,
 
     sss_log(SSS_LOG_INFO, "Starting up");
 
-    DEBUG(3, ("CONFDB: %s\n", conf_db));
+    DEBUG(SSSDBG_TRACE_FUNC, ("CONFDB: %s\n", conf_db));
 
     if (flags & FLAGS_INTERACTIVE) {
         /* terminate when stdin goes away */
