@@ -611,40 +611,26 @@ void to_sized_string(struct sized_string *out, const char *in)
     }
 }
 
-/* based on code from libssh <http://www.libssh.org> */
-ssize_t sss_atomic_io(int fd, void *buf, size_t n, bool do_read)
+/* This function only removes first and last
+ * character if the first character was '['.
+ *
+ * NOTE: This means, that ipv6addr must NOT be followed
+ * by port number.
+ */
+errno_t
+remove_ipv6_brackets(char *ipv6addr)
 {
-    char *b = buf;
-    size_t pos = 0;
-    ssize_t res;
-    struct pollfd pfd;
+    size_t len;
 
-    pfd.fd = fd;
-    pfd.events = do_read ? POLLIN : POLLOUT;
+    if (ipv6addr && ipv6addr[0] == '[') {
+        len = strlen(ipv6addr);
+        if (len < 3) {
+            return EINVAL;
+        }
 
-    while (n > pos) {
-        if (do_read) {
-            res = read(fd, b + pos, n - pos);
-        } else {
-            res = write(fd, b + pos, n - pos);
-        }
-        switch (res) {
-        case -1:
-            if (errno == EINTR) {
-                continue;
-            }
-            if (errno == EAGAIN || errno == EWOULDBLOCK) {
-                (void) poll(&pfd, 1, -1);
-                continue;
-            }
-            return -1;
-        case 0:
-            errno = EPIPE;
-            return pos;
-        default:
-            pos += (size_t) res;
-        }
+        memmove(ipv6addr, &ipv6addr[1], len - 2);
+        ipv6addr[len -2] = '\0';
     }
 
-    return pos;
+    return EOK;
 }
